"use strict";

//========================================================================================================================================

class NLEDcolorPicker {
    constructor(depthArg) {
        this.isOpen = false;
        this.mode = 0; //0 = standard fill,  
        this.type = "RGB"; //default
        this.container = document.getElementById('colorpicker-container');
        if (depthArg == '16') this.depth = 0xFFFF; //16-bit 65535
        else this.depth = 0xFF; //8-bit 255

        this.prevType = "RGB";
        this.width = 0;
        this.height = 0;
        this.offsetX = 0;
        this.offsetY = 0;
        this.targetX = 0;
        this.targetY = 0;
        this.red = 0;
        this.green = 0;
        this.blue = 0;
        this.white = 0;

        this.frame = 0;
        this.pix = 0;

        this.palette = ['#ff0000', '#ffff00', '#00ff00', '#00ffff', '#0000ff', '#ff00ff'];

        //----------------------------------------- Build elements ---------------------------------------------------------

        let theCont = document.getElementById("colorpicker-palette");

        for (let i = 0; i < 6; i++) {
            let theTag = document.createElement('div');
            theTag.id = 'colorpicker-palette-' + i;
            theTag.style.backgroundColor = this.palette[i];

            let self = this;

            theTag.addEventListener('click', function () { self.updateHex(self.palette[i]); });
            // theTag.onclick = function () {
            //     //pass the palette color
            // }
            theCont.appendChild(theTag) //write to the DIV tag, will close </DIV> with whatever has been put into it
        }
        this.updateEntries(); //set the initial slider/box values to the default

        //----------------------------------------- Attach events ---------------------------------------------------------

        let self = this; //store reference to color picker object before nesting(?)

        document.getElementById('colorpicker-area').addEventListener("mousedown", function (e) {
            self.drag(e);
            this.addEventListener("mousemove", self.drag);
        });

        document.getElementById('colorpicker-area').addEventListener("mouseup", function (e) {
            this.removeEventListener("mousemove", self.drag);
        });

        document.getElementById('colorpicker-area').addEventListener("mouseleave", function (e) {
            document.addEventListener("mouseup", colorPickerDraggedOff); //bit clunky but works the way I want it to
        });

        function colorPickerDraggedOff() {
            document.getElementById('colorpicker-area').removeEventListener("mousemove", self.drag); //user released button off picker, end dragging
            document.removeEventListener("mousemove", colorPickerDraggedOff); //remove itself....
        }

        this.loadPickerImage()
    }
    //---------------------------------------------------------------------------------
    loadPickerImage() {
        var colorPickerimage = new Image();
        colorPickerimage.addEventListener('load', function () {
            let canvas1 = document.getElementById('colorpicker-area-canvas');
            canvas1.width = this.width;
            canvas1.height = this.height;
            let ctx1 = canvas1.getContext('2d');
            ctx1.drawImage(colorPickerimage, 0, 0);
        });
        switch (this.type) {
            case "Mono":
                colorPickerimage.src = path.join(process.cwd(), 'assets/images/colorpicker-mono.png');
                break;
            case "RGB":
            case "RGBW":
                colorPickerimage.src = path.join(process.cwd(), 'assets/images/colorpicker-rgbw.png');
                break;
        }
        this.prevType = this.type;
    }
    //---------------------------------------------------------------------------------
    setColors(color, wht = this.white) {
        switch (this.type) {
            case "Mono":
                this.red = 0;
                this.green = 0;
                this.blue = 0;
                this.white = parseInt(color[0]); //choose any RGB since it passed white
                break;
            case "RGB":
                this.red = parseInt(color[0]);
                this.green = parseInt(color[1]);
                this.blue = parseInt(color[2]);
                this.white = 0; //should all be the same
            case "RGBW":
                this.red = parseInt(color[0]);
                this.green = parseInt(color[1]);
                this.blue = parseInt(color[2]);
                this.white = parseInt(wht);
                break;
        }
    }
    //---------------------------------------------------------------------------------
    updateEntries() {
        document.getElementById("colorpicker-redbox").value = this.depthConvertUp(this.red);
        document.getElementById("colorpicker-greenbox").value = this.depthConvertUp(this.green);
        document.getElementById("colorpicker-bluebox").value = this.depthConvertUp(this.blue);
        document.getElementById("colorpicker-whitebox").value = this.depthConvertUp(this.white);

        document.getElementById("colorpicker-redslider").value = this.depthConvertUp(this.red);
        document.getElementById("colorpicker-greenslider").value = this.depthConvertUp(this.green);
        document.getElementById("colorpicker-blueslider").value = this.depthConvertUp(this.blue);
        document.getElementById("colorpicker-whiteslider").value = this.depthConvertUp(this.white);

        document.getElementById("colorpicker-hexvalue").value = rgbToHex(this.red, this.green, this.blue);
        document.getElementById("colorpicker-rgbprev").style.backgroundColor = rgbToHex(this.red, this.green, this.blue);
        document.getElementById("colorpicker-whtprev").style.backgroundColor = rgbToHex(this.white, this.white, this.white);
    }
    //---------------------------------------------------------------------------------
    drag(e) {
        colorPicker.click(e); //this. doesn't work, nor does this. if the click() is ran directly from the drag events - find out why
    }
    //---------------------------------------------------------------------------------
    click(e) {
        let x = e.clientX;     // Get the horizontal coordinate
        let y = e.clientY;     // Get the vertical coordinate

        //-------------------------- If Mono color type, prevent the indicator from selecting the RGB area ----------------------------------

        //console.log("click() " + x + "  " + yF + "   Adjusted, X: " + (x - this.offsetX) + "   Y: " + (y - this.offsetY))

        this.targetX = (x - this.offsetX);
        this.targetY = (y - this.offsetY);

        //DOMRect {x: -1, y: 178, width: 42, height: 42, top: 178, …}
        const coords = document.getElementById("colorpicker-area").getBoundingClientRect(); //heard this is really slow......

        let theDiv = document.getElementById("colorpicker-indicator");

        theDiv.style.left = (x - coords.x) - (theDiv.offsetWidth / 2) + 'px';
        theDiv.style.top = (y - coords.y) - (theDiv.offsetHeight / 2) + 'px';

        let canvas = document.getElementById('colorpicker-area-canvas');
        let pickedRGBAColor = canvas.getContext('2d').getImageData(this.targetX, this.targetY, 1, 1).data;

        //console.log(pickedRGBAColor); //DEBUG

        //--------------------------- update indicator dot - probably should use the object color first, but whatever ------------------------

        document.getElementById("colorpicker-indicator").style.backgroundColor = rgbToHex(pickedRGBAColor[0], pickedRGBAColor[1], pickedRGBAColor[2]);

        this.setColors(pickedRGBAColor);  //update color pick object
        this.updateEntries(); //update text boxes and sliders
    }
    //---------------------------------------------------------------------------------
    open(x, y, type, frame, pix, mode = 0) {
        console.log("colorPicker open(" + x + "," + y + "," + type + "," + frame + "," + pix + ")");

        //------------------------------------- Toggle Fog -----------------------------------------

        document.getElementById('toplevel-fog').classList.replace('visibility-hidden', 'visibility-visible');
        this.container.classList.replace('visibility-hidden', 'visibility-visible');

        //----------------- If not already set, calculate width and height. Setting w&h prevents rendering off screen --------------------------

        if (this.width == 0 || this.height == 0) {
            const coords = this.container.getBoundingClientRect(); //heard this is slow......
            this.width = coords.width;
            this.height = coords.height;
        }

        //----------------- setup colorpicker object states and coordinates --------------------------

        this.isOpen = true;
        this.offsetX = x; //save the X, Y coords for later usage
        this.offsetY = y;
        this.frame = frame;
        this.pix = pix;
        this.type = type;
        this.mode = mode; //mode of picker - channel/live control/etc

        if (this.prevType != this.type) this.loadPickerImage(); //if opening with a different color type, update canvas image

        handleColorPickerSetInitial(mode, type, frame, pix); //user defined function

        //----------------------- Handle disabled/enabled states for entries --------------------------

        //Make all color entry elements enabled
        var elems = document.getElementById("colorpicker-interface").querySelectorAll('input');
        elems.forEach(function (element) {
            element.removeAttribute('disabled');
        });

        switch (this.type) {
            case "Mono":
                //only use the white slider and box.
                elems.forEach(function (element) {
                    element.setAttribute('disabled', true);
                    element.value = 0;
                });
                this.red = this.green = this.blue = 0;
                document.getElementById("colorpicker-whiteslider").removeAttribute('disabled');
                document.getElementById("colorpicker-whitebox").removeAttribute('disabled');
                break;
            case "RGB":
                document.getElementById("colorpicker-whiteslider").setAttribute('disabled', true);
                document.getElementById("colorpicker-whitebox").setAttribute('disabled', true);
                this.white = 0;
            case "RGBW":
                //no changes, all entry elements enabled
                break;
        }

        this.updateEntries(); //update text boxes and sliders with initial values

        //------------------------ Update Palette blocks ------------------------------------------

        let thePalettes = document.querySelectorAll('[id^=colorpicker-palette-]');
        let self = this; //save object reference for nested foreach

        thePalettes.forEach(function (elem, index) {
            elem.style.backgroundColor = self.palette[index];
        });

        //------------------------ Show function message ------------------------------------------

        handleColorPickerMessages(document.getElementById("colorpicker-message")); //user defined function

        //--------------- Keep the color picker container fully on the screen - update the X,Y variables if required -------------------------

        if (this.offsetX + this.width > window.innerWidth) this.offsetX = window.innerWidth - this.width;
        if (this.offsetY + this.height > window.innerHeight) this.offsetY = window.innerHeight - this.height;

        //----------------------------------- now update container position -------------------------------------------

        this.container.style.left = this.offsetX + 'px';
        this.container.style.top = this.offsetY + 'px';
    }
    //---------------------------------------------------------------------------------
    apply() {
        //------------------------- pass data to data handlers - essentially the callback ---------------------------------
        handleColorPickerSelection(this.frame, this.pix, this.type, this.mode, [this.red, this.green, this.blue, this.white]); //user defined function
        //------------------------------ handle palette ----------------------------------------------------------
        let chosen = rgbToHex(this.red, this.green, this.blue);

        if (this.palette.indexOf(chosen) > -1) {
            this.palette.splice(this.palette.indexOf(chosen), 1)
            this.palette.push(chosen);
        }
        else {
            this.palette.shift();
            this.palette.push(chosen);
        }

        //------------------------------ close ----------------------------------------------------------

        this.close(); //update UX close picker
    }
    //---------------------------------------------------------------------------------
    close() {
        document.getElementById('toplevel-fog').classList.replace('visibility-visible', 'visibility-hidden');
        this.container.classList.replace('visibility-visible', 'visibility-hidden');
        this.isOpen = false;
    }
    //---------------------------------------------------------------------------------
    depthConvertDown(arg) {
        if (this.depth == 0xFFFF) return (arg >> 8)
        else return arg;
    }
    depthConvertUp(val) {
        if (this.depth == 0xFFFF) return (val * 256) + val;
        else return val;
    }
    //---------------------------------------------------------------------------------
    entryUpdate(elem) {
        //make value a number, and ensure it is within range
        let val = parseInt(elem.value);
        if (val > this.depth) val = this.depth;
        else if (val < this.minVal) val = this.minVal;

        //simple but effective
        if (elem.id.includes('red')) this.red = this.depthConvertDown(val);
        else if (elem.id.includes('green')) this.green = this.depthConvertDown(val);
        else if (elem.id.includes('blue')) this.blue = this.depthConvertDown(val);
        else if (elem.id.includes('white')) this.white = this.depthConvertDown(val);

        this.updateEntries(); //update the UX element entries



    }
    updateDepth(depthArg) {
        this.depth = depthArg;
        let elems = this.container.querySelectorAll("input.ux-sliders, input.ux-textinput")
        // console.log(elems)

        for (const item of elems) {
            item.max = this.depth;

        }
    }
    //---------------------------------------------------------------------------------
    updateHex(val) {
        //function is double duty for color HEX entry and pallette selection
        //console.log("colorPicker.updateHex() ", val);

        //check if formated with # and 6 digits
        if (val.charAt(0) == '#' && val.length == 7) {
            let r = hexToDec(val.charAt(1) + val.charAt(2));
            let g = hexToDec(val.charAt(3) + val.charAt(4));
            let b = hexToDec(val.charAt(5) + val.charAt(6));
            this.red = r;
            this.green = g;
            this.blue = b;
            this.updateEntries(); //update the UX element entries
        }
        else {
            console.log("Wrong HEX value Format");
        }
    }
} //end colorPicker class

//========================================================================================================================================
