'use strict';

//========================== NORMAL STARTUP ACTIONS =================================

//===================================================================================================
const colorPicker = new NLEDcolorPicker(8);

function initApp() {



    fetch('./package.json')
        .then(response => response.json())
        .then(data => {
            stateSoftware.version.major = parseInt(data.version.split('.')[0]); //loads version from package.json into stateSoftware
            stateSoftware.version.minor = parseInt(data.version.split('.')[1]);
            stateSoftware.version.patch = parseInt(data.version.split('.')[2]);
            const cVerStr = `${stateSoftware.version.major}.${stateSoftware.version.minor}.${stateSoftware.version.patch}`;
            document.getElementById("ws-software-softver").innerText = cVerStr;
            document.getElementById("ws-software-date").innerText = data.date;

            stateSoftware.appTitle += cVerStr + ': ';
            console.log("Aurora Version: ", cVerStr);
            uxTargetSaveFileUpdate();
        });

    fileLoadSettingsFile();

    //===================================================================================================

    document.getElementById("ws-connection-devinfo-name").innerText = stateDevice.soft.name;
    document.getElementById("ws-connection-devinfo-pixels").innerText =
        `${stateSoftware.maxPixels} Mono, ${stateSoftware.maxPixels} RGB, ${stateSoftware.maxPixels} RGBW`;
    document.getElementById("ws-connection-devinfo-frames").innerText = `256 / ${stateSoftware.maxFrames}`;

    //===================================================================================================

    stateUX.serialPortScanTimer = setInterval(function () {
        serialPort.list().then(ports => {
            //this returns a promise eventually
            //if the number of ports available is not the same as the number in the dropdown, rerun and populated the drop down
            if (document.getElementById('ws-hardware-port').length != ports.length) {
                uxTabHardwarePopulatePorts(ports);
                //DOM is not updated yet, so trying anything with the ports drop down won't work. Wait a bit.....
                convertSelectInputOption("ws-hardware-port", stateCommunication.prevSerialPort); //fixed it
                convertSelectInputOption("ws-hardware-baud", stateCommunication.baud); //selects previous baud
                //if ports change, it will attempt the last port# that worked.
                if (stateSoftware.autoConnectEnabled && !stateCommunication.connected) setTimeout(commSerialPrevConnect, 100); //DOM needs to update before this works.. so just wait
            }
        });
    }, 1000);

    //===================================================================================================


    fileConfirmMessagesJSON();

    uxToolsLayoutUpdate();  //load default layout stuff
    uxAdjustZoom(); //zoom persists once changed, so make sure it is defaulted at startup

    uxInitialize(); //call right away. Guess from here.

    if (!stateSoftware.firstRun) {
        fetch(path.join(process.cwd(), 'readme.txt')).then((resp) => resp.text()).then(function (textLabelsData) {
            document.getElementById('ovmenu-7-workspace').innerHTML = textLabelsData;
            overlayMenuEvent(event, 'open', 'menu-7');
            stateSoftware.firstRun = true;
        });
    }


    if (stateSoftware.autoSequencesLoad) {
        try {
        systemLoadSequenceFileInit(stateSoftware.previousSaveFile, false);
        uxTargetLoadedFile(stateSoftware.previousSaveFile);
        }
        catch(e) {

            console.log("ERROR - Can not load previous file");
        }
    }

    //======================== Do debugging stuff, emulate user interactions, etc =====================

    //START HERE HAVE LOCALSTORAGE SAVE THE PREVIOUS SAVE FILE NAME AND LOAD IT AT STARTUP
    //add a key to the save file to force device simulation
    //breakout playback preview into a function that can be iterated to generate the rendered sequence data

    // let storedConfigID = localStorage.getItem("nled-aurora");
    // if (storedConfigID == undefined || storedConfigID == null) {
    //     storedConfigID = 0;//default
    //     console.log("Stored Config not found, setting default");
    // }
    // console.log(storedConfigID);

    // setTimeout(() => {
    //     localStorage.setItem("nled-aurora", "TESTTEST");
    //     console.log("WROTE LOCALSTORAGE");
    // }, 1000);




    //systemLoadSequenceFileInit('C:/languages/electron/nled-aurora/saves/purple-inf-UPDATED.auroraseq', false);
    //systemLoadSequenceFileInit('C:/languages/electron/nled-aurora/saves/devices/pixel-controller-electron-256px-strand.auroraseq', false);
    // systemLoadSequenceFileInit('C:/languages/electron/nled-aurora/saves/devices/pixelcontroller-ion-quark-proton-photon-170px-strand.auroraseq', false);
    // systemLoadSequenceFileInit('C:/languages/electron/nled-aurora/saves/aurora-v1-v2-save-files/16bitrgbw-lamp-v2.txt', false);
    //systemLoadSequenceFileInit('C:/languages/electron/nled-aurora/saves/hexagon-fixture-new.txt', false);
    //systemLoadSequenceFileInit('C:/languages/electron/nled-aurora/saves/hexagon-fixture-v2.txt', false);
    // systemLoadSequenceFileInit('C:/languages/electron/nled-aurora/saves/quadcc-test.auroraseq', false);
    // systemLoadSequenceFileInit('C:/languages/electron/nled-aurora/saves/devices/aurora-demonstration-file.auroraseq', false);
    // setTimeout(function() {
    // systemLoadSequenceFileInit('C:/languages/electron/nled-aurora/saves/test.auroraseq', false);
    // systemLoadSequenceFileInit('C:/languages/electron/nled-aurora/saves/temp16bit.auroraseq', false);
    // systemLoadSequenceFileInit('C:/languages/electron/nled-aurora/saves/temp.auroraseq', false);
    //systemLoadSequenceFileInit('C:/Users/Owner/Desktop/test-photon-1024.auroraseq', false);
    // systemLoadSequenceFileInit('C:/NLED/Projects/NLED Aurora Control/user sequences/NLED Sequence/NLED Aurora File/Test Files.auroraseq', false);
    // },1000);
    //setTimeout(sequenceSelect, 500, 2); //have to set timeout for a small delay or won't work

    //overlayMenuEvent(event, 'open', 'menu-2')
    //uxTriggerElementClick('navtab-timeline')
    // handleToolIconClick(document.getElementById("menubar-icon-ledicon"));


    // window.electronAPI.netConnect(23,'127.0.0.1', function() {
    // setTimeout(function () {
    //     networkOpenTCP();
    // }, 2000);
}
//================================================================================================

/*
function debugToolTipScanner() {
    let toolElements = document.querySelectorAll('button, select, input, label, textarea')
    let filteredNoTip = [];//toolElements.map(e => e.getAttribute('data-tiptext'))
    //.querySelectorAll('[data-tiptext]')
    let filteredNoIDNoTip = [];
    let filteredHasTip = [];

    toolElements.forEach(function (v, i) {
        let att = v.getAttribute('data-tiptext');
        if (att == undefined) {
            if (v.id != undefined && v.id != "") filteredNoTip.push(v.id);
            else filteredNoIDNoTip.push(v);
        }
        else filteredHasTip.push(v.id);
    });

    //console.log(toolElements[0])
    console.log(filteredNoTip)
    console.log(filteredNoIDNoTip); //doesn't matter only labels
    //console.log(filteredHasTip)

    let formatted = [];
    filteredNoTip.forEach(function (v, i) {

        let stuff = `"${v}":{"en": "tip"}`
        formatted.push(stuff);
    })

    fs.writeFile(__dirname+ path.sep + 'dump.json', formatted, function (err) {
        if (err) {
            console.log(err);
        }
    });
}
*/



function strRed(...str) {
    if (typeof str !== 'string') str = JSON.stringify(str);
    return "\x1b[31m" + str + "\x1b[0m";
}


function developmentGenerateAssembly(buffer) {
    console.log("developmentGenerateAssembly()", buffer.length)

    console.log(buffer)
    console.log("===================================")

    let assemblyStr = '#include <xc.h>\n\nasm(".section *, code, address(0x0000), keep");\n';

    const assemblyLineStart = '\nasm(".pword '
    const assemblyLineEnd = '");'
    let z = 0;
    let q = 0;
    let str = ''
    str += assemblyLineStart;

    const toHex = function (val) { if (val == undefined) return 0x00; else return val.toString(16).padStart(2, '0').toUpperCase(); }
    let i = 0;
    for (i = 0; i < buffer.length; i += 3) {
        console.log(buffer[i + 2], buffer[i + 1], buffer[i])
        str += '0x' + toHex(buffer[i + 2]) + toHex(buffer[i + 1]) + toHex(buffer[i]);

        q++;
        if (q == 64) {
            q = 0;
            str += assemblyLineEnd;
            assemblyStr += str;
            str = assemblyLineStart;
        }
        else str += ', ';

    }

    if (q > 0) {
        str += assemblyLineEnd;
        assemblyStr += str;
    }

    console.log(assemblyStr)
}

