"use strict";


//========================================================================================================================================

function systemStartLogging() {
    var today = new Date();
    let logTime = today.getHours() + "-" + today.getMinutes() + "-" + today.getSeconds() + "-" + today.getDate() + "-" + (today.getMonth() + 1) + "-" + today.getFullYear();

    var logFile = fs.createWriteStream(path.join(process.cwd(), 'logs', ('log-' + logTime + '.txt')), { flags: 'a' }); // Or 'w' to truncate the file every time the process starts.

    console.log("Starting Logging to ", path.join(process.cwd(), 'logs', ('log-' + logTime + '.txt')));
    var log = console.log;

    //extends
    if (DEVELOPMENTFLAG) {
        console.log("DISABLED LOGGING REDIRECT");
    }
    else {
        console.log = function () {
            // logFile.write(util.format.apply(null, arguments) + '\n');
            // fs.writeToLogFile(JSON.stringify(str) + '\n');
            var args = Array.from(arguments);
            fs.writeToLogFile(JSON.stringify(args) + '\n');//PRODUCTION
            log.apply(console, args); //DEVELOPMENT
        }
    }
    console.error = console.log;
}

//========================================================================================================================================

function commonFileDialogOpen() {
    if (stateUX.fileDialogOpen == true) {
        //console.log("File dialog already open");
        pushMessageToLog("File dialog is already open");
        return false;
    }
    stateUX.fileDialogOpen = true; //now set to true
    return true;
}

function commonFileDialogClose(filePaths, canceled) {
    if (canceled) console.log("Dialog closed without selecting a file.")
    else console.log("Dialog closed: " + filePaths)
    stateUX.fileDialogOpen = false; //clear this first, even if it errors this will still get cleared and dialog will close
}

//========================================================================================================================================

function fileFixPath(filePath) {
    //console.log("fixed path: " + filePath);
    return filePath.toString().replace(/\\/g, "/");
}

//========================================================================================================================================


//========================================================================================================================================

function fileSaveSettingsFile() {
    console.log("fileSaveSettingsFile() ")
    //console.log(fs.readFileSync(path.join(process.cwd(), 'settings.ini')));
    let settings = JSON.parse(fs.readFileSync(path.join(process.cwd(), 'settings.ini'), 'utf8'));


    Object.keys(settings.software).forEach(function (key, index) {
        settings.software[key] = stateSoftware[key];
    });

    Object.keys(settings.communication).forEach(function (key, index) {
        settings.communication[key] = stateCommunication[key];
    });

    fs.writeFile(path.join(process.cwd(), 'settings.ini'), JSON.stringify(settings), function (err) {
        if (err) {
            console.log(err);
        }
    });
}

//========================================================================================================================================

function fileLoadSettingsFile() {
    console.log("fileLoadSettingsFile()");
    let settings;
    try {
        settings = JSON.parse(fs.readFileSync(path.join(process.cwd(), 'settings.ini'), 'utf8'));

    }
    catch (e) {
        //rebuild
        console.log("Problem with settings.ini, rebuilding from default")
        settings = JSON.parse(JSON.stringify(settingsDefault));
        //save it just in case - do it synch otherwise will error
        fs.writeFile(path.join(process.cwd(), 'settings.ini'), JSON.stringify(settings), function (err) {
            if (err) {
                console.log(err);
            }
        });
    }
    console.log(settings);

    if (settings.software.logging) systemStartLogging(); //PRODUCTION only


    Object.keys(settings.software).forEach(function (key, index) {
        stateSoftware[key] = settings.software[key];
    });

    Object.keys(settings.communication).forEach(function (key, index) {
        stateCommunication[key] = settings.communication[key];
    });

    uxTargetSaveFileUpdate(); //always starts with this file as the target - defaulted at program start
    uxTipsAndLabelsUpdate(stateSoftware.language); //if no argument it defaults to english

    // console.log(stateSoftware)
    // console.log(stateCommunication)
    //need to update UX elements - baud scan, port scan, window size, tooltip enable, auto connect enableetc

    win.setSize(settings.software.screenWidth, settings.software.screenHeight);

    //caused issues depending on when file is loaded, will do this when the serialPort scanner fires
    //convertSelectInputOption("ws-hardware-port", stateCommunication.prevSerialPort); //selects previous port if found
    //convertSelectInputOption("ws-hardware-baud", stateCommunication.baud); //selects previous baud

    document.getElementById('ws-software-recentfilename').innerText = path.basename(stateSoftware.previousSaveFile);

    document.getElementById('communication-ipadr-1').value = stateCommunication.networkIP.split('.')[0];
    document.getElementById('communication-ipadr-2').value = stateCommunication.networkIP.split('.')[1];
    document.getElementById('communication-ipadr-3').value = stateCommunication.networkIP.split('.')[2];
    document.getElementById('communication-ipadr-4').value = stateCommunication.networkIP.split('.')[3];
    document.getElementById('communication-ipport').value = stateCommunication.networkPort;

    document.getElementById('ws-software-autoconnect-box').checked = stateSoftware.autoConnectEnabled;
    document.getElementById('ws-software-autoconfigdownload-box').checked = stateSoftware.autoDownloadEnabled;
    document.getElementById('ws-software-lowperformance-box').checked = stateSoftware.lowPerformance;
    document.getElementById('ws-software-autoload-box').checked = stateSoftware.autoSequencesLoad;

    document.getElementById('ws-software-disabletips-box').checked = stateSoftware.tooltipsEnabled;

    //elem.options[elem.selectedIndex].getAttribute("data-lang"));
    helper('ws-software-language', stateSoftware.language);

    function helper(elem, match) {
        let optionNodes = document.getElementById(elem).options; //make a HTMLCollection
        let optionList = [];
        for (let i = 0; i < optionNodes.length; i++) optionList.push(optionNodes[i].getAttribute("data-lang"));
        document.getElementById(elem).selectedIndex = optionList.findIndex(element => element.includes(match));
    }
}

//========================================================================================================================================

function fileImportImage(mode) {
    console.log("fileImportImage()");

    if (!commonFileDialogOpen()) return;

    let options = {
        title: "Load an image file",
        //defaultPath: process.cwd(),
        buttonLabel: "Load Image File",
        filters: [
            { name: 'Image File Types', extensions: ['png', 'bmp', 'tiff', 'jpg', 'jpeg', 'gif'] }
        ],
        properties: ['openFile']
    }

    if (stateUX.fileDialogFirstOpen.povFile == false) {
        stateUX.fileDialogFirstOpen.povFile = true;
        options.defaultPath = path.join(process.cwd(), 'example files/povs');
    }

    //-------------------- Asynchronous - using promise -------------------------------
    dialog.showOpenDialog(options).then(prom => {
        commonFileDialogClose(prom.filePaths[0], prom.canceled);
        if (!prom.canceled) {

            if (mode == 'pov') sequenceLoadImageAsData(fileFixPath(prom.filePaths[0]), SelSeq.povDirMode);
            else if (mode == 'seqWizard') {
                let ddelem = document.getElementById("ovmenu-importwizard-mode");
                sequenceInitImageWizard(fileFixPath(prom.filePaths[0]));
                //  sequenceLoadImageAsData(fileFixPath(prom.filePaths[0]),ddelem.options[ddelem.selectedIndex].value);
            }
        }
    });
}

//========================================================================================================================================

function fileLoadUserGammaFile(color) {
    console.log("fileLoadUserGammaFile() for " + color);
    if (!commonFileDialogOpen()) return;

    let options = {
        title: "Load a gamma table file",
        //defaultPath: process.cwd(),
        buttonLabel: "Load File",
        filters: [
            { name: 'CSV File Types', extensions: ['txt', 'csv'] }
        ],
        properties: ['openFile']
    }

    if (stateUX.fileDialogFirstOpen.gammaFile == false) {
        stateUX.fileDialogFirstOpen.gammaFile = true;
        options.defaultPath = path.join(process.cwd(), 'example files/gamma');
    }

    //-------------------- Asynchronous - using callback -------------------------------

    dialog.showOpenDialog(options).then(prom => {
        commonFileDialogClose(prom.filePaths[0], prom.canceled);
        try {
            if (!prom.canceled) {
                let csvData = fs.readFileSync(prom.filePaths[0].toString(), 'utf8');
                let csv = csvData.toString();
                csv.replace(/\r?\n|\r/g, "");
                let arr = csv.split(",");
                arr = arr.map(Number); //essentially runs parseInt() on all elements
                //console.log(arr);
                gammaTables[color] = [...arr]; // create TRUE copy
                ovmGammaRender();
            }
        }
        catch (e) {
            console.log('Error:', e.stack);
        }
    });
}

//========================================================================================================================================

function fileLoadLayoutMap() {
    console.log("fileLoadLayoutMap()");
    if (!commonFileDialogOpen()) return;

    let options = {
        title: "Load a patch or map file",
        //defaultPath: process.cwd(),
        buttonLabel: "Load File",
        filters: [
            { name: 'TXT/CSV File Types', extensions: ['txt', 'csv', 'pixelmap'] }
        ],
        properties: ['openFile']
    }

    if (stateUX.fileDialogFirstOpen.patchFile == false) {
        stateUX.fileDialogFirstOpen.patchFile = true;
        options.defaultPath = path.join(process.cwd(), 'example files/patches');
    }

    //-------------------- Asynchronous - using callback -------------------------------

    dialog.showOpenDialog(options).then(prom => {
        commonFileDialogClose(prom.filePaths[0], prom.canceled);
        try {
            if (!prom.canceled) {
                let csvData = fs.readFileSync(prom.filePaths[0].toString(), 'utf8');
                let csv = csvData.toString();
                let arr = csv.split("\n").map(function (row) { return row.split("\t"); }); //make 2D array from file
                mainObj.layout.patchCoords = [];

                arr.forEach(function (val, index) {
                    if (val.length == 1 || val.length > 2) {
                        console.log("either first line or error with coordinate entry");
                    }
                    else mainObj.layout.patchCoords.push((val[0] + "," + val[1]).trim()); //build the coordinates as string, and trim any CR/LF/etc
                });

                if (mainObj.layout.patchCoords.length != parseInt(arr[0][0])) {
                    console.log("Patch file is not properly defined. Specified pixels is does not match available data. Continuing.");
                }

                mainObj.layout.fileName = path.parse(fileFixPath(prom.filePaths[0])).base;

                if (mainObj.layout.advanced) {
                    if (stateUX.layoutAdvancedPreview) {
                        document.getElementById("toolmenu-ledicon-advnumofpixels").value = mainObj.layout.patchCoords.length;
                        layoutGraphicBuildIcons(); //need to redraw for preview
                    }
                    //else coordinates are entered, wait for user to 'add to'
                }
                else {
                    mainObj.layout.iconAmount = mainObj.layout.patchCoords.length; //set to the defined amount, not specified amount
                    uxToolsLayoutUpdate(); //update iconAmount box
                    layoutGraphicIconReset(); //reset first
                    layoutGraphicBuildIcons(); //now X-Y are intialized and populated, build the positions
                }
            }
        }
        catch (e) {
            console.log('Error:', e.stack);
        }
    });
}

//========================================================================================================================================

function fileLoadSeqDataFile() {
    console.log("fileLoadSeqDataFile()");
    if (!commonFileDialogOpen()) return;

    let options = {
        title: "Load a data file",
        buttonLabel: "Load file",
        filters: [
            { name: 'Data File Types', extensions: ['csv', 'txt', '', 'cod'] }
        ],
        properties: ['openFile']
    }

    if (stateUX.fileDialogFirstOpen.seqModeFile == false) {
        stateUX.fileDialogFirstOpen.seqModeFile = true;
        options.defaultPath = path.join(process.cwd(), 'example files/file mode');
    }

    //-------------------- Asynchronous - using callback -------------------------------

    dialog.showOpenDialog(options).then(prom => {
        commonFileDialogClose(prom.filePaths[0], prom.canceled);
        // console.log("prom.canceled: ",prom.canceled);
        try {
            if (!prom.canceled) {
                //identify file
                let fileData = fs.readFileSync(fileFixPath(prom.filePaths[0]), 'utf8');
                let fileExt = path.extname(prom.filePaths[0]).toLowerCase();
                let arr;


                switch (fileExt) {
                    case '.txt':
                    case '.csv':
                        console.log("Aurora FilePlay Format");
                        let csvData = fs.readFileSync(fileFixPath(prom.filePaths[0]), 'utf8');
                        let csv = csvData.toString().replace(/ +?/g, '').split("\n");
                        arr = csv.filter(function (item) { return item.length > 1; });
                        arr = arr.map(function (row) { return row.replace(/,(\s+)?$/, '').split(","); }); //make 2D array from file
                        //console.log(arr);
                        break;
                    case '.cod':
                        console.log("COD Format");
                        //could figure out the rest of the header info, doubt it matters
                        //assumes all frames are 512 bytes/full packets
                        const cFrameSz = 512;
                        const cFrameHeaderSz = 19;
                        const cFrameOffsetSz = cFrameSz + cFrameHeaderSz;
                        let codArr = fs.readFileSync(fileFixPath(prom.filePaths[0]));
                        // console.log(codArr);

                        arr = [];
                        for (let i = 512; i < (codArr.length - cFrameOffsetSz); i += cFrameOffsetSz) {
                            let q = cFrameHeaderSz;
                            let row = [];
                            for (q = q; q < cFrameOffsetSz; q++) {

                                // if(q != cFrameHeaderSz) data+= ",";
                                //  data+= (b[i+q] & 0xff);
                                row.push(codArr[i + q]);
                            } //end q for()
                            console.log(row);

                            arr.push(row);

                        }
                        break;
                } //end switch

                //--------------------------------------------------------------------------------------------------------------

                SelSeq.frames = arr.length; //top level length
                SelSeq.numPixels = arr[0].length / getColorModeMult(SelSeq.colorMode);


                arr.forEach(function (v, i) {
                    arr[i] = v.map(Number); //convert array into numbers from string
                    SelSeq.dataFrames[i] = [...arr[i]]; //save the dataframe
                });
                //resize the sequence pixel amount so all data can be viewed. Does not expand.
                if (SelSeq.numPixels * getColorModeMult(SelSeq.colorMode) < SelSeq.dataFrames[0].length)
                    SelSeq.numPixels = Math.ceil(SelSeq.dataFrames[0].length / getColorModeMult(SelSeq.colorMode));

                pushMessageToLog("The color sequence data file has been loaded. It contains " + SelSeq.frames + " frames and " + SelSeq.dataFrames[0].length + " channels.");
                sequenceValidateData(SelSeq);
                SelSeq.errorFlag = false; //probably was set, no need to
                refreshLayouts();
                uxToolsSequenceUpdate();
            }
            //--------------------------------------------------------------------------------------------------------------
        }
        catch (e) {
            console.log('Error:', e.stack);
        }
    });
}

//========================================================================================================================================

function fileLoadGraphicLayoutImage() {
    console.log("fileLoadGraphicLayoutImage()");
    if (!commonFileDialogOpen()) return;

    let options = {
        title: "Load an image file",
        //defaultPath: process.cwd(),
        buttonLabel: "Load Image",
        filters: [
            { name: 'Image File Types', extensions: ['png', 'gif', 'jpg', 'tiff', 'bmp', 'jpeg', 'tif'] }
        ],
        properties: ['openFile']
    }

    //no defaultPath setting

    //-------------------- Asynchronous - using callback -------------------------------

    dialog.showOpenDialog(options).then(prom => {
        commonFileDialogClose(prom.filePaths[0], prom.canceled);
        try {
            if (!prom.canceled) {
                systemUndoRedoPush('layout');
                mainObj.layout.image.filePath = ""; //force a reset and center
                layoutPositionImageFunc(fileFixPath(prom.filePaths[0]));
            }
        }
        catch (e) {
            console.log('Error:', e.stack);
        }
    });
}

//========================================================================================================================================

function systemSaveSequenceFile() {
    console.log("systemSaveSequenceFile() to " + stateUX.targetSaveFile);
    //save to software target save file
    //save to stateUX.targetSaveFile

    if (stateUX.targetSaveFile == null || !fs.existsSync(stateUX.targetSaveFile)) {
        pushMessageToLog("File path invalid. Please use Save Sequence File As")
    }
    else {
        let copyObj = { ...mainObj };
        //console.log(copyObj)

        //---------------------------------------------------------------------------------------------
        mainObj.seq.forEach(function (seq, index) {
            let chanSz = sequenceGetChannelAmount(seq); //shorten this

            //-------------- Remove seq variables that are not required for the specified sequence mode --------------------

            //console.log(copyObj.seq[index].startVals) 
            //console.log(seq.name, seq.mode, seq.startVals);

            if (!cSeqModesStartVals.includes(seq.mode) && seq.hasOwnProperty("startVals")) {
                console.log("DESTROYED startvals on " + seq.name)
                delete copyObj.seq[index].startVals; //destroy
            }

            if (!cDataSeqModesDataFrames.includes(seq.mode) && seq.hasOwnProperty("frameVar")) {
                console.log("DESTROYED frameVar on " + seq.name)
                delete copyObj.seq[index].frameVar; //destroy
            }

            if (seq.mode == 'mList' && seq.hasOwnProperty("dataFrames")) {
                console.log("DESTROYED dataFrames on " + seq.name)
                delete copyObj.seq[index].dataFrames; //destroy
            }

            if (seq.mode != 'mPOV') {
                if (seq.hasOwnProperty("povDirMode")) delete copyObj.seq[index].povDirMode;
                if (seq.hasOwnProperty("povVideoMode")) delete copyObj.seq[index].povVideoMode;
                if (seq.hasOwnProperty("povGridSpeed")) delete copyObj.seq[index].povGridSpeed;
            }

            if (seq.mode != 'mList' && seq.hasOwnProperty("listFlags")) delete copyObj.seq[index].listFlags; //destroy
            if (seq.mode != 'mList' && seq.hasOwnProperty("listID")) delete copyObj.seq[index].listFlags; //destroy
            if (seq.mode != 'mList' && seq.hasOwnProperty("listIteration")) delete copyObj.seq[index].listFlags; //destroy

            //------------------------ Crop off Data, if larger than sequence's channel size ------------------

            if (seq.hasOwnProperty("startVals")) {
                if (seq.startVals.length > chanSz) copyObj.seq[index].startVals = systemResize8Array(seq.startVals, chanSz);
            }

            if (seq.hasOwnProperty("frameVar")) {
                if (seq.frameVar.length > chanSz) copyObj.seq[index].frameVar = systemResize8Array(seq.frameVar, chanSz);
            }

            if (seq.hasOwnProperty("dataFrames")) {
                //resize [frames][chanSz]
                //console.log(seq.name + "+++++++" + seq.dataFrames.length, seq.frames);
                if (seq.dataFrames.length > seq.frames) {
                    let newArr = Array.from(seq.dataFrames);
                    newArr.length = seq.frames;
                    copyObj.seq[index].dataFrames = []; //clear it, think this is easier than pop()

                    newArr.forEach(function (data, i) {
                        copyObj.seq[index].dataFrames.push(newArr[i]);
                    });
                }

                seq.dataFrames.forEach(function (data, i) {
                    if (seq.dataFrames[i].length > chanSz) copyObj.seq[index].dataFrames[i] = systemResize8Array(data, chanSz);
                });
            } //end dataFrames if()


            //-------- format data for json saving, typed arrays are considered an object ------

            if (seq.hasOwnProperty("dataFrames")) {
                seq.dataFrames.forEach(function (data, i) {
                    copyObj.seq[index].dataFrames[i] = Array.from(data);
                });
            }

            if (seq.hasOwnProperty("frameVar")) {
                copyObj.seq[index].frameVar = Array.from(copyObj.seq[index].frameVar);
            }

            if (seq.hasOwnProperty("startVals")) {
                copyObj.seq[index].startVals = Array.from(copyObj.seq[index].startVals);
            }

        }); //end seq for each()

        //---------------------- Device Information ----------------------------------------------

        let devObj = {
            simulate: (mainObj.device.simulate) ? true : false,
            id: stateDevice.hard.idNum,
            ver: stateDevice.hard.version,
            rev: stateDevice.hard.revision
        }

        copyObj.device = {};
        Object.assign(copyObj.device, devObj);

        //--------------------------- Software Information -----------------------------------------

        copyObj.software = stateSoftware.version;

        //------------------------------- Layout Information -------------------------------------

        if (mainObj.layout.shape != "shapeMap" && copyObj.layout.patchCoords != undefined) delete copyObj.layout.patchCoords;
        if (mainObj.layout.shape != "shapeMap" && mainObj.layout.shape != "shapeVector" && copyObj.layout.fileName != undefined) delete copyObj.layout.fileName;

        if (!mainObj.layout.advanced) {
            //nothing anymore.....
        }

        if (!mainObj.layout.imageEnabled) delete mainObj.layout.image; //not enabled, when saved, crop off data

        //--------------------------------------------------------------------

        //console.log(copyObj); //DEBUG

        fs.writeFile(stateUX.targetSaveFile, JSON.stringify(copyObj), function (err) {
            if (err) {
                console.log(err);
            }
        });

        //--------------------------------------------------------------------

        if (stateSoftware.previousSaveFile != stateUX.targetSaveFile && path.basename(stateSoftware.targetSaveFile) != 'aurora-demonstration-file.auroraseq') {
            stateSoftware.previousSaveFile = fileFixPath(stateUX.targetSaveFile);
            fileSaveSettingsFile();
        }

        pushMessageToLog("Finished Saving Sequence file to " + stateUX.targetSaveFile);
    }
}

//========================================================================================================================================

function fileSaveAsSequenceFile() {
    console.log("fileSaveAsSequenceFile()");
    //open dialog to select a new file name

    if (!commonFileDialogOpen()) return;

    let options = {
        title: "Create or Select a File",
        //defaultPath: process.cwd(),
        buttonLabel: "Save File",
        filters: [
            { name: 'Aurora File', extensions: ['auroraseq'] }
        ],
        properties: ['saveFile']
    }

    if (stateUX.fileDialogFirstOpen.sequenceFile == false) {
        stateUX.fileDialogFirstOpen.sequenceFile = true;
        options.defaultPath = path.join(process.cwd(), 'saves');
    }

    //-------------------- Asynchronous - using callback -------------------------------

    dialog.showSaveDialog(options).then(prom => {
        commonFileDialogClose(prom.filePath, prom.canceled);
        try {
            if (!prom.canceled) {
                stateUX.targetSaveFile = fileFixPath(prom.filePath); //user has specified the file path is the target
                uxTargetSaveFileUpdate();
                //create the file first, fileSave() checks to make sure the file exists first
                fs.writeFile(prom.filePath.toString(), JSON.stringify('hold'), function (err) {
                    if (err) {
                        return console.log(err);
                    }
                    systemSaveSequenceFile();
                });
            }
        }
        catch (e) {
            console.log('Error:', e.stack);
        }
    });
}

//========================================================================================================================================

function fileLoadPreviousSequenceFile() {
    console.log("fileLoadPreviousSequenceFile() ");
    systemLoadSequenceFileInit(stateSoftware.previousSaveFile, true);
}

//========================================================================================================================================
function fileQuickSaveSequenceFile() {
    console.log("fileQuickSaveSequenceFile()");
    //incrementing quick save
    systemSaveSequenceFile();
}

//========================================================================================================================================

function fileLoadSequenceFile() {
    console.log("fileLoadSequenceFile()")

    if (!commonFileDialogOpen()) return;

    let options = {
        title: "Load Aurora Sequence File .auroraseq or .txt",
        //defaultPath: path.join(process.cwd(), 'saves'),
        buttonLabel: "Load Sequence File",
        filters: [
            { name: 'Custom File Type', extensions: ['auroraseq', 'txt'] } //custom filename
        ],
        properties: ['openFile']
    }

    if (stateUX.fileDialogFirstOpen.sequenceFile == false) {
        stateUX.fileDialogFirstOpen.sequenceFile = true;
        options.defaultPath = path.join(process.cwd(), 'saves');
    }

    //-------------------- Asynchronous - using callback -------------------------------
    dialog.showOpenDialog(options).then(prom => {
        commonFileDialogClose(prom.filePaths[0], prom.canceled);
        if (!prom.canceled) systemLoadSequenceFileInit(fileFixPath(prom.filePaths[0]), true);
    });
} //end func

//========================================================================================================================================

function fileConvertSequenceFile(filePath) {
    console.log("fileConvertSequenceFile() " + filePath);
    let tarVer = 0;

    try {
        var csvData = fs.readFileSync(filePath, 'utf8');
        var csv = csvData.toString();
    }
    catch (e) {
        console.log('Error:', e.stack);
    }

    let raw = csvToArray(csv);

    //-----------------------------------------------------------------------------------------

    //remove all the leading empty slot so doesn't have to be done individually
    raw.forEach(function (arr, index) {
        arr.splice(0, 1);
    });

    //-----------------------------------------------------------------------------------------

    console.log("Converting from Ver: " + raw[0][0] + "" + String.fromCharCode(97 + parseInt(raw[0][1])));

    //branch to version 1 and 2
    //seems like only the file header and the per sequence header was altered.
    //frameSpecialVar and frameSpecial were removed from v2

    let numOfUserChans = raw[0][4]; //UserChannelAmt - not used I think
    let numOfSequences = 0;
    //let advancedFlag = false;
    //let configShapeID = 0;

    mainObj.device = {}; //reset device object

    if (raw[0][0] == '1') {
        //file is version 1
        tarVer = 1;
        console.log("Running File Conversion for version 1");

        //VERSION 1 FILE HEADER
        //SoftVer - SoftRev - Month(mon/day/year) - HardwareID - UserChannelAmt - device.Channels - 
        //      LEDs Per Chan - ConfigShape ID - LEDSizeID - Sequences Length - graphicImgFilePath
        mainObj.layout.shape = convertLayoutShapeV2(parseInt(raw[0][7])); //NOT SURE IF V1 is the same as V2
        mainObj.layout.iconAmount = 32; //just chose a number, whatever, I don't want it to stay whatever it was set to

        mainObj.device.id = parseInt(raw[0][3]);

        numOfSequences = parseInt(raw[0][9]); //only used locally
        if (raw[0][10].length > 1) pushMessageToLog("Ignoring Graphic Layout Image, file specified: " + raw[0][10]);
    }
    else if (raw[0][0] == '2') {
        //file is version 2
        tarVer = 2;
        console.log("Running File Conversion for version 2");

        //VERSION 2 FILE HEADER
        //SoftV - SoftRev - Month(mon/day/year) - HardwareID - UserChannelAmt - device.maxPixels - Sequences Length[6]
        //		ConfigShape ID - LEDSizeID - layoutAdvanced flag[9] - ImgEnabled - ImgFilePath[11] - ImgX - ImgY - ImgWidth - ImgHeight - 
        //	    GlobalIntensity[16] - GlobalIntensity Enable  - idleSeqID - LEDSpacingID - SequencesTotalLength OR CurrentPixelUsage

        mainObj.device.id = parseInt(raw[0][3]); //set device, no ver/rev available

        numOfSequences = parseInt(raw[0][6]); //local usage
        mainObj.layout.shape = convertLayoutShapeV2(parseInt(raw[0][7])); //local usage
        mainObj.layout.advanced = (raw[0][9] == "true");  //layoutAdvanced flag
        if (mainObj.layout.shape == 'shapeManual' || mainObj.layout.shape == 'shapeMap') mainObj.layout.advanced = true;


        //this is all screwed up, whatever. Not a great solution...
        if (raw[0][20] != undefined) {
            mainObj.layout.iconAmount = parseInt(raw[0][20]);  //CurrentPixelUsage has something to do with the graphic layout, packet cloning and weird shit like that
            if (mainObj.layout.iconAmount > stateSoftware.maxPixels) mainObj.layout.iconAmount = stateSoftware.maxPixels;
        }
        else mainObj.layout.iconAmount = parseInt(raw[0][4]); //UserChannelAmt

        //stateUX.graphicImgEnabled = raw[0][10]; //ImgEnabled flag
        //stateUX.graphicImgFilePath = raw[0][11]; //ImgFilePath
        //stateUX.graphicImgX = raw[0][12]; //ImgX
        //stateUX.graphicImgY = raw[0][13]; //ImgY
        //stateUX.graphicImgW = raw[0][14]; //ImgWidth
        //stateUX.graphicImgH = raw[0][15]; //ImgHeight

        mainObj.layout.imageEnabled = (raw[0][10] == "true"); //convert to boolean
        if (mainObj.layout.imageEnabled) {
            mainObj.layout.image = {}; //create image sub-object
            mainObj.layout.image.filePath = parseInt(raw[0][11]); //ImgFilePath
            mainObj.layout.image.offsetX = parseInt(raw[0][12]); //ImgX
            mainObj.layout.image.offsetY = parseInt(raw[0][13]); //ImgY
            mainObj.layout.image.width = parseInt(raw[0][14]); //ImgWidth
            mainObj.layout.image.height = parseInt(raw[0][15]); //ImgHeight
        }

        stateSeq.globalIntensity = parseInt(raw[0][16]);
        if (stateSeq.globalIntensity > 100) stateSeq.globalIntensity = 100;
        stateSeq.globalIntensityEnable = (raw[0][17] == "true");
        mainObj.idleSeqID = parseInt(raw[0][18]) - 1; //idleSeqID
        if (mainObj.idleSeqID < 0 || mainObj.idleSeqID > 255) mainObj.idleSeqID = 0; //out of range, set to default
        //LED spacing will have to manually setup
    }
    else {
        console.log("Can not validate version for save file.");
        pushMessageToLog("File may be corrupt or an incompatible version");
        return;
    }

    console.log("Channels: " + numOfUserChans + "  #OfSequences: " + numOfSequences);

    //-----------------------------------------------------------------------------------------

    //mainObj = { "seq": [] }; //reset sequence object to blank/empty/template
    mainObj.seq = []; //reset sequences

    let slideArr = [];
    let slideFlagArr = [];
    let seqid = 0;
    let row = 1; //row0 held file header

    if (tarVer == 1) { //---------------------------------------------------------------------------------------------------------------------
        //V1 sequence header
        //UserSequences[SelSeq].UserID - SeqLength - UserSequences[SelSeq].DataSize - UserSequences[SelSeq].SeqFrameAmount - 
        //  UserSequences[SelSeq].DataFrameAmount - UserSequences[SelSeq].Speed - UserSequences[SelSeq].SeqMode - UserSequences[SelSeq].DataType - holdLastValue);
        for (seqid = 0; seqid < numOfSequences; seqid++) {
            //console.log(arr);
            console.log("Ver1 - Running sequence iteration of " + seqid + ". Starting on row " + row)

            //mainObj.seq.push(seqTemplate); //causes mainObj.seq[index] to reference seqTemplate
            mainObj.seq.push(JSON.parse(JSON.stringify(seqTemplate))); //dirty but pure JS way to copy an object rather than reference

            console.log(raw[row][0]);
            //parse sequence header
            mainObj.seq[seqid].name = raw[row][0];
            mainObj.seq[seqid].description = "Ported from v1";//Ported from Version 1 Aurora file: ADDFILENAME"
            mainObj.seq[seqid].memory = parseInt(raw[row][1]); //SeqLength - just recalculate it

            mainObj.seq[seqid].frames = parseInt(raw[row][3]) + 1;    //SeqFrameAmount
            //mainObj.seq[seqid].frames = parseInt(raw[row][4])+1;   //DataFrameAmount  - the +1 makes it compatible with the new base 1 value
            mainObj.seq[seqid].speed = parseInt(raw[row][5]);   //Speed
            mainObj.seq[seqid].mode = convertSeqModeV1(raw[row][6]);   //SeqMode
            mainObj.seq[seqid].colorMode = convertColorMode(parseInt(raw[row][7]));   //DataType
            //holdLastValue used to store either gradient steps or POVconfig type
            if (mainObj.seq[seqid].mode == 'mPOV') {
                mainObj.seq[seqid].povGridSpeed = raw[row][8];
                mainObj.seq[seqid].generateSteps = 0;
                mainObj.seq[seqid].povDirMode = 0; //no way to know, use default
            }
            else {
                mainObj.seq[seqid].generateSteps = parseInt(raw[row][8]);
                //mainObj.seq[seqid].povGridSpeed = 0; //don't need this then....
            }

            //now with other data loaded:
            mainObj.seq[seqid].numPixels = Math.floor(parseInt(raw[row][2]) / getColorModeMult(mainObj.seq[seqid].colorMode)); //DataSize - convert to number of pixels

            if (mainObj.seq[seqid].mode == 'mPOV') mainObj.seq[seqid].povGridSpeed = raw[row][8];
            else mainObj.seq[seqid].generateSteps = parseInt(raw[row][8]);

            row++;

            //now start populating payloads
            if (mainObj.seq[seqid].mode == 'mList') {
                //Software Side: frameVar=TransitionArray[],  SlideValues[] = IterationArray[] , SlideFlags[] = SequenceIDArray[] 
                //On save file SlideFlags -> SlideValues -> frameVar

                //adds these properties to object
                //------------------------------- array of sequence ID numbers -------------------------------
                mainObj.seq[seqid].listID = raw[row].map(function (val) {
                    return parseInt(val) - 1; //apply base 0 offset, v1 & v2 use base 1, v3 uses base 0 - TEST THIS
                }); //8-bit
                row++;
                //------------------------------- iteration values -------------------------------
                mainObj.seq[seqid].listIteration = raw[row].map(Number); //16-bit
                row++;
                //------------------------------- transition and other flags -------------------------------
                mainObj.seq[seqid].listFlags = raw[row].map(Number); //8-bit
                row++;
            }
            else {
                //all other sequence modes
                //------------------------------- startvals -------------------------------
                mainObj.seq[seqid].startVals = Uint8Array.from(raw[row]);
                row++;
                //------------------------------- DUMP special framevar -------------------------------
                row++;
                //------------------------------- frameVar -------------------------------
                mainObj.seq[seqid].frameVar = Uint8Array.from(raw[row]);
                row++;
                //------------------------------- DUMP specialval -------------------------------
                row++;
                //------------------------------- Slidevalues? -------------------------------
                if (mainObj.seq[seqid].mode == 'mInstant') //should also catch mCombo, which no longer exists, not really mTech......
                {
                    //Dump Slidevals
                    slideArr = raw[row];
                    //console.log(slideArr);
                    //doesn't check or know if slide feature was used, notified later
                    row++;
                }
                //------------------------------- FrameSpeeds? -------------------------------
                if (mainObj.seq[seqid].mode == 'mCombo') //which no longer exists, not really mTech......
                {
                    //Dump Frame Speeds
                    row++;
                }
                //------------------------------- DataFrames -------------------------------
                mainObj.seq[seqid].dataFrames = []; // NOT SURE THIS IS CORRECT

                if (mainObj.seq[seqid].mode == 'mInstant') {
                    //handles framespecial bullshit
                    for (let q = 0; q < mainObj.seq[seqid].frames; q++) {
                        if (slideArr[q] == 0) {
                            //console.log(seqid + " filling frame: " + q);
                            mainObj.seq[seqid].dataFrames[q] = raw[row];
                            row++;
                        }
                        else {
                            //console.log("Filling in slide frame " + q + " by duplicating previous");
                            mainObj.seq[seqid].description += ", slide frame ignored, slide frame data duplicated";
                            if (q > 0) mainObj.seq[seqid].dataFrames[q] = Uint8Array.from(mainObj.seq[seqid].dataFrames[q - 1]);
                            else mainObj.seq[seqid].dataFrames[q] = Uint8Array.from(raw[row]); //error happened first frame should not be a slide
                            //row++; //added this for some reason, then some files didn't work. WHy why why
                        }
                    }
                }
                else {
                    //normal dataFrame loading
                    for (let q = 0; q < mainObj.seq[seqid].frames; q++) {
                        //console.log(seqid + " filling frame: " + q);
                        mainObj.seq[seqid].dataFrames[q] = Uint8Array.from(raw[row]);
                        row++;
                    }
                }

                if (mainObj.seq[seqid].mode == 'mCombo') {
                    //handle this, since mCombo is not defined, just set it
                    //console.log("mCombo sequence mode is discontinued, defaulting sequence - please remove")
                    mainObj.seq[seqid].description += ", sequence was a Combo and is discontinued";
                    mainObj.seq[seqid].mode = 'mFade'; //anything really, it should be deleted
                    mainObj.seq[seqid].description += ", defaulted to Fade - delete this sequence and remake it";
                }
            } //end else for sequence modes != mList
        } //end for

        //------------------------------------ now load index ---------------------------------------------------

        mainObj.userIndex = raw[row];
        row++;
        //console.log(mainObj.userIndex);
        mainObj.userIndex.forEach(function (val, index) {
            //console.log(val)
            mainObj.userIndex[index] = val - 1; //was base 1, now base 0
            if (val >= 255) mainObj.userIndex[index] = ''; //make it empty/undefined/whatever rather than key number
        });

        mainObj.userIndex = mainObj.userIndex.filter(function (e) { return e }); //filter out any now empty elements
        //console.log(mainObj.userIndex);
        //----------------------------------------------------------------------------
        //attempt to recover coordinates

        if (mainObj.layout.shape == 'shapeMap' || mainObj.layout.shape == 'shapeManual') {
            //then the coordinates are probably there
            mainObj.layout.advanced = true;
        }

        //----------------------------------------------------------------------------

        mainObj.description = "Ported sequence file from Aurora Software Ver: " + raw[0][0] + "  Rev: " + raw[0][1] + ". Was for Device ID: " + mainObj.device.id + "  Dated: " + raw[0][2];

        pushMessageToLog("Version 1 Aurora file converted & loaded successfully");
        //----------------------------------------------------------------------------
    } //end version 1
    else if (tarVer == 2) { //---------------------------------------------------------------------------------------------------------------------
        //V2 Sequence Header
        //UserSequences[SelSeq].UserID - UserSequences[SelSeq].Length - UserSequences[SelSeq].Channels - UserSequences[SelSeq].SeqFrames - 
        //	UserSequences[SelSeq].Speed(4) - UserSequences[SelSeq].SeqMode - UserSequences[SelSeq].ColorType - UserSequences[SelSeq].SeqPlayMode(7) - 
        //	UserSequences[SelSeq].SeqTransitionFlags - UserSequences[SelSeq].SeqTransSpeed - UserSequences[SelSeq].SequenceIntensity(10) - holdLastValue1 - 
        //	holdLastValue2 - UserSequences[SelSeq].PacketCloneMode(13);

        //could just be the sequence headers - well no, no framespecial shit, but still slideframes - so slightly different
        console.log("Starting Ver2 Conversion of " + numOfSequences);

        for (seqid = 0; seqid < numOfSequences; seqid++) {
            console.log("Ver2 - Running sequence iteration of " + seqid + ". Starting on row " + row)
            //console.log(raw[row]);
            //console.log(mainObj.seq);

            //mainObj.seq.push(seqTemplate); //causes mainObj.seq[index] to reference seqTemplate
            mainObj.seq.push(JSON.parse(JSON.stringify(seqTemplate))); //dirty but pure JS way to copy an object rather than reference

            //parse sequence header
            mainObj.seq[seqid].name = raw[row][0];
            mainObj.seq[seqid].description = "Ported from v2";//Ported from Version 1 Aurora file: ADDFILENAME"
            mainObj.seq[seqid].memory = parseInt(raw[row][1]); //SeqLength - just recalculate it
            //[2] was datasize, handled below
            mainObj.seq[seqid].frames = parseInt(raw[row][3]) + 1;    //SeqFrameAmount
            mainObj.seq[seqid].speed = parseInt(raw[row][4]);   //Speed
            mainObj.seq[seqid].mode = convertSeqModeV2(raw[row][5]);   //SeqMode
            mainObj.seq[seqid].colorMode = convertColorMode(parseInt(raw[row][6]));   //DataType
            mainObj.seq[seqid].playMode = convertPlayMode(parseInt(raw[row][7]));   //SeqPlayMode
            mainObj.seq[seqid].transition = convertTransitionFlags(parseInt(raw[row][8]));   //SeqTransitionFlags
            //mainObj.seq[seqid]. = parseInt(raw[row][9]);   //SeqTransSpeed
            mainObj.seq[seqid].intensity = parseInt(raw[row][10]);   //SequenceIntensity
            //raw[row][11]; //holdLastValue1
            //raw[row][12]; //holdLastValue2

            if (mainObj.seq[seqid].mode == 'mPOV') {
                mainObj.seq[seqid].povGridSpeed = raw[row][11];
                //mainObj.seq[seqid].povFramesAmt = raw[row][12]; //figure out what this does
                mainObj.seq[seqid].generateSteps = 0;
                mainObj.seq[seqid].povDirMode = 0; //no way to know, use default
            }
            else {
                mainObj.seq[seqid].generateSteps = parseInt(raw[row][11]);
                //mainObj.seq[seqid].povGridSpeed = 0; //Do not think I need these....
                //mainObj.seq[seqid].povFramesAmt = 0;
            }

            mainObj.seq[seqid].packetClone = parseInt(raw[row][13]);

            //now that other data loaded:
            mainObj.seq[seqid].numPixels = Math.floor(parseInt(raw[row][2]) / getColorModeMult(mainObj.seq[seqid].colorMode)); //DataSize - convert to number of pixels
            row++;

            //--------------------------------------- now populate payload ------------------------------------------------

            if (mainObj.seq[seqid].mode == 'mList') {
                //Software Side: frameVar=TransitionArray[],  SlideValues[] = IterationArray[] , SlideFlags[] = SequenceIDArray[] 
                //On save file SlideFlags -> SlideValues -> frameVar

                mainObj.seq[seqid].numPixels = 0; //force to 0 - prevents errors displaying pixel icons....

                //adds these properties to object
                //array of sequence ID numbers - v1 and v2 used base 1 for IDs, v3 uses base 0
                mainObj.seq[seqid].listID = raw[row].map(function (val) {
                    return parseInt(val) - 1; //apply base 0 offset
                }); //8-bit
                row++;
                //iteration values
                mainObj.seq[seqid].listIteration = raw[row].map(Number); //16-bit
                row++;
                //transition and other flags
                mainObj.seq[seqid].listFlags = raw[row].map(Number); //8-bit
                row++;
            }
            else {
                if (mainObj.seq[seqid].mode == 'mInstant') {
                    //frameVar - slide values - slide flags
                    mainObj.seq[seqid].frameVar = Uint8Array.from(raw[row]);
                    row++;
                    slideArr = Uint8Array.from(raw[row]); //slide values
                    row++;
                    slideFlagArr = Uint8Array.from(raw[row]); //keep slide flags for later polling
                    row++;
                }
                else if (mainObj.seq[seqid].mode == 'mFile' || mainObj.seq[seqid].mode == 'mPOV') {
                    //nothing, just dataframes
                }
                else if (mainObj.seq[seqid].mode == 'mTech') {
                    //should never be a mTech coming from V2
                }
                else { //fade,gradient,tech
                    //startvalues - frameVar
                    mainObj.seq[seqid].startVals = Uint8Array.from(raw[row]);
                    row++;
                    mainObj.seq[seqid].frameVar = Uint8Array.from(raw[row]);
                    row++;
                }

                //------------------------- dataframes -------------------------

                mainObj.seq[seqid].dataFrames = []; // NOT SURE THIS IS CORRECT

                if (mainObj.seq[seqid].mode == 'mInstant') {
                    //handles slideframe bullshit
                    for (let q = 0; q < mainObj.seq[seqid].frames; q++) {
                        if (slideFlagArr[q] == 0) {
                            //console.log(seqid + " filling frame: " + q);
                            mainObj.seq[seqid].dataFrames[q] = Uint8Array.from(raw[row]); //CONVERT FROM 16-BIT!
                            row++;
                        }
                        else {
                            /*
                            //slide is enabled
                            if ((0x40 & slideFlagArr[q]) == 0) {
                                //down
                            }
                            else {
                                //up
                                let buildArr = Array.from(mainObj.seq[seqid].dataFrames[q]);
                                console.log("Inserting ", (q+slideCount));
                                mainObj.seq[seqid].dataFrames.splice(q+SlideCount,0,...buildArr);
                                SlideCount++;
                            }
                            */
                            //console.log("Filling in slide frame by duplicating previous");
                            mainObj.seq[seqid].description += ", slide frame ignored, slide frame data duplicated";
                            mainObj.seq[seqid].dataFrames[q] = Uint8Array.from(mainObj.seq[seqid].dataFrames[q - 1]);
                            row++;
                        }
                    }

                    if (mainObj.seq[seqid].frames == 1) { //&& mode is mInstant
                        mainObj.seq[seqid].mode = 'mScene'; //convert it to a scene mode
                    }
                }
                else {
                    //Normal dataFrame loading
                    //console.log(mainObj.seq[seqid].dataFrames);
                    for (let q = 0; q < mainObj.seq[seqid].frames; q++) {
                        //console.log(seqid + " filling frame: " + q + " of " + mainObj.seq[seqid].frames + "   row: " + row);
                        mainObj.seq[seqid].dataFrames[q] = Uint8Array.from(raw[row]);
                        row++;
                    }
                }
            } //end not mList
        } //end seqid for()

        //-------------------------------- Index --------------------------------------

        let newUserIndex = raw[row];
        row++;

        mainObj.userIndex = []; //clear

        newUserIndex.forEach(function (val, index) {
            if (val < 255) mainObj.userIndex.push(val - 1);
            //mainObj.userIndex[index] = val - 1; //was base 1, now base 0
            //if (val > 255) mainObj.userIndex[index] = ''; //make it empty/undefined/whatever rather than key number. Keys would be for old serial/dmx index tiles
        });
        //mainObj.userIndex = mainObj.userIndex.filter(function (e) { return e }); //filter out any now empty elements

        //----------------------------------------------------------------------

        mainObj.description = "Ported sequence file from Aurora Software Ver: " + raw[0][0] + "  Rev: " + raw[0][1] + ". Was for Device ID: " + mainObj.device.id + "  Dated: " + raw[0][2];

        pushMessageToLog("Version 2 Aurora file converted & loaded successfully");
    } //end version 2 //---------------------------------------------------------------------------------------------------------------------

    //----------------------------------- COMMON FOR V1 and V2, last 3 lines --------------------------------------------------------
    //Either last line of junk data..... on really old versions(purple-inf) or perhaps those without XY & groups - appeared to be the extended index
    //OR X-Y-ZoneID

    if (mainObj.layout.advanced) {
        try {
            //coordinates, groupIDs(zoneIDs),
            if (mainObj.layout.shape == 'shapeMap' || mainObj.layout.shape == 'shapeManual') {
                console.log("Specified configshape uses coordinates, loading");
                mainObj.layout.ledIconCoordX = raw[row].map(Number).filter(val => val != 0); //LED icon coordinates or positionsX - remove all 0 coordinates, could happen but shouldn't
                row++;
                mainObj.layout.ledIconCoordY = raw[row].map(Number).filter(val => val != 0); //LED icon coordinates or positionsY
                row++;
                //console.log(mainObj.layout.ledIconCoordX);
                //console.log(mainObj.layout.ledIconCoordY);
            }
            //Since iconAmount was set with an uncertain value, this will be more correct. Expects RGB

            //LED Icon group IDs
            mainObj.layout.ledIconZoneID = raw[row].map(function (val) {
                return parseInt(val) + 1; //apply base 0 offset, v1 & v2 use base 1, v3 uses base 0 - TEST THIS
            }); //8-bit       
            //console.log(mainObj.layout.ledIconZoneID);

            //crops the arrays down, incase of bad data or something
            let min = Math.min(mainObj.layout.ledIconCoordX.length, mainObj.layout.ledIconCoordY.length);
            min = Math.min(min, mainObj.layout.ledIconZoneID.length);
            mainObj.layout.iconAmount = min;

            if (mainObj.layout.ledIconCoordX.length > mainObj.layout.iconAmount) mainObj.layout.ledIconCoordX.length = mainObj.layout.iconAmount;
            if (mainObj.layout.ledIconCoordY.length > mainObj.layout.iconAmount) mainObj.layout.ledIconCoordY.length = mainObj.layout.iconAmount;
            if (mainObj.layout.ledIconZoneID.length > mainObj.layout.iconAmount) mainObj.layout.ledIconZoneID.length = mainObj.layout.iconAmount;
        }
        catch (e) {
            console.log("An error occured porting coordinates, zone IDs. Disable advanced layout.")
            mainObj.layout.advanced = false;
        }
    }

    //----------------------------------------------------------------------

    mainObj.layout.centerX = 0; //setting to 0 indicates a reset of positioning
    mainObj.layout.centerY = 0;

    delete mainObj.device; //not used ever.... only if device Profiles are loaded with a save file which is no longer the case

    //----------------------------------------------------------------------
} //end func

//========================================================================================================================================

function convertColorMode(arg) {
    if (arg == 'RGB' || arg == 1) {
        return "RGB";
    }
    if (arg == 'Mono' || arg == 0) {
        return "Mono";
    }
    if (arg == 'RGBW' || arg == 2) {
        return "RGBW";
    }
    return "RGB"; //set to default
}

//========================================================================================================================================

function convertSeqModeV1(arg) {
    //console.log("convertSeqModeV1() with "+arg)
    switch (parseInt(arg)) {
        default:
        case 0: //mStall
            //serious error occurred, no idea how to parse the rest of the data
            console.log("could not convertSeqModeV1 with: " + arg);
            return "UHOH";
        case 1: //mFade
            return "mFade";
        case 2: //mInstant
            return "mInstant";
        case 3: //mGradient
            return "mGradient";
        case 4: //mCombo
            return "mCombo"; // will not convert
        case 5: //mLinked
            return "mList"; //new name
        case 6: //mPOV
            return "mPOV"; //new name
    } //end switch
}

//========================================================================================================================================

function convertSeqModeV2(arg) {
    //console.log("convertSeqModeV1() with "+arg)
    switch (parseInt(arg)) {
        default:
        case 0: //mStall
            return "mStall";
        case 1: //mFade
            return "mFade";
        case 2: //Gradient
            return "mGradient";
        case 3: //Instant
            return "mInstant";
        case 4: //mTech
            return "mTech";
        case 5: //POV
            return "mPOV";
        case 6: //File
            return "mFile";
        case 7: //Linked/List
            return "mList";
    } //end switch
}

//========================================================================================================================================

function convertTransitionFlags(val) {
    console.log("convertTransitionFlags()");
    switch (val) {
        default:
        case 0:
            return 'transInstant';
        case 1:
            return 'transSlowFade';
        case 2:
            return 'transFastFade';
    }
}

//========================================================================================================================================

function convertPlayMode(val) {
    console.log("convertPlayMode()");
    switch (val) {
        default:
        case 0: //loop
            return 'playLoop';//default
        case 1: //one shot and go to next
            return 'playNext';
        case 2: //One shot start 
            return 'playStart';
        case 3: //One shot end
            return 'playEnd';
        case 4: //one shot blank
            return 'playBlank';
        case 5: //one shot idle
            return 'playIdle';
    }
}

//========================================================================================================================================

function convertLayoutShapeV2(val) {
    console.log("convertLayoutShapeV2()" + val);
    switch (val) {
        default:
            pushMessageToLog("The specified layout shape could not be ported, defaulting.");
        case 1:
            return 'shapeRound';
        case 2:
            return 'shapeSquare';
        case 3:
            return 'shapeSpiral';
        case 4:
            return 'shapeLine';
        case 5:
            return 'shapeMap';
        case 6:
            return 'shapeManual'; //only used for legacy file conversion. No longer a layout shape
    }
}

//========================================================================================================================================
