'use strict';

//========================================================================================================================================

function getColorModeMult(arg) {
    if (arg == 'Mono' || arg == "0") {
        return 1;
    }
    if (arg == 'RGB' || arg == "1") {
        return 3;
    }
    if (arg == 'RGBW' || arg == "2") {
        return 4;
    }
    return 3; //RGB is default
}

//========================================================================================================================================

function getValue16BitBinary(val) {
    let newVal = val.toString(2);
    while (newVal.length < 16) {
        newVal = "0" + newVal;
    }
    return newVal;
}

//========================================================================================================================================

function getValue8BitBinary(val) {
    let newVal = val.toString(2);
    while (newVal.length < 8) {
        newVal = "0" + newVal;
    }
    return newVal;
}


//========================================================================================================================================

function getValue16BitMSB(val) {
    return ((val & 0x0000FF00) >> 8);
}

function getValue16BitLSB(val) {
    return (val & 0x000000FF);
}

//========================================================================================================================================

function getBreak32BitValue(val) {
    let arr = []
    arr.push(((val & 0xFF000000) >> 24));
    arr.push(((val & 0x00FF0000) >> 16));
    arr.push(((val & 0x0000FF00) >> 8));
    arr.push(val & 0x000000FF);
    return arr; //returns MSB first
}

//========================================================================================================================================

function convertSelectInputOption(elem, match) {
    //console.log(elem, match);
    if (match === null) return; //if  error just leave
    match = match.toString(); //even if a number is passed the option will still be a string, even if it is a number. So always compare strings.
    let optionNodes = document.getElementById(elem).options; //make a HTMLCollection
    let optionList = [];
    for (let i = 0; i < optionNodes.length; i++) optionList.push(optionNodes[i].value);
    let index = optionList.findIndex(element => element == match);
    if (index < 0) {
        console.log('error in convertSelectInputOption(), option not found', match);
        index = 0;
    }//value not found, use default
    document.getElementById(elem).selectedIndex = index;
    //console.log(optionNodes.length);
    //console.log(optionList);
    //console.log(index);
}

//========================================================================================================================================

function stringToHex(str) {
    let hex = '';
    for (let i = 0; i < str.length; i++) {
        hex += '' + str.charCodeAt(i).toString(16);
    }
    return hex;
}

//========================================================================================================================================

function decToHex(rgb) {
    var hex = Number(rgb).toString(16);
    if (hex.length < 2) {
        hex = "0" + hex;
    }
    return hex;
};

//========================================================================================================================================

function rgbToHex(red, green, blue) {
    return "#" + decToHex(red) + decToHex(green) + decToHex(blue);
}

//========================================================================================================================================

function csvToArray(csv) {
    let rows = csv.split("\n");

    //catches and removes the last line which is empty
    rows.forEach(function (value, index) {
        if (value == "") {
            rows.splice(index, 1);
        }
    });

    return rows.map(function (row) {
        return row.split("\t");
    });
}

//========================================================================================================================================

function hexToDec(hex) {
    return parseInt(hex, 16);
}

//========================================================================================================================================

function getRandomInt(max) {
    return Math.floor(Math.random() * Math.floor(max));
}

//========================================================================================================================================

function checkWithinRange(start, end, val) {
    //console.log("checkWithinRange("+start,end,val+")");
    return (val <= Math.max(start, end) && val >= Math.min(start, end));
}

function checkValidStartVals(startArr, endArr, val) {
    let flag = true;
    startArr.forEach(function (v, i) {
        if (!checkWithinRange(startArr[i], endArr[i], val[i])) flag = false;
    });
    //console.log("checkValidStartVals() is ", flag)
    return flag;
}

//========================================================================================================================================

function isArrayEqual(arr1, arr2) {
    if (arr1.length !== arr2.length)
        return false;
    for (var i = arr1.length; i--;) {
        if (arr1[i] !== arr2[i])
            return false;
    }
    return true;
}

//========================================================================================================================================

function systemResize8Array(arr, size) {
    let a = Array.from(arr); //convert to normal array first. So it can be resized.
    a.length = size; //added to make it expand also, does not affect usage
    return new Uint8Array(Array.from(a).slice(0, size));
}

//========================================================================================================================================
//take an array of 8-bit values and output array of converted 16-bit values arranged MSB first in a 8-bit wide element array
function systemResize16Array(arr, size) {
    let a = Array(size * 2).fill(0);
    let q = 0;
    for (let i = 0; i < a.length; i += 2) {
        let calc = (arr[q] * 256) + arr[q];
        a[i] = ((calc >> 8) & 0x00FF); //MSB
        a[i + 1] = (calc & 0x00FF); //LSB
        q++;
    }
    return new Uint8Array(Array.from(a).slice(0, (size * 2)));
}

//========================================================================================================================================

function systemVisClassShow(el) {
    document.getElementById(el).classList.add('visibility-visible');
    document.getElementById(el).classList.remove('visibility-hidden');
}

//========================================================================================================================================

function systemVisClassHide(el) {
    document.getElementById(el).classList.add('visibility-hidden');
    document.getElementById(el).classList.remove('visibility-visible');
}

//========================================================================================================================================

function systemClassBuilder(name, rules, flag = true) {
    let head = document.getElementsByTagName('head')[0].children;
    if (flag) head[head.length - 1].innerHTML = name + ' {' + rules + '}';
    else head[head.length - 1].innerHTML += name + ' {' + rules + '}';
    //could use addrule() insertRule().......
}

//========================================================================================================================================

//Applies the color values when the color picker is applied(not closed)
function handleColorPickerSelection(frame, pix, colorMode, mode, color) {
    console.log("handleColorPickerSelection() " + frame, pix, mode, colorMode, [...color]);

    let offset = pix * getColorModeMult(SelSeq.colorMode);
    let dataOffset = pix * getColorModeMult(SelSeq.colorMode);
    if (SelSeq.mode == 'mGradient') dataOffset = 0;

    switch (mode) {
        case 'paint1':
            systemUndoRedoPush('paint1');
            stateUX.paintManColor = [...color];
            uxToolsUpdatePaintRect(1, color);
            break;
        case 'paint2':
            systemUndoRedoPush('paint2');
            stateUX.paintFuncColor = [...color];
            uxToolsUpdatePaintRect(2, color);
            break;
        case 'pixel':
            offset = pix * getColorModeMult(SelSeq.colorMode); //update to correct offset
            systemUndoRedoPush('seq');
            helper(); //run once

            sequenceSetPixelData(SelSeq, 'frameVar', pix, frame); //update frameVar 
            sequenceSetStartValues(SelSeq, pix); //start value color is calculated based on indicator position, so offset stay the same

            //old way - start indicator jumped around annoyingly
            //sequenceSetPixelData(SelSeq, 'startVal', pix, frame, color);
            //sequenceLoadStartValPositions(SelSeq, pix);

            if (SelSeq.mode == 'mFade') startValHelper();
            else if (SelSeq.mode == 'mGradient') {
                pix = 0;  //run on all startVals
                for (let i = 0; i < SelSeq.numPixels; i++) {
                    offset = i * getColorModeMult(SelSeq.colorMode);
                    pix = i;
                    startValHelper();
                }
            }
            //if (SelSeq.mode == 'mGradient') //WTF! - removed v3.0.1
            layoutUpdateBuffers();
            break;
        case 'liveicon':
            let colorRect = document.getElementById('ovmenu-2-color');
            colorRect.innerText = (pix * getColorModeMult(colorMode)) + 1;
            offset = pix * getColorModeMult(colorMode);
            ovmLiveControlColorLabels(color);
            if (colorMode == 'Mono') {
                //stateUX.liveControlElem.style.backgroundColor = 'rgb(' + color[3] + ',' + color[3] + ',' + color[3] + ')';
                //colorRect.style.backgroundColor = 'rgb(' + parseInt(color[3]) + ',' + parseInt(color[3]) + ',' + parseInt(color[3]) + ')';
                stateCommunication.liveControlPacket[offset] = parseInt(color[3]);
            }
            else if (colorMode == 'RGB') {
                //stateUX.liveControlElem.style.backgroundColor = 'rgb(' + color[0] + ',' + color[1] + ',' + color[2] + ')';
                //colorRect.style.backgroundColor = 'rgb(' + parseInt(color[0]) + ',' + parseInt(color[1]) + ',' + parseInt(color[2]) + ')';
                stateCommunication.liveControlPacket[offset] = parseInt(color[0]);
                stateCommunication.liveControlPacket[offset + 1] = parseInt(color[1]);
                stateCommunication.liveControlPacket[offset + 2] = parseInt(color[2]);
            }
            else if (colorMode == "RGBW") {
                //stateUX.liveControlElem.style.backgroundColor = 'rgb(' + color[0] + ',' + color[1] + ',' + color[2] + ')';
                //colorRect.style.backgroundColor = 'rgb(' + parseInt(color[0]) + ',' + parseInt(color[1]) + ',' + parseInt(color[2]) + ')';
                //stateUX.liveControlElem.style.backgroundImage = "linear-gradient(180deg, transparent 60%," + rgbToHex(parseInt(color[3]), parseInt(color[3]), parseInt(color[3])) + " 40%)";
                stateCommunication.liveControlPacket[offset] = parseInt(color[0]);
                stateCommunication.liveControlPacket[offset + 1] = parseInt(color[1]);
                stateCommunication.liveControlPacket[offset + 2] = parseInt(color[2]);
                stateCommunication.liveControlPacket[offset + 3] = parseInt(color[3]);
            }
            stateCommunication.liveUpdatePending = true;
            ovmLiveControlRefresh();
            break;
        case 'framezone':
        case 'frameall':
            systemUndoRedoPush('seq');
            if (SelSeq.mode == 'mGradient') {
                pix = 0;
                for (let i = 0; i < SelSeq.numPixels; i++) {                 //run on all start valus
                    offset = i * getColorModeMult(SelSeq.colorMode);
                    pix = i;
                    helper();
                    startValHelper();
                }
            }
            else {
                dataOffset = 0;
                offset = 0;
                for (let i = 0; i < SelSeq.numPixels; i++) {
                    if (!mainObj.layout.advanced || mode == 'frameall' || stateUX.zoneView == 0 || mainObj.layout.ledIconZoneID[i] == stateUX.zoneView) {
                        dataOffset = i * getColorModeMult(SelSeq.colorMode);
                        offset = dataOffset;
                        pix = i;
                        helper();
                        startValHelper();
                    }

                    sequenceSetStartValues(SelSeq,i)
                } //end for()
            }

            
            layoutUpdateBuffers();
            break;
        case 'zoneall':
            systemUndoRedoPush('seq');
            frame = 0;
            for (let q = 0; q < SelSeq.frames; q++) {
                for (let i = 0; i < SelSeq.numPixels; i++) {
                    if (mainObj.layout.advanced && (stateUX.zoneView == 0 || mainObj.layout.ledIconZoneID[i] == stateUX.zoneView)) {
                        offset = i * getColorModeMult(SelSeq.colorMode);
                        if (SelSeq.mode != 'mGradient') dataOffset = offset;
                        pix = i;
                        helper();
                    }
                } //end i for()
                frame++;
            } //end q for()
            frame = 0;
            for (let i = 0; i < SelSeq.numPixels; i++) {
                if (mainObj.layout.advanced && (stateUX.zoneView == 0 || mainObj.layout.ledIconZoneID[i] == stateUX.zoneView)) {
                    pix = i;
                    startValHelper();
                }
            }
            layoutUpdateBuffers();
            break;
        case 'pixelall':
            systemUndoRedoPush('seq');
            if (SelSeq.mode == 'mGradient') {
                //IGNORE
                pushMessageToLog("Cannot paint all pixels in Gradient sequence mode"); //don't want this here or at all.....
            }
            else {
                frame = 0;
                for (let i = 0; i < SelSeq.frames; i++) {
                    helper();
                    frame++;
                } //end for()
                frame = 0;
                dataOffset = pix * getColorModeMult(SelSeq.colorMode);
                offset = dataOffset;
                startValHelper();
            }
            layoutUpdateBuffers();
            break;
        default:
            console.log("Color picker passed null mode");
            break;
    } //end switch

    //-------------------------------------------------------------------------------------
    function startValHelper() {
        if (!cSeqModesStartVals.includes(SelSeq.mode)) return; //ignore if no .startVals

        let frameArr = sequenceGetPixelData(SelSeq, 'frames', pix, 0); //get this first, need to reference the frameVar for pixel
        let startArr = sequenceGetPixelData(SelSeq, 'begin', pix, frameArr[0]); //it really couldn't be anything else than frame[0].....
        let endArr = sequenceGetPixelData(SelSeq, 'end', pix, frameArr[0]);     // but not how I would want to do it....
        let valArr = sequenceGetPixelData(SelSeq, 'start', pix, frameArr[0]);

        if (!checkValidStartVals(startArr, endArr, valArr)) {
            //only runs if current start vals are invalid
            SelSeq.frameVar[offset] = frame;
            SelSeq.startVals[offset] = SelSeq.dataFrames[frame][dataOffset];
            if (colorMode != 'Mono') {
                SelSeq.frameVar[offset + 1] = frame;
                SelSeq.startVals[offset + 1] = SelSeq.dataFrames[frame][dataOffset + 1];
                SelSeq.frameVar[offset + 2] = frame;
                SelSeq.startVals[offset + 2] = SelSeq.dataFrames[frame][dataOffset + 2];
                if (colorMode == 'RGBW') {
                    SelSeq.frameVar[offset + 3] = frame;
                    SelSeq.startVals[offset + 3] = SelSeq.dataFrames[frame][dataOffset + 3];
                }
            }
            sequenceLoadStartValPositions(SelSeq, pix); //update startValPos float based on new start values
        }
    }
    //-------------------------------------------------------------------------------------
    function helper() {
        console.log(frame, pix)
        if (colorMode == "Mono") {
            SelSeq.dataFrames[frame][dataOffset] = color[3];
        }
        else if (colorMode == "RGB") {
            SelSeq.dataFrames[frame][dataOffset] = color[0];
            SelSeq.dataFrames[frame][dataOffset + 1] = color[1];
            SelSeq.dataFrames[frame][dataOffset + 2] = color[2];
        }
        else if (colorMode == "RGBW") {
            SelSeq.dataFrames[frame][dataOffset] = color[0];
            SelSeq.dataFrames[frame][dataOffset + 1] = color[1];
            SelSeq.dataFrames[frame][dataOffset + 2] = color[2];
            SelSeq.dataFrames[frame][dataOffset + 3] = color[3];
        }
    }
    //-------------------------------------------------------------------------------------
}

//========================================================================================================================================

//loads the initial color values when the color picker is opened
function handleColorPickerSetInitial(mode, colorMode, frame, pix) {
    //console.log((mode, colorMode, frame, pix))
    pix = pix * getColorModeMult(colorMode); //update to correct offset
    switch (mode) {
        case 'paint1':
            colorPicker.red = stateUX.paintManColor[0];
            colorPicker.green = stateUX.paintManColor[1];
            colorPicker.blue = stateUX.paintManColor[2];
            colorPicker.white = stateUX.paintManColor[3];
            break;
        case 'paint2':
            colorPicker.red = stateUX.paintFuncColor[0];
            colorPicker.green = stateUX.paintFuncColor[1];
            colorPicker.blue = stateUX.paintFuncColor[2];
            colorPicker.white = stateUX.paintFuncColor[3];
            break;
        case 'pixel':
            if (SelSeq.mode == 'mGradient') pix = 0;

            if (colorMode == "Mono") {
                colorPicker.white = SelSeq.dataFrames[frame][pix];
            }
            else if (colorMode == "RGB") {
                colorPicker.red = SelSeq.dataFrames[frame][pix];
                colorPicker.green = SelSeq.dataFrames[frame][pix + 1];
                colorPicker.blue = SelSeq.dataFrames[frame][pix + 2];
            }
            else if (colorMode == "RGBW") {
                colorPicker.red = SelSeq.dataFrames[frame][pix];
                colorPicker.green = SelSeq.dataFrames[frame][pix + 1];
                colorPicker.blue = SelSeq.dataFrames[frame][pix + 2];
                colorPicker.white = SelSeq.dataFrames[frame][pix + 3];
            }
            break;
        case 'liveicon':
            if (colorMode == "Mono") {
                colorPicker.white = stateCommunication.liveControlPacket[pix];
            }
            else if (colorMode == "RGB") {
                colorPicker.red = stateCommunication.liveControlPacket[pix];
                colorPicker.green = stateCommunication.liveControlPacket[pix + 1];
                colorPicker.blue = stateCommunication.liveControlPacket[pix + 2];
            }
            else if (colorMode == "RGBW") {
                colorPicker.red = stateCommunication.liveControlPacket[pix];
                colorPicker.green = stateCommunication.liveControlPacket[pix + 1];
                colorPicker.blue = stateCommunication.liveControlPacket[pix + 2];
                colorPicker.white = stateCommunication.liveControlPacket[pix + 3];
            }
            break;
        case 'frameall':

            break;
        case 'pixelall':

            break;
        default:
            console.log("Color picker passed null mode");
            break;
    }
}

//========================================================================================================================================

function handleColorPickerMessages(msgDiv) {
    let pix = (colorPicker.pix + 1); //base 1 label
    switch (colorPicker.mode) {
        case 'pixel':
            msgDiv.innerText = "Paint Pixel #" + pix + ", Frame #" + (colorPicker.frame + 1);
            break;
        case 'liveicon':
            msgDiv.innerText = "Live Control: Pixel #" + pix;
            break;
        case 'frameall':
            msgDiv.innerText = "Paint All Pixels in Frame " + (colorPicker.frame + 1);
            break;
        case 'pixelall':
            msgDiv.innerText = "Paint Pixel #" + pix + " in all frames.";
            break;
    }
}

//========================================================================================================================================

function uxTabLoadDefaultGamma(color) {
    console.log("uxTabLoadDefaultGamma() for " + color);
    gammaTables[color] = []; //makes .length 0, so if refresh runs it will reset to the default there
    ovmGammaRender();
}

//========================================================================================================================================

function convertRGBtoRGBW(seq) {
    console.log("convertRGBtoRGBW()");
    let numPixels = ((seq.mode == 'mGradient') ? 1 : seq.numPixels);

    for (let i = 0; i < seq.frames; i++) {
        let scratchData = []; //reset for every frame

        for (let pix = 0; pix < numPixels; pix++) {
            let dataArr = Array.from(sequenceGetPixelData(seq, 'begin', pix, i)); //returns RGB values for starting frame
            let valArr = []; //reset for every pixel
            //console.log("Running ", i, pix, dataArr);
            if (dataArr[0] == dataArr[1] && dataArr[1] == dataArr[2]) {
                //pix data for frame is white in RGB
                //console.log("white " + pix);
                valArr.push(0);
                valArr.push(0);
                valArr.push(0);
                valArr.push(dataArr[0]); //chose any RGB value, all the same
                scratchData.push(...valArr);
            }
            else {
                //console.log("rgb " + pix);
                dataArr.push(0); // set W channel to 0
                scratchData.push(...dataArr);
            }
        } //end pix for()
        seq.dataFrames[i] = Uint8Array.from(scratchData);
    } //end i for()
}

//========================================================================================================================================

function convertRGBWtoRGB(seq) {
    console.log("convertRGBtoRGBW()");
    let numPixels = ((seq.mode == 'mGradient') ? 1 : seq.numPixels);
    for (let i = 0; i < seq.frames; i++) {
        let scratchData = []; //reset for every frame
        for (let pix = 0; pix < numPixels; pix++) {
            let dataArr = Array.from(sequenceGetPixelData(seq, 'begin', pix, i)); //returns RGBW values for starting frame
            //console.log("Running ", i, pix, dataArr);
            dataArr.splice(-1); //remove last element, which is white
            //console.log(dataArr);
            scratchData.push(...dataArr); //push the RGB data
        } //end pix for()
        seq.dataFrames[i] = Uint8Array.from(scratchData);
    } //end i for()
}

  //========================================================================================================================================
