'use strict';

/*
Layout:


    Load File -> Reset -> If Advanced(false) -> build shape, render into container
                          If Advanced(true) ->

                

    Initialize: resize arrays[iconAmount], runs position
    Position: Normal coordinate fill, defined coordinate fill

    When changed: layoutShape, iconAmount(for normal shapes), CCW/CW
        -> Intialize -> Position -> Render
    When changed: iconAmount(for user defined shapes)
        -> Intialize with persistance -> Position -> Render
    When changed: Icon Size, iconShape
        -> Render
    When color is changed:
        -> Update	

    Normal: round/square/spiral/line - coordinates generated by the function, can be re-rendered anytime
    Normal, Defined: Map - coordinates stored with sequence data
    Advanced: manual, 
        CHANGE OR DEFINE: each render, does not clear the coordinates, just redraws them...
*/

//========================================================================================================================================

function renderGraphicLayout() {
    console.log("renderGraphicLayout()");

    stateUX.refreshGraphic = false; //clear UX state notification
    let theCont = document.getElementById("ws-graphic-seqmessage");

    //---------------------- Perfrom render action based on sequence mode ---------------------------------------

    if (mainObj.layout.imageEnabled && mainObj.layout.image != undefined) {
        //a graphic layout image is defined. Check if enabled
        layoutPositionImageFunc(mainObj.layout.image.filePath);
    }

    switch (SelSeq.mode) {
        case 'mFade':
        case 'mGradient':
        case 'mInstant':
        case 'mScene':
        case 'mFile':
        case 'mPOV':
            if (SelSeq.mode == 'mPOV') theCont.innerHTML = "<p>&nbsp;&nbsp; Use the Timeline tab to load files and adjust settings.</p>";
            else {
                if (mainObj.layout.advanced) uxZoneViewLabel(); //clear any potential messages from other sequence modes
                else theCont.innerHTML = "";
            }
            layoutGraphicIconsUpdate(); //just update what is there
            break;
        case 'mList':
            theCont.innerHTML = "<p>&nbsp;&nbsp; List Mode - All settings can be found on the Timeline Tab</p>";
            layoutGraphicIconsUpdate(); //just update what is there
            break;
        case 'mTechnical':
            theCont.innerHTML = "<p>&nbsp;&nbsp; Not Yet Implemented. Use a different Mode</p>";
            break;
        default:
            console.log("Sequence mode did not match renderer: " + SelSeq.mode);
            theCont.innerHTML = "<p>&nbsp;&nbsp; An error occured, the sequence mode is invalid.</p>";
            break;
    } //end switch

    //-------------------------------------------------------------

    //find out if it should show the pixels or other menu(list sequence mode)
    //find out what shape or patch
    //generate with parameters
    //place graphic layout image with sizing

    //-------------------------------------------------------------
}

//========================================================================================================================================

function layoutGraphicResetPosition() {
    console.log("layoutGraphicResetPosition()");
    if (mainObj.layout.advanced && !stateUX.layoutAdvancedPreview) return; //just in case but element would be disabled
    stateUX.shapeCenterEnable = true;
    layoutGraphicIconReset();
    layoutGraphicBuildIcons();
}

//========================================================================================================================================

function layoutGraphicRotation() {
    //console.log("layoutGraphicRotation() ", angle);
    if (mainObj.layout.advanced && !stateUX.layoutAdvancedPreview) return; //just in case but element would be disabled
    mainObj.layout.rotation = document.getElementById('toolmenu-ledicon-rotation').value; //update state
    uxToolsLayoutUpdate(); //update rotation elements with new value
    layoutGraphicBuildIcons(); //now build/preview
}

//========================================================================================================================================

function layoutGraphicIconReset() {
    console.log("layoutGraphicIconReset()");
    //console.log("RESETTING ICONS");
    mainObj.layout.ledIconCoordX = [];
    mainObj.layout.ledIconCoordY = [];
    mainObj.layout.ledIconZoneID = [];
    document.getElementById("ws-graphic-layout-container").innerHTML = ''; //remove all pixel icons - do it here?
}

//========================================================================================================================================

function layoutShapeCenterDragStart() {
    console.log("layoutShapeCenterDragStart()")
    document.addEventListener('mouseup', layoutShapeCenterDragStop, { once: true });
    document.getElementById("ws-graphic-layout-container").addEventListener('mousemove', layoutShapeCenterDragging);
}

function layoutShapeCenterDragging() {
    console.log("layoutShapeCenterDragging() " + event)
    //throttle mousemove speed
    stateUX.mouseTimer = setTimeout(function () {
        document.getElementById("ws-graphic-layout-container").addEventListener("mousemove", layoutShapeCenterDragging)
    }, 50);
    document.getElementById("ws-graphic-layout-container").removeEventListener("mousemove", layoutShapeCenterDragging);

    //-----------------------------------------------------------
    mainObj.layout.centerX = event.clientX - cWSOffsetX;
    if (mainObj.layout.centerX < 0) mainObj.layout.centerX = 1; //set to one so it doesn't trip the centering code
    mainObj.layout.centerY = event.clientY - cWSOffsetY;
    if (mainObj.layout.centerY < 0) mainObj.layout.centerY = 1;
    layoutGraphicBuildIcons();
}

function layoutShapeCenterDragStop() {
    console.log("layoutShapeCenterDragStop()");
    clearTimeout(stateUX.mouseTimer); //prevent mousemove throttling from triggering after release
    //document.removeEventListener('mouseup', layoutShapeCenterDragStop);
    document.getElementById("ws-graphic-layout-container").removeEventListener('mousemove', layoutShapeCenterDragging);
}

function layoutShapeCenterDragEnd() {
    if (stateUX.layoutAdvancedPreview) return;
    console.log("layoutShapeCenterDragEnd()");
    stateUX.shapeCenterEnable = false;
    layoutShapeCenterDragStop(); //run a few things, so code is not duplicated

    if (document.getElementById('shape-center-handle') != null) document.getElementById('shape-center-handle').remove();
    //document.getElementById('shape-center-handle').remove();
}

//========================================================================================================================================

function layoutGraphicBuildIcons() {
    //console.log("layoutGraphicBuildIcons()");

    const cWSPadding = 10;
    let iconSize = mainObj.layout.iconSize;
    let iconDensity = mainObj.layout.iconDensity;
    let iconAmount = mainObj.layout.iconAmount;
    let layoutShape = mainObj.layout.shape;
    let layoutDir = mainObj.layout.shapeDir;

    let theWorkspace = document.getElementById("ws-graphic-container");

    //let cCenterX = (theWorkspace.clientWidth / 2);
    //let cCenterY = (theWorkspace.clientHeight / 2);
    if (mainObj.layout.centerX == 0 && mainObj.layout.centerY == 0) {
        //indicates new placement
        mainObj.layout.centerX = (theWorkspace.clientWidth / 2); //not .scrollW or .scrollH
        mainObj.layout.centerY = (theWorkspace.clientHeight / 2);
        if (theWorkspace.clientHeight < 620) mainObj.layout.centerY = (620 / 2); //TODO STATIC BUG FIX!!!! Without any children theWorkspace is only 0 high until a redraw/repaint. Only affects  start of software
    }

    let cCenterX = mainObj.layout.centerX;
    let cCenterY = mainObj.layout.centerY;

    //mainObj.layout.centerX
    //mainObj.layout.centerY
    //console.log(theWorkspace.clientWidth,theWorkspace.clientHeight,cCenterX,cCenterY);
    //console.log(theWorkspace.offsetWidth, theWorkspace.offsetHeight)

    //------------------
    if (stateUX.shapeCenterEnable) {
        //drag handle
        if (document.getElementById('shape-center-handle') == null) {
            let theHandle = document.createElement('div');
            theHandle.classList = 'layout-image-handle buttonIcon';
            theHandle.id = 'shape-center-handle';
            theHandle.style.transform = "translate(-50%, -50%)";
            theHandle.style.cursor = 'move';

            theHandle.addEventListener('mousedown', layoutShapeCenterDragStart);
            document.getElementById("ws-graphic-layout-container").appendChild(theHandle);
        }
        document.getElementById('shape-center-handle').style.left = mainObj.layout.centerX + 'px';
        document.getElementById('shape-center-handle').style.top = mainObj.layout.centerY + 'px';
    }
    //-----------------------------------------------------------------------------------------------------------------------------

    let zoneID = parseInt(document.getElementById("toolmenu-ledicon-zoneval").value);
    if (!mainObj.layout.advanced) zoneID = 1; // if not in advanced layout mode, always 1

    let localX = [];
    let localY = [];
    let localZone = [];

    //-----------------------------------------------------------------------------------------------------------------------------

    if (mainObj.layout.advanced) {
        iconAmount = parseInt(document.getElementById("toolmenu-ledicon-advnumofpixels").value); //Advanced Icons - no state variable
        if (stateUX.layoutAdvancedPreview) {
            //console.log("ADVANCED WITH PREVIEW IS RUNNING " + layoutShape);
            helper(); //generate X/Y position coordinates for shape

            layoutSetZoneView(0); //show all zolnes
            layoutGraphicIconsUpdate(); //apply zone view
            layoutGraphicIconsPreviewRender(iconAmount, localX, localY);
        }
        else {
            //console.log("NORMAL ADVANCED IS RUNNING " + layoutShape);
            helper(); //generate position coordinates for shape
            mainObj.layout.ledIconCoordX.push(...localX);
            mainObj.layout.ledIconCoordY.push(...localY);
            mainObj.layout.ledIconZoneID.push(...localZone);

            mainObj.layout.iconAmount = mainObj.layout.ledIconCoordX.length; //in advanced mode the icon amount is automatically incremented, but the user can also edit it
            layoutSetZoneView(zoneID); //set the view to the zone the shape was just placed into
            uxToolsLayoutUpdate(); //UX?
            layoutGraphicIconsRender();
        }
    }
    else {
        //console.log("STANDARD SHAPE RUNNING " + layoutShape, layoutDir);
        helper(); //generate position coordinates for shape
        mainObj.layout.ledIconCoordX = [...localX];
        mainObj.layout.ledIconCoordY = [...localY];
        mainObj.layout.ledIconZoneID = [...localZone];
        layoutGraphicIconsRender();
    }

    //-------------------------------------- If centering, do this last to keep the shape on workspace-------------------------------------------------------------------

    if (stateUX.shapeCenterEnable) {
        document.getElementById('shape-center-handle').style.left = mainObj.layout.centerX + 'px';
        document.getElementById('shape-center-handle').style.top = mainObj.layout.centerY + 'px';
    }

    //-----------------------------------------------------------------------------------------------------------------------------

    function helper() {
        let radius, x, y, shapeSize, count, tempCX, tempCY; // multiple cases can't define the same variable name apparently
        switch (layoutShape) {
            default:
                //console.log("ERROR " + layoutshape);
                break;
            case 'shapeRound':
                let degreeChange = 360 / iconAmount;
                let angle = 360;
                radius = 4 * iconDensity; //4 was just chosen. Close to the size of other shapes
                //radius + padding should be the centerX and Y

                tempCX = cWSPadding + (iconSize / 2) + radius;
                tempCY = cWSPadding + (iconSize / 2) + radius;

                if (tempCX < cCenterX) tempCX = cCenterX;
                if (tempCY < cCenterY) tempCY = cCenterY;

                mainObj.layout.centerX = tempCX; //force the shape on workspace, and update the center handle position
                mainObj.layout.centerY = tempCY;

                for (let i = 0; i != iconAmount; i++) {
                    x = Math.round(tempCX + (radius * Math.sin(angle * Math.PI / 180)));
                    y = Math.round(tempCY + (radius * -Math.cos(angle * Math.PI / 180)));

                    if (layoutDir == "cw") angle += degreeChange;
                    else angle -= degreeChange;

                    //-------------------------------------------------

                    localX.push(x);
                    localY.push(y);
                    localZone.push(zoneID);

                    //-------------------------------------------------
                }
                break;
            case 'shapeSpiral':
                radius = 1000; //picked it, seems to work fine
                let coils = iconDensity;
                let rotation = 0;
                let thetaMax = coils * 2 * Math.PI; // How far to step away from center for each side.
                let awayStep = radius / thetaMax;   // distance between points to plot
                let chord = iconSize + (iconSize / 4);

                //----------------------------- get the diameter, no idea how, so do it clunky way -----------------------------

                let minX = 0;
                let minY = 0; //set to 0

                count = 0; //pixel counter
                for (let theta = chord / awayStep; theta <= thetaMax;) {
                    if (count == iconAmount) break; //but why?
                    count++;
                    let away = awayStep * theta; // How far away from center
                    let around = theta + rotation; // How far around the center.
                    x = Math.round(0 + Math.cos(around) * away); // to a first approximation, the points are on a circle
                    y = Math.round(0 + Math.sin(around) * away); // so the angle between them is chord/radius
                    theta += chord / away;
                    if (x < minX) minX = x;
                    if (y < minY) minY = y;

                    //-------------------------------------------------

                    localX.push(x);
                    localY.push(y);
                    localZone.push(zoneID);

                    //-------------------------------------------------
                } //end for()

                if (minX < 0) minX *= -1;
                if (minY < 0) minY *= -1;
                //console.log(maxX, maxY, minX, minY);

                tempCX = cWSPadding + (iconSize / 2) + minX;
                tempCY = cWSPadding + (iconSize / 2) + minY;

                if (tempCX < cCenterX) tempCX = cCenterX;
                if (tempCY < cCenterY) tempCY = cCenterY;
                //mainObj.layout.centerX = tempCX; //force the shape on workspace, and update the center handle position
                //mainObj.layout.centerY = tempCY;
                tempCX = mainObj.layout.centerX; //enables placement anywhere
                tempCY = mainObj.layout.centerY;

                //------------------ Apply center position to coordinates -----------------------------

                for (let i = 0; i < iconAmount; i++) {
                    localX[i] += tempCX;
                    localY[i] += tempCY;
                }

                //------------------ Apply CCW direction if required  -----------------------------

                if (layoutDir == "cw") {
                    //nothing needs to happen
                }
                else {
                    localX.reverse(); //easy
                    localY.reverse();
                    localZone.reverse();
                }
                break;
            case 'shapeSquare':
                shapeSize = (iconAmount / 4) * iconDensity;
                //------------CENTERING------------
                tempCX = cWSPadding + (iconSize / 2) + (shapeSize / 2);
                tempCY = cWSPadding + (iconSize / 2) + (shapeSize / 2);

                if (tempCX < cCenterX) tempCX = cCenterX;
                if (tempCY < cCenterY) tempCY = cCenterY;
                mainObj.layout.centerX = tempCX; //force the shape on workspace, and update the center handle position
                mainObj.layout.centerY = tempCY;

                count = 0;
                let i = 0;
                //------------BOTTOM------------
                for (i = 0; i < (iconAmount / 4); i++) {
                    x = ((tempCX - (shapeSize / 2)) + (iconDensity / 2) + (iconDensity * i));
                    y = tempCY + (shapeSize / 2);
                    localX.push(x);
                    localY.push(y);
                    localZone.push(zoneID);
                    count++;
                }
                //------------RIGHT------------
                for (i = 0; i < (iconAmount / 4); i++) {
                    x = (tempCX + (shapeSize / 2));
                    y = (tempCY + (shapeSize / 2)) - (iconDensity / 2) - (iconDensity * i);
                    localX.push(x);
                    localY.push(y);
                    localZone.push(zoneID);
                    count++;
                }
                //------------TOP------------
                for (i = 0; i < (iconAmount / 4); i++) {
                    x = ((tempCX + (shapeSize / 2)) - (iconDensity / 2) - (iconDensity * i));
                    y = (tempCY - (shapeSize / 2));
                    localX.push(x);
                    localY.push(y);
                    localZone.push(zoneID);
                    count++;
                }
                //------------LEFT------------
                for (i = 0; i < (iconAmount / 4); i++) {
                    x = tempCX - (shapeSize / 2);
                    y = ((tempCY - (shapeSize / 2)) + (iconDensity / 2) + (iconDensity * i));
                    localX.push(x);
                    localY.push(y);
                    localZone.push(zoneID);
                    count++;
                }

                if (layoutDir == "cw") {
                    localX = localX.reverse()
                    localY = localY.reverse()
                    localZone = localZone.reverse()
                }

                break;
            // case 'shapeLineVert':
            //     shapeSize = iconAmount * iconDensity;
            //     //------------CENTERING------------
            //     tempCX = cWSPadding + (iconSize / 2);
            //     tempCY = cWSPadding + (shapeSize / 2);

            //     if (tempCX < cCenterX) tempCX = cCenterX;
            //     if (tempCY < cCenterY) tempCY = cCenterY;         //no y required, use default
            //     mainObj.layout.centerX = tempCX; //force the shape on workspace, and update the center handle position
            //     mainObj.layout.centerY = tempCY;

            //     for (let i = 0; i < iconAmount; i++) {
            //         if (layoutDir == "cw") y = (tempCY - (shapeSize / 2)) + (iconDensity / 2) + (iconDensity * i);
            //         else y = (tempCY + (shapeSize / 2)) - (iconDensity / 2) - (iconDensity * i);
            //         x = tempCX;
            //         localX.push(x);
            //         localY.push(y);
            //         localZone.push(zoneID);
            //     }
            //     break;
            case 'shapeLine':
                shapeSize = iconAmount * iconDensity;
                //------------CENTERING------------
                tempCX = cWSPadding + (shapeSize / 2);
                tempCY = cWSPadding + (iconSize / 2);

                if (tempCX < cCenterX) tempCX = cCenterX;
                if (tempCY < cCenterY) tempCY = cCenterY;         //no y required, use default
                // mainObj.layout.centerX = tempCX; //force the shape on workspace, and update the center handle position
                // mainObj.layout.centerY = tempCY;

                tempCX = mainObj.layout.centerX; //enables placement anywhere
                tempCY = mainObj.layout.centerY;

                for (let i = 0; i < iconAmount; i++) {
                    if (layoutDir == "cw") x = (tempCX - (shapeSize / 2)) + (iconDensity / 2) + (iconDensity * i);
                    else x = (tempCX + (shapeSize / 2)) - (iconDensity / 2) - (iconDensity * i);
                    y = tempCY;
                    localX.push(x);
                    localY.push(y);
                    localZone.push(zoneID);
                }
                break;
            case 'shapeMap':
                let offsetX = 0
                let offsetY = 0;

                if (mainObj.layout.patchCoords != undefined) {
                    for (let i = 0; i < iconAmount; i++) {
                        let coords = mainObj.layout.patchCoords[i].split(",");
                        if (parseInt(coords[0]) > offsetX) offsetX = coords[0];
                        if (parseInt(coords[1]) > offsetY) offsetY = coords[1];
                        x = (parseInt(coords[0] - 1) * iconDensity) + (iconSize / 2);
                        y = (parseInt(coords[1] - 1) * iconDensity) + (iconSize / 2);
                        localX.push(x);
                        localY.push(y);
                        localZone.push(zoneID);
                    }

                    let holdOffX = offsetX;
                    let holdOffY = offsetY;

                    //----------------- NOW CENTER OR PLACE INTO UPPER LEFT CORNER -------------------

                    offsetX = ((cCenterX * 2) - (offsetX * iconDensity)) / 2; //fixes the bug with clientHeight being 0 the first run
                    offsetY = ((cCenterY * 2) - (offsetY * iconDensity)) / 2; //tested and adjusted centerY for 0 height

                    if (offsetX < 0) mainObj.layout.centerX = (holdOffX * iconDensity) / 2; //keeps shape on the workspace, and handle centered
                    if (offsetY < 0) mainObj.layout.centerY = (holdOffY * iconDensity) / 2;

                    for (let i = 0; i < localX.length; i++) {
                        if (offsetX > 0) localX[i] += offsetX;
                        else localX[i] += cWSPadding;
                    }

                    for (let i = 0; i < localY.length; i++) {
                        if (offsetY > 0) localY[i] += offsetY;
                        else localY[i] += cWSPadding;
                    }
                } //end undefined check
                break;
            case 'shapeVector':
                //Way off in the future. Wanted to use SVGs, but not sure how to access the individual elements or convert SVG to DIVs
                break;
        } //end switch


        //--------------------------------------------- Apply rotation ----------------------------------------------------------
        mainObj.layout.rotation
        let angle = parseInt(mainObj.layout.rotation);
        let rad = (angle * Math.PI) / 180;
        //console.log(angle, rad);

        for (let i = 0; i < localX.length; i++) {
            let x = localX[i] - mainObj.layout.centerX;
            let y = localY[i] - mainObj.layout.centerY;

            let newX = x * Math.cos(rad) - y * Math.sin(rad);
            let newY = y * Math.cos(rad) + x * Math.sin(rad);
            localX[i] = newX + mainObj.layout.centerX;
            localY[i] = newY + mainObj.layout.centerY;
        }

        //-----------------------------------------------------
    } //end helper
}

//========================================================================================================================================

function layoutGraphicIconsPreviewRender(amount, xCoords, yCoords) {
    console.log("layoutGraphicIconsPreviewRender() " + amount);
    //----------------------------------------------------------------------------------- 

    let newClass = 'width: ' + mainObj.layout.iconSize + 'px;height: ' + mainObj.layout.iconSize + 'px;';
    if (mainObj.layout.iconShape == "round") newClass += 'border-radius:' + mainObj.layout.iconSize + 'px;';
    else newClass += 'border-radius: 0px;';
    newClass += 'font-size:' + (mainObj.layout.iconSize / 2) + 'px;line-height:' + mainObj.layout.iconSize + 'px;';
    newClass += 'border-color: red;pointer-events: none;'
    systemClassBuilder('.pxPrevSz', newClass, true); //true = clear the classes, and place new

    //change the class properties for the current pixels. all Displayed via zoneview #0
    newClass = 'width: ' + mainObj.layout.iconSize + 'px;height: ' + mainObj.layout.iconSize + 'px;';
    if (mainObj.layout.iconShape == "round") newClass += 'border-radius:' + mainObj.layout.iconSize + 'px;';
    else newClass += 'border-radius: 0px;';
    newClass += 'font-size:' + (mainObj.layout.iconSize / 2) + 'px;line-height:' + mainObj.layout.iconSize + 'px;';
    newClass += 'background-color: grey !important;opacity: 0.5;pointer-events: none;'
    systemClassBuilder('.pxSz', newClass, false); //false = add another class
    //will also allow resize of the already placed coordinates, but that is by design..... I guess

    //---------------------------------------------------------------------------------------------

    let theCont = document.getElementById("ws-graphic-layout-container");
    // theCont.innerHTML = ''; //clear all icons - no longer cleared
    //Remove only the preview icons then render the new ones
    let clearPrevIcons = theCont.querySelectorAll(".pxPrevSz");
    if (clearPrevIcons != null) { //will be null first time it runs
        for (let i = 0; i < clearPrevIcons.length; i++) clearPrevIcons[i].remove();
    }

    //---------------------------------------------------------------------------------------------

    for (let i = 0; i != amount; i++) {
        if (xCoords[i] != undefined && yCoords[i] != undefined) {
            let theIcon = document.createElement('div');
            theIcon.classList = "ws-graphic-icon pxPrevSz";
            theIcon.innerText = (i + 1);
            theIcon.style.left = Math.round(xCoords[i]) + 'px';
            theIcon.style.top = Math.round(yCoords[i]) + 'px';
            theCont.appendChild(theIcon);
        }
    }
}

//========================================================================================================================================

function layoutGraphicIconClass() {
    //console.log("layoutGraphicIconClass()");
    let newClass = 'width: ' + mainObj.layout.iconSize + 'px;height: ' + mainObj.layout.iconSize + 'px;';
    if (mainObj.layout.iconShape == "round") newClass += 'border-radius:' + mainObj.layout.iconSize + 'px;';
    else newClass += 'border-radius: 0px;';
    newClass += 'font-size:' + (mainObj.layout.iconSize / 2) + 'px;line-height:' + mainObj.layout.iconSize + 'px;';
    systemClassBuilder('.pxSz', newClass, true); //true = overwrites
}

//========================================================================================================================================

function layoutGraphicIconsRender() {
    //console.log("layoutGraphicIconsRender() ");
    //-------------------------------------------------------------

    layoutGraphicIconClass(); //Update the class to match current settings

    //---------------------------------------------------------------------------------------------

    let theCont = document.getElementById("ws-graphic-layout-container");
    //theCont.innerHTML = ""; //can't or will remove center handle too

    let theIcons = document.getElementById("ws-graphic-layout-container").querySelectorAll("div.pxSz");
    theIcons.forEach(function (val, index) {
        theIcons[index].remove();
    });

    //-----------------------------------------------------------------------------------------------

    for (let i = 0; i < mainObj.layout.iconAmount; i++) {
        if (mainObj.layout.ledIconCoordX[i] != undefined && mainObj.layout.ledIconCoordY[i] != undefined) {
            let theIcon = document.createElement('div');
            theIcon.classList = "ws-graphic-icon pxSz";
            theIcon.innerText = (i + 1);

            if (mainObj.layout.ledIconCoordX[i] == 'deleted') {
                theIcon.style.left = '20px'; //STATIC
                theIcon.style.top = '20px'; //STATIC
                theIcon.style.visibility = 'hidden'; //guess this is fine
            }
            else {
                theIcon.style.left = mainObj.layout.ledIconCoordX[i] + 'px';
                theIcon.style.top = mainObj.layout.ledIconCoordY[i] + 'px';
            }

            if (stateUX.layoutManualEnable) { //ugly but works
                theIcon.style.cursor = 'grab';
                theIcon.addEventListener('mousedown', function () { selectManualIcon(this, i) });
                if (mainObj.layout.ledIconCoordX[i] == 'deleted') {
                    theIcon.style.visibility = 'hidden';
                }
            }
            else {
                theIcon.addEventListener('click', function () { selectGraphicPixel(i) });
            }
            uxToolTipAttach(theIcon, (i + 1));
            theCont.appendChild(theIcon);
        }
    }

    //------------------------------------------------------------

    layoutGraphicIconsUpdate(); //run last
}

//========================================================================================================================================

function layoutGraphicIconsUpdate() {
    //console.log("layoutGraphicIconsUpdate()");

    //------------------------------------------------------------

    let theCont = document.getElementById("ws-graphic-layout-container");
    let theIcons = theCont.querySelectorAll("div.pxSz")
    //console.log(theIcons);

    //------------------------------------------------------------

    if (theIcons.length > stateSoftware.maxPixels) {
        console.error("Too many pixels are defined, a major error occured.")
        return;
    }

    //------------------------------------------------------------

    if (theIcons.length > 0) {

        // for (let i = 0; i < theIcons.length; i++)theIcons[i].style.removeProperty('border-color');

        for (let i = 0; i < theIcons.length; i++) {

            theIcons[i].style.removeProperty('border-color');
            theIcons[i].style.removeProperty('background-image');
            theIcons[i].style.removeProperty('border');
            


            if (mainObj.layout.advanced) {
                if (stateUX.zoneView == mainObj.layout.ledIconZoneID[i] || stateUX.zoneView == 0) {
                    //In currently viewed zone, show it
                    theIcons[i].style.removeProperty('display'); //show pixels, they mave been hidden/display:none
                    if (theIcons[i].style.backgroundImage != undefined) theIcons[i].style.removeProperty('background-image'); //theIcons[i].style.backgroundImage = 'unset'; //this would remove the backgroundImage used for RGBW

                    if (i >= SelSeq.numPixels) {
                        theIcons[i].classList.add('disabled');
                    }
                    else {
                        theIcons[i].classList.remove('disabled'); //just in case it was set
                        getColorHelper(theIcons[i], i);
                    }
                }
                else {
                    //Not in the zone view, hide it.
                    theIcons[i].style.display = 'none'; //hide pixels not in zone view
                }
            }
            else {
                //not advanced layout. Check if within number of pixels range
                if (i >= SelSeq.numPixels) {
                    theIcons[i].classList.add('disabled');
                }
                else {
                    theIcons[i].classList.remove('disabled'); //just in case it was set
                    getColorHelper(theIcons[i], i);
                }
            }
        }
    }

    //------------------------------------------------------------

    function getColorHelper(elem, num) {
        let mult, index, color, white;

        if (cSeqModesStartVals.includes(SelSeq.mode)) {
            //check for preload
            let frame = SelSeq.frameVar[num * getColorModeMult(SelSeq.colorMode)]; //all the same for each icon
            let startArr = sequenceGetPixelData(SelSeq, 'begin', num, frame);
            let endArr = sequenceGetPixelData(SelSeq, 'end', num, frame);

            if (SelSeq.mode != 'mGradient') {
                setPixelElemColor(elem, num, stateSeq.selectedFrame);
                if (isArrayEqual(startArr, endArr)) {
                    //preload
                    mult = getColorModeMult(SelSeq.colorMode);
                    index = num * mult;
                    let frameData = SelSeq.dataFrames;
                    switch (mult) {
                        case 1: //Mono
                            elem.style.borderColor = rgbToHex(frameData[SelSeq.frameVar[index]][index], frameData[SelSeq.frameVar[index]][index], frameData[SelSeq.frameVar[index]][index]);
                            //    elem.style.borderColor = "transparent";
                            break;
                        case 4: //RGBW
                            //colorWht = rgbToHex(SelSeq.startVals[index + 3], SelSeq.startVals[index + 3], SelSeq.startVals[index + 3]);
                            color = rgbToHex(frameData[SelSeq.frameVar[index]][index], frameData[SelSeq.frameVar[index + 1]][index + 1], frameData[SelSeq.frameVar[index + 2]][index + 2]);
                            white = rgbToHex(frameData[SelSeq.frameVar[index + 3]][index + 3], frameData[SelSeq.frameVar[index + 3]][index + 3], frameData[SelSeq.frameVar[index + 3]][index + 3]);
                            elem.style.borderColor = color + " " + color + " " + white + " " + color;
                            elem.style.borderBottomWidth = "4px";
                            break;
                        case 3: //RGB
                            elem.style.borderColor = rgbToHex(frameData[SelSeq.frameVar[index]][index], frameData[SelSeq.frameVar[index + 1]][index + 1], frameData[SelSeq.frameVar[index + 2]][index + 2]);
                            break;
                    }
                }
                else setStartValIndicatorColor(elem, num);
            }
            else {
                if (isArrayEqual(startArr, endArr)) {
                    //preload
                    setPixelElemColor(elem, 0, frame);
                }
                else setStartValIconColor(elem, num);
            }
        }
        else if (SelSeq.mode == 'mInstant') {
            getFrameSolid(elem, num, stateSeq.selectedFrame);
            mult = getColorModeMult(SelSeq.colorMode);
            index = num * mult;
            let frameData = SelSeq.dataFrames;
            switch (mult) {
                case 1: //Mono
                    elem.style.borderColor = rgbToHex(frameData[SelSeq.frameVar[index]][index], frameData[SelSeq.frameVar[index]][index], frameData[SelSeq.frameVar[index]][index]);
                    //    elem.style.borderColor = "transparent";
                    break;
                case 4: //RGBW
                    //colorWht = rgbToHex(SelSeq.startVals[index + 3], SelSeq.startVals[index + 3], SelSeq.startVals[index + 3]);
                    color = rgbToHex(frameData[SelSeq.frameVar[index]][index], frameData[SelSeq.frameVar[index + 1]][index + 1], frameData[SelSeq.frameVar[index + 2]][index + 2]);
                    white = rgbToHex(frameData[SelSeq.frameVar[index + 3]][index + 3], frameData[SelSeq.frameVar[index + 3]][index + 3], frameData[SelSeq.frameVar[index + 3]][index + 3]);
                    elem.style.borderColor = color + " " + color + " " + white + " " + color;
                    elem.style.borderBottomWidth = "4px";
                    break;
                case 3: //RGB
                    elem.style.borderColor = rgbToHex(frameData[SelSeq.frameVar[index]][index], frameData[SelSeq.frameVar[index + 1]][index + 1], frameData[SelSeq.frameVar[index + 2]][index + 2]);
                    break;
            }
        }
        else if (cDataSeqModeGlobal.includes(SelSeq.mode)) {
            // console.log("RAN4");
            getFrameSolid(elem, num, stateSeq.selectedFrame);
        }
    }

    //------------------------------------------------------------
}

//========================================================================================================================================

function layoutImageReset() {
    console.log("layoutImageReset()");
    let thePath = mainObj.layout.image.filePath; //store
    systemUndoRedoPush('layout');
    mainObj.layout.image.filePath = ""; //set to reset with the same filepath
    layoutPositionImageFunc(thePath);
}

//========================================================================================================================================

function layoutImageRemove() {
    console.log("layoutImageRemove()");
    let theCont = document.getElementById("ws-graphic-image-container");
    theCont.innerHTML = '';
    theCont.style.removeProperty('background-image');   //backgroundImage = 'unset';
}

//========================================================================================================================================

function layoutImagePosition(flag) {
    console.log("layoutImagePosition()");
    let elem = document.getElementById('toolmenu-graphicimage-positioning');
    let theCont = document.getElementById("ws-graphic-image-container");

    if (flag != undefined) elem.checked = flag;

    if (elem.checked) {
        systemUndoRedoPush('layout');
        //--------------- add drag handles, corners and center, inset ----------------------------
        let theTag;
        theTag = document.createElement('div');
        theTag.classList = 'layout-image-handle tl';;
        theTag.addEventListener('mousedown', function () { layoutImageDraggingClick('tl') });
        theCont.appendChild(theTag);

        theTag = document.createElement('div');
        theTag.classList = 'layout-image-handle tr';
        theTag.addEventListener('mousedown', function () { layoutImageDraggingClick('tr') });
        theCont.appendChild(theTag);

        theTag = document.createElement('div');
        theTag.classList = 'layout-image-handle bl';
        theTag.addEventListener('mousedown', function () { layoutImageDraggingClick('bl') });
        theCont.appendChild(theTag);

        theTag = document.createElement('div');
        theTag.classList = 'layout-image-handle br';
        theTag.addEventListener('mousedown', function () { layoutImageDraggingClick('br') });
        theCont.appendChild(theTag);

        theTag = document.createElement('div');
        theTag.classList = 'layout-image-handle cent';
        theTag.addEventListener('mousedown', function () { layoutImageDraggingClick('cent') });
        theCont.appendChild(theTag);
    }
    else {
        //elem is false - remove drag handles
        theCont.innerHTML = '';
    }
}

//========================================================================================================================================

function layoutImageDraggingClick(arg) {
    console.log("layoutImageDraggingClick() " + arg)
    stateUX.indicatorDragging = arg; //Double duty
    document.addEventListener('mouseup', layoutImageDraggingStop, { once: true });
    document.getElementById("ws-graphic-container").addEventListener('mousemove', layoutImageDragging);
}

function layoutImageDraggingStop() {
    console.log("layoutImageDraggingStop()");
    clearTimeout(stateUX.mouseTimer); //prevent mousemove throttling from triggering after release
    document.getElementById("ws-graphic-container").removeEventListener('mousemove', layoutImageDragging);
}

function layoutImageDragging(event) {
    //throttle mousemove speed
    stateUX.mouseTimer = setTimeout(function () {
        document.getElementById("ws-graphic-container").addEventListener("mousemove", layoutImageDragging)
    }, 25);
    document.getElementById("ws-graphic-container").removeEventListener("mousemove", layoutImageDragging);

    console.log("DRAGGING " + stateUX.indicatorDragging);
    //Offset is static 195, 95 for now, rather than getBounding() etc
    let x = (event.clientX - cWSOffsetX) + document.getElementById("ws-graphic-container").scrollLeft;
    let y = (event.clientY - cWSOffsetY) + document.getElementById("ws-graphic-container").scrollTop;
    let xc = mainObj.layout.image.offsetX - x; //amount of change
    let yc = mainObj.layout.image.offsetY - y;
    //console.log(x, y, mainObj.layout.image.offsetX, mainObj.layout.image.offsetY, xc, yc);

    //handle grabs by the corner handles
    switch (stateUX.indicatorDragging) {
        case 'cent': //center
            mainObj.layout.image.offsetX = x - (mainObj.layout.image.width / 2);
            mainObj.layout.image.offsetY = y - (mainObj.layout.image.height / 2);;
            break;
        case 'tl': //top left
            mainObj.layout.image.width += xc;
            mainObj.layout.image.height += yc;
            mainObj.layout.image.offsetX = x;
            mainObj.layout.image.offsetY = y;
            break;
        case 'tr': //top right
            mainObj.layout.image.width = x - mainObj.layout.image.offsetX;
            mainObj.layout.image.height += yc;
            mainObj.layout.image.offsetY = y;
            break;
        case 'bl': //bottom left
            mainObj.layout.image.width += xc;
            mainObj.layout.image.height = y - mainObj.layout.image.offsetY;
            mainObj.layout.image.offsetX = x;
            break;
        case 'br': //bottom right
            mainObj.layout.image.width = x - mainObj.layout.image.offsetX;
            mainObj.layout.image.height = y - mainObj.layout.image.offsetY;
            break;
    } //end switch
    layoutPositionImageFunc(mainObj.layout.image.filePath); //update display
}

//========================================================================================================================================

function layoutPositionImageFunc(filePath) {
    console.log("layoutPositionImageFunc() " + filePath)

    let theCont = document.getElementById("ws-graphic-image-container");
    let theParent = document.getElementById("ws-graphic-image-container").parentElement;

    //--------------------------------------------------------------------------------------

    //if the path is too short, or otherwise wrong, ignore it to prevent errors.
    if (filePath.length < 4 || filePath == NaN || filePath.length == undefined && fs.existsSync(filePath)) {
        console.error("problem with image file path, ignoring");
        return;
    }

    //--------------------------------------------------------------------------------------

    theCont.style.backgroundImage = 'url("' + filePath + '")'; //NOTE THE QUOTES, will work with out quotes if file path doesn't have any spaces....
    theCont.style.backgroundSize = "100% 100%"; //the magic

    //--------------------------------------------------------------------------------------

    let img = new Image();
    img.addEventListener('load', function () {
        //---------------- size container div, ensuring it stays on the workspace view ------------------
        if (mainObj.layout.image.filePath.length < 2) {
            //new placement, create and fill the data

            //console.log("MAKING NEW")
            mainObj.layout.image = {}; //init storage object

            var ratio = Math.min(theParent.clientWidth / this.width, theParent.clientHeight / this.height);
            //console.log(this.width, this.height, ratio);
            //console.log(theParent.clientWidth, theParent.clientHeight);
            let newW = Math.round(this.width * ratio);
            let newH = Math.round(this.height * ratio);

            theCont.style.width = newW + "px";
            theCont.style.height = newH + "px";

            mainObj.layout.image.width = newW;
            mainObj.layout.image.height = newH;

            let newOffsetX = ((theParent.clientWidth - newW) / 2);
            let newOffsetY = ((theParent.clientHeight - newH) / 2);

            theCont.style.left = newOffsetX + "px";
            theCont.style.top = newOffsetY + "px";

            mainObj.layout.image.offsetX = newOffsetX;
            mainObj.layout.image.offsetY = newOffsetY;

            mainObj.layout.image.filePath = filePath; //save new path
        }
        else {
            //place the DIV in the specified position
            //console.log("RAN")
            theCont.style.width = mainObj.layout.image.width + "px";
            theCont.style.height = mainObj.layout.image.height + "px";
            theCont.style.left = mainObj.layout.image.offsetX + "px";
            theCont.style.top = mainObj.layout.image.offsetY + "px";
        }

        //when the user changes the box this would need to re-run defating the purpose
        //theCont.style.left = (((theParent.clientWidth - newW) / 2) / theParent.clientWidth) * 100 + "%";
        //theCont.style.top = (((theParent.clientHeight - newH) / 2) / theParent.clientHeight) * 100 + "%";

        uxToolsLayoutUpdate(); //update label - runs during dragging, a side effect of double duty initial placement and update
    });

    img.addEventListener('error', function () {
        console.log("ERROR - Graphic Image Layout");
        //clear the storage object or anything?
    });

    img.src = filePath;
}

//========================================================================================================================================

function layoutEnableGraphicImage(elem) {
    console.log("layoutEnableGraphicImage()");
    systemUndoRedoPush('layout');
    mainObj.layout.imageEnabled = elem.checked;
    if (mainObj.layout.imageEnabled) {
        if (mainObj.layout.image == undefined) {
            //if it doesn't exist, it is easier to add place holder than handle undefined all over the place
            mainObj.layout.image = {
                filePath: '',
                width: 0,
                height: 0,
                offsetX: 0,
                offsetY: 0
            }; //init storage object
        }
        else {
            //if(isNaN(mainObj.layout.image.filePath) && mainObj.layout.image.filePath != undefined) 
            layoutPositionImageFunc(mainObj.layout.image.filePath);
        }
    }
    else {
        layoutImageRemove();
    }
    uxToolsLayoutUpdate(); //may run rwice if layoutPositionImage runs it from promise, so if it this runs now it won't have the file name
}

//========================================================================================================================================
