'use strict';


//========================================================================================================================================

function systemValidateDeviceID(devID) {
    let flag = fs.existsSync(path.join(process.cwd(), 'assets/json/devices/hwid-' + devID + '.json'));

    if (flag == false) {
        //file does not exist, try the special folder next
        flag = fs.existsSync(path.join(process.cwd(), 'addin/hwid-' + devID + '.json')); //try the addin folder for device expansion and client specific
    }

    return flag; //return true or false
}

//========================================================================================================================================

function systemLoadDevice() {
    let fileName;

    if (fs.existsSync(path.join(process.cwd(), ('addin/hwid-' + stateDevice.hard.idNum + '.json')))) {
        // console.log("WAS TRUE");
        fileName = path.join(process.cwd(), 'addin/hwid-' + stateDevice.hard.idNum + '.json');
    }
    else fileName = path.join(process.cwd(), 'assets/json/devices/hwid-' + stateDevice.hard.idNum + '.json');

    console.log("systemLoadDevice() with " + fileName + "   Device Ver: " + stateDevice.hard.version + "   " + stateDevice.hard.revision)

    let devFile = JSON.parse(fs.readFileSync(fileName, 'utf8').toString());

    //starts with stateDevice.hard populated - ver/rev/IDnum/userIDNum

    //------------------- Transfers the device Profile data to the device object ----------------------------

    stateDevice.soft = devFile.default; //set default device properties

    //------------------- Checks if software is compatible with ver/rev of device ----------------------------

    uxButtonIndication('ws-hardware-devupdatewebpage', false);

    if (stateDevice.hard.hasOwnProperty("outdatedVerRev")) delete stateDevice.hard.outdatedVerRev; //clean up if reloading a device Profile
    if (stateDevice.hard.hasOwnProperty("updateVerRev")) delete stateDevice.hard.updateVerRev;

    // console.log("=================================",devFile.min.rev,  stateDevice.hard.version)
    if (stateDevice.hard.version < devFile.min.ver || (stateDevice.hard.version == devFile.min.ver && stateDevice.hard.revision < devFile.min.rev)) {
        //version 1 or got a 0, which is also bad
        if (stateDevice.hard.version < devFile.min.ver) pushMessageToLog("Device reported as Aurora version " + stateDevice.hard.version + ", and is not compatible. Please update device firmware, or use an older software version.");
        else pushMessageToLog("Device's firmware revision is not compatible. Please update the device firmware, or use an older software version.");
        //commDisconnectFromPort(); //not doing this anymore.... want to access commands even if outdated
        stateDevice.hard.outdatedVerRev = true;
        uxButtonIndication('ws-hardware-devupdatewebpage', true);
        uxTabHardwareUpdate();
        return; //just leave, it can't load anything, device not compatible
    }

    //-------------------------- version and revision is compatible. Now check against webVer for device updates -----------------------------

    try { //without the try it errors or something and stuff
        if (stateDevice.hard.version < stateSoftware.updateVer.devices[stateDevice.hard.idNum.toString()][0] ||
            (stateDevice.hard.version == stateSoftware.updateVer.devices[stateDevice.hard.idNum.toString()][0] && stateDevice.hard.revision < stateSoftware.updateVer.devices[stateDevice.hard.idNum.toString()][1])) {
            //ver wouldn't ever catch, not for a long time. Rev would though.
            uxButtonIndication('ws-hardware-devupdatewebpage', true);
            stateDevice.hard.updateVerRev = true;
            uxTabHardwareUpdate();
            pushMessageToLog("Device has an update available. It is recommended but not required.");
        }
    }
    catch (e) {
        console.log("Some error happened, this will happen if the device ID is not on the update version list")
    }

    //------------- merge the device Profile base with default ----------------------------

    var merged = {};
    Object.assign(merged, stateDevice.soft, devFile.base); //base is the standard device properties, merged with the default
    //console.log(stateDevice.soft)
    //console.log(merged)

    stateDevice.soft = merged; //overwrite with newly merged object

    //console.log(stateDevice);

    let devVer = stateDevice.hard.version;
    let devRev = stateDevice.hard.revision;

    //Test Simulate
    //devVer = 3;
    //devRev = 5; //DEBUG

    //--------------- attempts to find the configuration updates for ver/rev. If found it replaces stateDevice key values -----------------

    //Checks only the same version for revs. iterate rev up from 0, if entry is found. Use its data and overwrite .base, iterate all the way up to device's current rev
    for (let i = 0; i <= devRev; i++) {
        try {
            let updates = devFile["ver" + devVer]["rev" + i]; //this does not always exist
            //console.log("Running ",i,updates)
            if (updates != undefined) {
                console.log("New firmware rev found, applying updates to configurations", i);
                //  console.log(stateDevice.soft)
                let conv = Object.entries(updates); //converts the object to an array
                conv.forEach(function (val, index) {
                    // console.log(val)
                    //iterate the array, and overwrite device object with new values
                    //TODO: DONE Key could contain an object with multiple key/values, this would overwrite the entire object. Workaround any updated keys that are objects, insert the entire object 
                    // stateDevice.soft[val[0]] = val[1];

                    if (typeof val[1] === 'array') stateDevice.soft[val[0]] = val[1]; //over write array
                    if (typeof val[1] === 'object') {
                        if (val[1].hasOwnProperty("updateMethod")) {
                            let updateMethod = JSON.stringify(val[1].updateMethod).replaceAll('"',"");
                            delete val[1].updateMethod;
                            if (updateMethod == "overwrite") stateDevice.soft[val[0]] = val[1]; //over write object
                            else utilityObjectMerge(stateDevice.soft[val[0]], val[1])
                        }
                        else utilityObjectMerge(stateDevice.soft[val[0]], val[1])
                    }
                    else stateDevice.soft[val[0]] = val[1]; //over write value
                });
                console.log(stateDevice.soft)
            }
        }
        catch (err) {
            //console.log("Device ver/rev has no configuration updates", i);
        }
    }

    //-------------------------------------------------------------------------------------------------------------------------

    let configFlags = stateDevice.soft.configFlags;
    let configBytes = stateDevice.soft.configBytes;

    //console.log(configFlags)
    uxTabHardwareConfigModuleRender(configFlags, configBytes);
    uxTabHardwareUpdate();

    document.getElementById("ws-connection-devinfo-name").innerText = stateDevice.soft.name;
    document.getElementById("ws-connection-devinfo-pixels").innerText =
        `${stateDevice.soft.channels} Mono, ${Math.floor(stateDevice.soft.channels / 3)} RGB, ${Math.floor(stateDevice.soft.channels / 4)} RGBW`;
    document.getElementById("ws-connection-devinfo-frames").innerText = `256 / ${stateSoftware.maxFrames}`;
    document.getElementById("ws-connection-hwpvseqtile-mem").innerText = stateDevice.soft.hwpvSize;

    if (stateDevice.soft.bit16Data) colorPicker.updateDepth(0xFFFF);
    else colorPicker.updateDepth(0xFF);

}

//========================================================================================================================================\

function commGetDeviceConfigVal(module, type, len) {
    console.log("commGetDeviceConfigVal() " + module + "    " + type + "  " + len);

    let el1 = document.getElementById(module + "-elem1");
    let el2 = document.getElementById(module + "-elem2");

    switch (module) {

        //--------------------- 16bit Number Values ----------------------- 

        case "configModule-dmxaddress":
            if (len == 0) return parseInt(getValue16BitMSB(el1.value)); //MSB - ensure it is a number
            else return parseInt(getValue16BitLSB(el1.value)); //LSB - ensure it is a number

        //--------------------- standard dropdown index as ID - starting at 0, byte based ----------------------- 
        case 'configModule-pwmprofile':
        case "configModule-accmetedoubletap":
        case "configModule-dmxreceptionmode":
        case "configModule-serialbaud":
        case "configModule-pixelchipset": //value is a ID number
        case "configModule-colororder":
            if (type == 'byte') return parseInt(el1.options[el1.selectedIndex].value); //these modules are always a number
            else console.log("Error Requesting Bit");
            return;

        //--------------------- standard dropdown index as ID - starting at 0, bit based ----------------------- 
        case "configModule-dmxmaster16bit":
        case "configModule-dmxmaster": //0 or 1 only, bit based drop down
            if (type == 'bit') return parseInt(el1.options[el1.selectedIndex].value);
            else console.log("Error Requesting Byte");
            return;

        //--------------------- standard true/false checkbox ----------------------- 

        case "configModule-nativeinfrared":
        case "configModule-accmeterspeedadjust":
        case "configModule-enablegamma":
        case "configModule-autodetectcommmode":
            if (type == 'bit') return (el1.checked ? 1 : 0);
            else console.log("Error Requesting Byte")
            return;

        //--------------------- conversions ----------------------- 

        case "configModule-expansionmode":
        case "configModule-communicationmode":
            //both have bits and bytes. Both are the same.... probably
            return parseInt(el1.options[el1.selectedIndex].value); //these modules are always a number

        case "configModule-serialreception":
            //f8or16Bit, 0 = 8-bit, 1 = 16-bit. fRS485Enable 1= enabled
            switch (el1.selectedIndex) {
                default:
                case 0: //8-bit, TTL
                    return 0;
                case 1: //8-bit RS485
                    return 2;
                case 2: //16-bit, TTL
                    return 1;
                case 3: //16-bit RS485
                    return 3;
            }

        case "configModule-externalleddisplay":
            //00 Always On, 10 Countdown, 01 dim, 11 countdown+dim
            switch (el1.selectedIndex) {
                default:
                case 0: //0b00 Always On
                    return 0;
                case 1: //0b10 Countdown
                    return 2;
                case 2: //0b01 Dim
                    return 1;
                case 3: //11 countdown+dim
                    return 3;
            }


        case "configModule-ledactivity1":
        case "configModule-ledactivity2":
            //11"Both", 10 "RX", 01"Stand-Alone",  00"Disabled"
            switch (el1.selectedIndex) {
                default:
                case 0:
                    return 3; //0b11
                case 1:
                case 2:
                    return el1.selectedIndex; //0b10 or 0b01
                case 3:
                    return 0; //0b00
            }

        case "configModule-dmxreceptiontimeout":
            //00 - 10 - 11   "fDMXAutoRelease","fDMXReleaseIdle"
            switch (el1.selectedIndex) {
                default:
                case 0:
                    return 0; //0b00
                case 1:
                    return 1; //0b10
                case 2:
                    return 3; //0b11
            }
            break;
        case "configModule-accmetermovedetect":
            let convertShutoffVals = [0, 16, 32, 64, 96, 128, 192, 250];
            if (type == 'byte') return convertShutoffVals[el2.selectedIndex];
            else return el1.selectedIndex; //bit
            break;
    } //end switch

    console.error("DID NOT FIND AND RETURN THE DATA ===========================================",module);
}

//========================================================================================================================================

function commApplyDeviceConfigValues(buf) {
    console.log("commApplyDeviceConfigValues()", buf);

    let json = JSON.parse(fs.readFileSync(path.join(process.cwd(), 'assets/json/config-modules.json'), 'utf8'));

    if (buf.every(element => element > 255)) {
        console.log("An error occured, a value is over the maximum"); //will never happen......
    }
    else {
        //console.log(buf);
        let moduleCompleted = [];
        let bConfigurations = (buf[helperGetIndex("bConfigurationsMSB")] << 8) | buf[helperGetIndex("bConfigurationsLSB")];

        console.log("Downloaded Config Bits: " + getValue16BitBinary(bConfigurations));

        //Prepare to be confused. Made it extra confusing as I wanted to support bytes out of order in device memory. This always respects
        //  byte order as specified by the device Profile. Regardless of consecutive ordering within memory.
        //-------------------------------------- BITS ----------------------------------------------

        let obj = {
            name: '',
            num: 0,
            val: 0,
        }

        let trackObj = [];
        let mask;
        stateDevice.soft.configFlags.forEach(function (val, index) {
            for (var key in json) {
                if (json.hasOwnProperty(key)) {
                    if (json[key].bitStr.includes(val)) {// && !moduleCompleted.includes(key)) {
                        //the module scanned does have the matching config bit
                        let num = json[key].bitStr.indexOf(val); // DID THIS FIX IT????????????? really think so
                        //let num = moduleCompleted.filter(x => x == key).length; //= number of times the module is on the list. Which indicates the bit order
                        mask = (1 << index); //make the bit mask
                        let bit = 0;
                        if ((bConfigurations & mask) > 0) bit = 1;

                        obj.name = key;
                        obj.val = (bit << num);
                        trackObj.push(JSON.parse(JSON.stringify(obj)));

                        //console.log("Bit " + index + "   " + key + "  " + val, num, bit); //DEBUG

                        moduleCompleted.push(key);
                    }
                } //end if()
            } //end for()
        });

        //console.log(trackObj);
        //console.log(moduleCompleted);

        let moduleFiltered = [];

        moduleCompleted.forEach(function (v, i) {
            let built = 0;
            if (!moduleFiltered.includes(v)) {
                trackObj.forEach(function (vv, ii) {
                    if (vv.name == v) built += vv.val; //merges and converts binray to number
                });
                //built would then get set to the module element
                commSetDeviceConfigVal(v, 'bit', built); // Still not respecting some bit orders.... but does respect only matching flags, not extra module flags
                moduleFiltered.push(v);
            }
        });

        //----------------------------------- BYTES --------------------------------------------

        let configVal;
        moduleCompleted = []; //clear parsed list
        trackObj = [];

        stateDevice.soft.configBytes.forEach(function (val, index) {
            for (var key in json) {
                if (json.hasOwnProperty(key)) {
                    //console.log(json[key].byteStr, json[key].byteStr.includes(val));
                    if (json[key].byteStr.includes(val)) { // && !moduleCompleted.includes(key)) {
                        //the module scanned does have the matching config byte

                        //let num = moduleCompleted.filter(x => x == key).length; //= number of times the module is on the list. Which indicates the byte order
                        let num = json[key].byteStr.indexOf(val); // DID THIS FIX IT????????????? really think so

                        configVal = buf[helperGetIndex(val)];
                        obj.name = key;
                        obj.val = configVal;//(configVal << (8*num));
                        obj.num = num; //already base 0 - indicates the bit significance
                        trackObj.push(JSON.parse(JSON.stringify(obj)));

                        //console.log("Byte " + index + "   " + key + "  " + val + "  " + json[key].byteStr, num); //DEBUG

                        moduleCompleted.push(key); //list to track modules that have been parsed
                    }
                } //end if()
            } //end for()
        });

        moduleFiltered = [];

        moduleCompleted.forEach(function (v, i) {
            let built = 0;
            if (!moduleFiltered.includes(v)) {
                trackObj.forEach(function (vv, ii) {
                    if (vv.name == v) {
                        let num = moduleCompleted.filter(e => e == vv.name).length; //= number of times the module is on the list. Which indicates how many bytes
                        num--; //make base 0
                        built |= (vv.val << (8 * (num - vv.num))); //grabs the 8-bit byte. based on the module config byte order to indicate significance
                    }
                });
                commSetDeviceConfigVal(v, 'byte', built);
                moduleFiltered.push(v);
            }
        });

        //console.log(trackObj);
        //console.log(moduleCompleted);

        //-------------------------------------------------------------------------------
    }

    //-------------------------------------------------------------------------------

    function helperGetIndex(str) {
        return stateDevice.soft.configBytes.findIndex(element => element.includes(str));
    }

    //-------------------------------------------------------------------------------
}

//========================================================================================================================================

function commSetDeviceConfigVal(conModule, type, passedVal) {
    console.log("commSetDeviceConfigVal() with ", conModule, passedVal);

    let el1 = document.getElementById(conModule + "-elem1");
    let el2 = document.getElementById(conModule + "-elem2"); //might be undefined....

    //--------------------- Blinks the config module elements shadow to indicate to the  user ----------------------- 
    //Would prefer it only flashes the changed elements, but that is too much
    blink(el1);
    if (el2 != undefined) blink(el2);

    switch (conModule) {
        //--------------------- 16bit Number Values ----------------------- 

        case "configModule-dmxaddress":
            if (type == 'byte') el1.value = passedVal;
            return;

        //--------------------- Standard dropdown index as ID - starting at 0, byte based ----------------------- 

        case "configModule-accmetedoubletap":
        case "configModule-dmxreceptionmode":
        case "configModule-serialbaud":
        case "configModule-colororder":
            //OR FIND THE OPTION VALUE, that would be more controllable
            if (type == 'byte') el1.selectedIndex = passedVal; //these modules are always a number
            //else console.log("Error Setting Bit")
            return;

        //--------------------- Special dropdown option as index - starting at 0, byte based ----------------------- 

        case "configModule-pixelchipset":
            //have to find the value on the option list
            convertSelectInputOption(el1.id, passedVal);
            return;

        //--------------------- Standard dropdown index as ID - starting at 0, bit based ----------------------- 

        case "configModule-dmxmaster":
            //OR FIND THE OPTION VALUE, that would be more controllable
            if (type == 'bit') el1.selectedIndex = passedVal; //these modules are always a number
            //else console.log("Error Setting Byte")
            return;

        //--------------------- Standard true/false checkbox ----------------------- 

        case "configModule-nativeinfrared":
        case "configModule-accmeterspeedadjust":
        case "configModule-enablegamma":
        case "configModule-autodetectcommmode":
        case "configModule-restoreindex":
            if (type == 'bit') el1.checked = (passedVal == 0 ? false : true);
            //else console.log("Error Setting Byte")
            return;

        //--------------------- Conversions ----------------------- 
        case 'configModule-pwmprofile':
        case "configModule-expansionmode":
        case "configModule-communicationmode":
            el1.selectedIndex = passedVal; //these modules are always a number
            return;

        case "configModule-serialreception":
            switch (passedVal) {
                default:
                case 0: //8-bit, TTL
                    el1.selectedIndex = 0;
                    return;
                case 2: //8-bit RS485
                    el1.selectedIndex = 1;
                    return;
                case 1: //16-bit, TTL
                    el1.selectedIndex = 2;
                    return;
                case 3: //16-bit RS485
                    el1.selectedIndex = 3;
                    return;
            }

        case "configModule-externalleddisplay":
            //00 Always On, 10 Countdown, 01 dim, 11 countdown+dim
            switch (passedVal) {
                default:
                case 0: //0b00 Always On
                    el1.selectedIndex = 0;
                    return;
                case 1: //0b10 Countdown
                    el1.selectedIndex = 2;
                    return;
                case 2: //0b01 Dim
                    el1.selectedIndex = 1;
                    return;
                case 3: //11 countdown+dim
                    el1.selectedIndex = 3;
                    return;
            }

        case "configModule-ledactivity1":
        case "configModule-ledactivity2":
            //11"Both", 10 "RX", 01"Stand-Alone",  00"Disabled"
            switch (passedVal) {
                default:
                case 0:
                    el1.selectedIndex = 3; //0b00
                    return;
                case 1:
                case 2:
                    el1.selectedIndex = passedVal; //0b10 or 0b01
                    return;
                case 3:
                    el1.selectedIndex = 0; //0b11
                    return;
            }

        case "configModule-dmxreceptiontimeout":
            //00 - 10 - 11   "fDMXAutoRelease","fDMXReleaseIdle"
            switch (passedVal) {
                case 2:  //0b10 would be an error, second bit(fDMXReleaseIdle) indicates blank or idle, needs bit1(fDMXAutoRelease) to enable
                case 0:
                    el1.selectedIndex = 0; //0b00
                    return;
                case 1:
                    el1.selectedIndex = 1; //0b01
                    return;
                case 3:
                    el1.selectedIndex = 2; //0b11
                    return;
            }

        case "configModule-accmetermovedetect":
            let convertShutoffVals = [0, 16, 32, 64, 96, 128, 192, 250];
            if (type == 'byte') {
                //  el2.selectedIndex = convertShutoffVals.findIndex(element => element.includes(passedVal)); // TEST NOT SURE IF RIGHT
                el2.selectedIndex = convertShutoffVals.indexOf(passedVal);
                return;
            }
            else {
                el1.selectedIndex = passedVal; //must be 'bit'
                return;
            }
        case "configModule-dmxmaster16bit":
            el1.selectedIndex = passedVal;
            return;
    } //end switch

    console.error("DID NOT FIND AND RETURN THE DATA ===========================================",conModule);

    //-------------------------------------------------------------------------------

    function blink(elem) {
        if (elem.tagName == 'INPUT' && elem.type == 'checkbox') elem = elem.parentElement;
        //else normal
        elem.classList.add('shadow-flash');
        setTimeout(function () {
            elem.classList.remove('shadow-flash');
        }, 500);
    }
}

//========================================================================================================================================

