"use strict";

//========================================================================================================================================

function ovmLiveControlColorMult(viewMode) {
    if (viewMode.indexOf('Mono') > -1) return 1;
    else if (viewMode.indexOf('RGBW') > -1) return 4; //do before RGB or it will find it instead
    else if (viewMode.indexOf('RGB') > -1) return 3;
}

function ovmLiveControlViewMode() {
    let viewMode = document.getElementById('ovmenu-2-viewmode');
    viewMode = viewMode.options[viewMode.selectedIndex].getAttribute("data-viewmode"); //string label method
    return viewMode;
}

function ovmLiveControlColorLabels(color) {
    document.getElementById('ovmenu-2-colorred-val').innerText = '' + color[0];
    document.getElementById('ovmenu-2-colorgreen-val').innerText = '' + color[1];
    document.getElementById('ovmenu-2-colorblue-val').innerText = '' + color[2];
}

//========================================================================================================================================

function ovmLiveControlSliderRefresh(elem, id) {
    let viewMode = ovmLiveControlViewMode();
    let colorMult = ovmLiveControlColorMult(viewMode);
    let newVal = elem.value;
    //console.log('drag on ' + id)

    //----------------------------------- update the current color block ---------------------------------------

    let colorRect = document.getElementById('ovmenu-2-color');
    colorRect.innerText = (id + 1) + ''

    switch (viewMode) {
        case 'slidersRGB':
            colorRect.style.backgroundColor = 'hsl(' + (newVal * 1.411) + ', 100%, 50%)'; //scale up to 360
            let bgcolor = document.getElementById('ovmenu-2-color').style.backgroundColor; //CSS converted the hsl to rgb, now get the RGB
            let convRGB = bgcolor.split("(")[1].split(")")[0];
            convRGB = convRGB.split(",");
            ovmLiveControlColorLabels(convRGB);
            stateCommunication.liveControlPacket[id * colorMult] = parseInt(convRGB[0]);
            stateCommunication.liveControlPacket[(id * colorMult) + 1] = parseInt(convRGB[1]);
            stateCommunication.liveControlPacket[(id * colorMult) + 2] = parseInt(convRGB[2]);
            break;
        case 'slidersMono':
            colorRect.style.backgroundColor = 'rgb(' + newVal + ',' + newVal + ',' + newVal + ')';
            ovmLiveControlColorLabels([newVal, newVal, newVal]);
            stateCommunication.liveControlPacket[id] = parseInt(newVal);
            break;
        case 'slidersRGBW':
            //never gets here
            break;
    } //end switch
    stateCommunication.liveUpdatePending = true;
}

//========================================================================================================================================

function ovmLiveControlIconSelect(elem, id) {
    let viewMode = ovmLiveControlViewMode();
    const cmodes = ['', 'Mono', '', 'RGB', 'RGBW'];     //hacky way to pass the color mode
    let localColorMode = cmodes[ovmLiveControlColorMult(viewMode)];
    //console.log(localColorMode)
    stateUX.liveControlElem = elem; //set global
    colorPicker.open(event.clientX, event.clientY, localColorMode, 0, id, 'liveicon'); //last indicates mode
}

//========================================================================================================================================

function ovmLiveControlReset() {
    let channels = Math.floor(document.getElementById('ovmenu-2-numofpix').value * ovmLiveControlColorMult(ovmLiveControlViewMode()));
    stateCommunication.liveControlPacket = new Uint8Array(channels);
    stateCommunication.liveControlPacket.fill(0);
    stateCommunication.liveUpdatePending = true;
    ovmLiveControlRefresh();
}

//========================================================================================================================================

function ovmLiveControlUpdate(reset = false) {
    let numOfPixElem = document.getElementById('ovmenu-2-numofpix');
    let colorMult = ovmLiveControlColorMult(ovmLiveControlViewMode());

    let maxChannels = stateDevice.soft.channels;
    if (maxChannels == 0 || maxChannels == undefined) numOfPixElem.max = stateSoftware.maxPixels; //no device is connected
    else numOfPixElem.max = Math.floor(maxChannels / colorMult); //set max based on device maxChannels
    numOfPixElem.min = 1;

    if (numOfPixElem.value == undefined) numOfPixElem.value = 1; //ensure it is defined and no less than 1

    if (numOfPixElem.value < numOfPixElem.min) numOfPixElem.value = numOfPixElem.min; //set it to 1...
    //else if (numOfPixElem.value > numOfPixElem.max) numOfPixElem.value = numOfPixElem.max; //set to max if over max 

    if (stateDevice.soft.lockChannelSize) {
        numOfPixElem.value = Math.floor(stateDevice.soft.channels / colorMult);
        numOfPixElem.min = numOfPixElem.value;
        numOfPixElem.max = numOfPixElem.value;
    }

    uxClampNumberBox(numOfPixElem);

    let channels = Math.floor(numOfPixElem.value * colorMult);
    document.getElementById('ovmenu-2-numofpixmax').innerText = Math.round(numOfPixElem.max);

    if (reset == true) {
        stateCommunication.liveControlPacket = new Uint8Array(channels);
        stateCommunication.liveControlPacket.fill(0);
        stateCommunication.liveUpdatePending = true;
    }

    if (stateCommunication.connected == false) {
        pushMessageToLog("No NLED device is connected, can not start Live Control mode.");
    }
    else if (stateCommunication.connected) auroraCMD.setLiveControlMode(true, 8, channels); //enable, 8-bit data, using local channels

    ovmLiveControlRefresh();
}

//========================================================================================================================================

function targetSequence(seqID) {
    console.log("targetSequence()  SeqID#:" + seqID + "   SeqTarget: " + stateUX.seqTarget + "   SlotTarget: " + stateUX.slotTarget);
    switch (stateUX.seqTarget) {
        case 'idle':
            mainObj.idleSeqID = seqID;
            refreshTab();
            break;
        case 'hwpv':
            commUploadHWPV(seqID);
            break;
        case 'list':
            if (mainObj.seq[seqID].mode == 'mList') return;
            sequenceAddToList(seqID, stateUX.slotTarget)
            break;
        case 'indexadd':
            indexAddTile(seqID)
            break;
        case 'indexreplace':
            indexReplaceTile(seqID)
            break;
    }
    overlayMenuEvent(event, 'close'); //now close overlay menu
}

//========================================================================================================================================

function standaloneMenuIntensitySlide(elem) {
    document.getElementById("ovmenu-3-con-intenslider-val").value = elem.value;
}

function standaloneMenuIntensityBox(elem) {
    document.getElementById("ovmenu-3-con-intenslider").value = elem.value; //convert from 255 to 100
}

//========================================================================================================================================

function standaloneMenuFadeControlSlide(elem) {
    document.getElementById("ovmenu-3-con-fadecyclebox").value = elem.value; //convert from 255 to 100
}

//========================================================================================================================================

function standaloneMenuCmd(cmd, val) {

    let modifier;
    if (!stateCommunication.connected) {
        document.getElementById("ovmenu-3-sect-messages").innerText = new Date().toLocaleTimeString() + " - Device not connected.";
        return;
    }

    document.getElementById("ovmenu-3-sect-messages").innerText = new Date().toLocaleTimeString() + " - Attempting command....."; //default message in case connection is lost

    switch (cmd) {
        case 'onOff':
            auroraCMD.setOnOff(message("Toggled On/Off soft power"));
            break;
        case 'seqPrev':
            auroraCMD.setSequencePrevious(message("Previous Sequence On Index Selected"));
            break;
        case 'seqNext':
            auroraCMD.setSequenceNext(message("Next Sequence On Index Selected"));
            break;
        case 'playPause':
            auroraCMD.setPlayPauseToggle(message("Play/Pause Toggled"));
            break;
        case 'speedUp':
            auroraCMD.setSpeedIncrease(message("Speed Value Increased (Higher is slower)"));
            break;
        case 'speedDown':
            auroraCMD.setSpeedDecrease(message("Speed Value Decreased (Lower is faster)"));
            break;
        case 'setIdle':
            auroraCMD.setIdleSequence(message("Idle Sequence Selected (defaults to Indexed Sequence #1 if not specified)"));
            break;
        case 'seqSetForm':
            val = document.getElementById("ovmenu-3-con-seqid").value;
        case 'seqSet':
            auroraCMD.setSequenceByID(val, message("Indexed Sequence #" + val + " selected."));
            break;
        case 'stepNext':
            auroraCMD.setStepForward(message("Frame# Incremented"));
            break;
        case 'stepPrev':
            auroraCMD.setStepPrevious(message("Frame# Decremented"));
            break;
        case 'frameSet':
            val = document.getElementById("ovmenu-3-con-frameselectbox").value;
            modifier = (document.getElementById("ovmenu-3-con-frameselectpause").checked) ? 1 : 0;
            auroraCMD.setFrameNumber(val, modifier, message(`Set to Frame# ${val}. ${(modifier == 1) ? "Pause Enabled" : ""}`));
            break;
        case 'speedSet':
            val = document.getElementById("ovmenu-3-con-speedbox").value;
            auroraCMD.setSpeed(val, message("Speed has been set to " + val));
            break;
        case 'intenSet':
            val = Math.ceil(document.getElementById("ovmenu-3-con-intenslider-val").value * 2.55);
            auroraCMD.setIntensity(val, message("Device global intensity has been set to " + document.getElementById("ovmenu-3-con-intenslider-val").value + "%"));
            break;
        case 'fadeConSend':
            val = document.getElementById("ovmenu-3-con-fadecyclebox").value;
            auroraCMD.setControlFade(val, message("Running Fade Control with " + val + " steps."));
            break;
        case 'intensityDown':
        case 'intensityUp':
            noSendMessage("No action performed. Use the 'Global Intensity' tools to the right.")
            break;
        default: //just incase
        case 'colorR':
        case 'colorG':
        case 'colorB':
        case 'colorW':
        case 'colorSet':
        case 'colorReset':
        case 'seq':
            noSendMessage("No action implemented yet.")
            break;
    }
    //-----------------------------------------------------------------------------------------------------------------------

    function noSendMessage(msg) {
        document.getElementById("ovmenu-3-sect-messages").innerText = msg;
    }

    function message(msg) {
        return function () {
            console.log("standaloneMenuMessage() " + msg);
            document.getElementById("ovmenu-3-sect-messages").innerText = new Date().toLocaleTimeString() + " - SUCCESS: " + msg;
        }
    }
    //-----------------------------------------------------------------------------------------------------------------------
}

//========================================================================================================================================

function uxModalDeviceProfiles(event) {

    const devicesFolder = path.join(process.cwd(), '/assets/json/devices/');
    // const fs = require('fs');
    overlayMenuEvent(event, 'open', 'menu-8');
    const modalWorkspace = document.getElementById("menu-8-workspace");
    modalWorkspace.innerHTML ="";
    let profileList = [];


    try {
        fs.readdirSync(devicesFolder).forEach((fileName, i) => {
            //console.log(fileName);
            let file = fs.readFileSync(path.join(devicesFolder, fileName), 'utf8');
            let name = JSON.parse(file).default.name;
           // console.log(name);
            profileList.push(fileName);
            modalWorkspace.innerHTML += `<button id="menu-8-${i}" class="ux-buttons buttonstyle" style="width:50ch">${name}</button><br>`;
        });



        for (const i in profileList) {
          //  console.log(i);
            document.getElementById(`menu-8-${i}`).addEventListener('click', function () {
                console.log("Clicked ", profileList[i]);
                stateDevice.hard.idNum = profileList[i].split("-")[1].split(".")[0];

                let file = fs.readFileSync(path.join(devicesFolder, profileList[i]), 'utf8');

                stateDevice.hard.version = JSON.parse(file).min.ver;
                stateDevice.hard.revision = JSON.parse(file).min.rev;
                systemLoadDevice();
                overlayMenuEvent(event, 'close'); 
            });
        }
    }
    catch (e) { }
}


//========================================================================================================================================

