"use strict";

//========================================================================================================================================

function uxTabHardwarePopulatePorts(ports) {
    console.log("uxTabHardwarePopulatePorts()");

    let elem = document.getElementById("ws-hardware-port");
    let option;
    var portCount = 0;
    elem.innerText = null //clear all the options currently listed

    //rewrite the options to the select drop down
    ports.forEach(function (port) {
        //console.log("Port: ", auroraCMD.port.path);
        option = document.createElement("option");
        option.value = port.path;
        option.text = port.path.substring(port.path.length - 14); //shorten name for Mac/Linux
        elem.add(option);
        portCount++;
    })
    // });

    //-------------- Now Do baud rate, here instead of separate function ----------------

    let myBaudSelect = document.getElementById("ws-hardware-baud");
    let baudRates = [9600, 19200, 38400, 57600, 115200, 230400, 250000, 500000, 1000000];

    myBaudSelect.innerText = null //clear all the options currently listed

    //rewrite the options to the select drop down
    baudRates.forEach(function (rate) {
        option = document.createElement("option");
        option.text = rate;
        option.value = rate;
        myBaudSelect.add(option);

    });

    myBaudSelect.selectedIndex = 4; //select option to default baud rate - 19200

    //hacky, but portCount is not ready by the time the function finishes. for whatever reason, so delay pushing message, then it works.
    // setTimeout(function () {
    //     pushMessageToLog("Serial Port Scan, Complete. Found " + portCount + " ports.");
    // }, 10);

    utilityAwaitDOMUpdate(function () {
        pushMessageToLog("Serial Port Scan, Complete. Found " + portCount + " ports.");
    }); //wait until DOM has created the elements
} //end func

//========================================================================================================================================

function uxTabHardwareConfigModuleRender(bits, bytes) {
    console.log("uxTabHardwareConfigModuleRender()");
    //console.log(bits);
    //console.log(bytes);

    let theDiv = document.getElementById("ws-hardware-configcontainer");
    theDiv.innerHTML = ''; //clear container

    let json = JSON.parse(fs.readFileSync(path.join(process.cwd(), 'assets/json/config-modules.json'), 'utf8'));

    //----------------------- Scan the modules and compare with bits & bytes -----------------------------

    let moduleList = [];

    bits.forEach(function (val, index) {
        for (var key in json) {
            if (json.hasOwnProperty(key)) {
                //console.log(key + " -> " + json[key].bitStr);
                if (json[key].bitStr.includes(val)) {
                    //console.log("YES " + key + "  " + val)
                    if (json[key].enabled == undefined) //not defined, so it is not false
                    {
                        json[key].enabled = true; //not sure if I will use this
                        moduleList.push(key);
                    }
                    else console.log("module(bit) already defined:", key)
                }
            } //end if()
        } //end for()
    });

    bytes.forEach(function (val, index) {
        for (var key in json) {
            if (json.hasOwnProperty(key)) {
                if (json[key].byteStr.includes(val)) {
                    //console.log("YES " + key + "  " + val)
                    if (json[key].enabled == undefined) //not defined, so it is not false
                    {
                        json[key].enabled = true; //not sure if I will use this
                        moduleList.push(key);
                    }
                    else console.log("module(byte) already defined")
                }
            } //end if()
        } //end for()
    });

    //console.log(moduleList);

    //--------------------------- List of enabled modules has been collected -------------------------------

    let theTag;
    let theElem;
    let lg = stateSoftware.language;
    let configObj;
    let theHelp;

    let lang = lg;

let priorityOrder = 1;

let sortedModuleList = [];


for (let i = 0; i < moduleList.length; i++) {
    sortedModuleList.push([moduleList[i],json[moduleList[i]].priority]);

}

sortedModuleList.sort((a, b) => a[1] - b[1]);

// console.log(moduleList)

    for (let i = 0; i < moduleList.length; i++) {
           configObj = json[sortedModuleList[i][0]];
        // configObj = json[moduleList[i][0]];

        theTag = document.createElement('div');
        theTag.className = 'ws-hardware-config-module';
        theTag.style.order = configObj.priority+priorityOrder;
        priorityOrder++;

        //test if language specified language is defined
        if (configObj.title[lg] == undefined) lang = 'en'; //default to english if not defined
        else lang = lg; //set language to look for user specified
        theTag.innerHTML = '<h2>' + configObj.title[lang] + '</h2>';

        //test if language specified language is defined
        if (configObj.title[lg] == undefined) lang = 'en'; //default to english if not defined 
        else lang = lg; //set language to look for user specified
        if (configObj.description[lang] != undefined) theTag.innerHTML += configObj.description[lang] + "<br>";

        if (configObj.elem1 != undefined) builder(sortedModuleList[i][0], 'elem1');
        if (configObj.elem2 != undefined) builder(sortedModuleList[i][0], 'elem2');

        theHelp = document.createElement('div');
        theHelp.className = 'ws-hardware-config-module-help tooltip';
        theHelp.id = sortedModuleList[i][0] + '-tooltip';
        theHelp.textContent = '?';

        theDiv.appendChild(theTag); //else if it has no elements, write config module tag to container

        if (configObj.help[lg] == undefined) lang = 'en'; //default to english if not defined
        else lang = lg; //set language to look for user specified
        if (configObj.help != undefined && configObj.help[lang] != undefined) {
            uxToolTipAttach(theHelp, configObj.help[lang], 'up')
            theTag.appendChild(theHelp); //puts the question mark into the config module
        }
    } //end for

    //-----------------------------------------------------------------------------------------------------

    function builder(name, elem) {
        //console.log(configObj.title[lang]);
        let elemID = name + '-' + elem;

        if (elem == 'elem2') theTag.innerHTML += '<br>';
        //only one with an input element label
        if (configObj[elem].type == 'checkbox') {

            if (configObj[elem].label[lg] == undefined) lang = 'en'; //default to english if not defined
            else lang = lg; //set language to for user specified
            theTag.innerHTML += configObj[elem].label[lang] + ' ';
        }

        switch (configObj[elem].type) {
            case 'select':
                theElem = document.createElement('select');
                theElem.id = elemID;
                theElem.className = 'ux-dropdowns ddstyle';
                theTag.appendChild(theElem)

                if (configObj[elem].options[lg] == undefined) lang = 'en'; //default to english if not defined
                else lang = lg; //set language to for user specified
                let options = configObj[elem].options[lang];

                if (options.length == 1 && options[0] == 'DYNAMIC') {
                    loader(name);
                }
                else {
                    for (let q = 0; q < options.length; q++) {
                        var opt = options[q];
                        var el = document.createElement("option");
                        el.textContent = opt;
                        el.value = q;
                        theElem.appendChild(el);
                    }
                }
                break;
            case 'checkbox':
                theElem = document.createElement('label');
                theElem.className = 'ux-switch';
                theTag.appendChild(theElem)
                let theChild = document.createElement('input');
                theChild.type = "checkbox";
                theChild.id = elemID;
                theElem.appendChild(theChild);
                theChild = document.createElement('span');
                theChild.className = "switch-slider roundswitch"
                theElem.appendChild(theChild);
                break;
            case 'input':
                theElem = document.createElement('input');
                theElem.type = "number";
                theElem.id = elemID;
                theElem.className = "ux-textinput textinput-style";
                theElem.value = configObj[elem].value;
                theElem.min = configObj[elem].min;
                theElem.max = configObj[elem].max;
                theElem.style.width = configObj[elem].width + 'px';
                theElem.addEventListener('input', function () { uxClampNumberBox(this) }); //set to default clamper
                theTag.appendChild(theElem)
                break;
        } //end switch
    } //end helper

    //-----------------------------------------------------------------------------------------------------

    function loader(name) {
        let options = [];
        let values = [];

        switch (name) {
            case 'configModule-pixelchipset':
                let conv = Object.entries(stateDevice.soft.pixelChipsets); //converts the object to an array
                console.log(conv);

                for (let q = 0; q < conv.length; q++) {
                    options.push(conv[q][0])
                    values.push(conv[q][1])
                }
                //options = stateDevice.soft.pixelChipsets
                //console.log(options);
                //console.log(values);
                break;
            case 'configModule-expansionmode':
                options = stateDevice.soft.expansionMode;
                break;
            case 'configModule-communicationmode':
                options = stateDevice.soft.communicationMode;
                break;
            case 'configModule-dmxreceptionmode':
                options = stateDevice.soft.dmxRXModes;
                break;
            case 'configModule-pwmprofile':
                options = stateDevice.soft.pwmModes;
                break;
        }

        //console.log(values.length)

        for (let q = 0; q < options.length; q++) {
            var opt = options[q];
            var el = document.createElement("option");
            el.textContent = opt;
            if (values.length == 0) el.value = q;
            else el.value = values[q];
            theElem.appendChild(el);
        }
    } //end helper

    //-----------------------------------------------------------------------------------------------------


    console.log(theDiv.children)


/*
    console.log("+++++++++++++++++++++++++++++++++ DEBUG");


    let contSz = theDiv.getBoundingClientRect();
let index = 0;
    let moduleSizes = [];
    for (const childs of theDiv.children) {
        console.log(childs)
       // childs.style.removeProperty('order')

        let bounding = childs.getBoundingClientRect();

// moduleSizes.push([childs.getBoundingClientRect().x,childs.getBoundingClientRect().y,childs.getBoundingClientRect().width,childs.getBoundingClientRect().height])
moduleSizes.push([index++, Math.round(bounding.x),Math.round(bounding.y),Math.round(bounding.width),Math.round(bounding.height)])
    }
    console.log(moduleSizes)

    //let entries = Object.entries(moduleSizes);

 let   arr = moduleSizes.sort((a, b) => a[2] - b[2]);


let rowSpaces = [];
let rowPx = arr[0][2];
let rowSpace = 0;
for(const q in arr) {
    console.log(q, arr[q][0],"checking:",arr[q][2],rowPx)
    if(arr[q][2] == rowPx) rowSpace +=  arr[q][3];
    else {
        console.log( arr[q][0],"rowspace:",rowSpace)
        rowSpaces.push(contSz.width - rowSpace);
        rowSpace = 0;
        rowPx = arr[q][2];
    }


}
console.log(rowSpaces)
console.log(arr)
console.log(contSz)
console.log(moduleSizes)
*/
} //end func

//========================================================================================================================================

function uxTabHardwareUpdate() {
    document.getElementById("ws-hardware-devicename").innerText = stateDevice.soft.name;

    let devFullVer = document.getElementById("ws-hardware-devicever");
    let devBLVer = document.getElementById("ws-hardware-deviceblver");
    let devUserID = document.getElementById("ws-hardware-userid");
    let devSerialNum = document.getElementById("ws-hardware-deviceserialnum");

    let verMsg = String.fromCharCode(48 + stateDevice.hard.version) + String.fromCharCode(97 + stateDevice.hard.revision);

    if (stateCommunication.connected) {
        if (stateDevice.hard.version > 100) {
            devFullVer.innerText = verMsg + " (unverified)";
        }
        else {
            if (stateDevice.hard.hasOwnProperty("outdatedVerRev")) {
                //it is true
                devFullVer.innerText = verMsg + " (INCOMPATIBLE)";
            }
            else if (stateDevice.hard.hasOwnProperty("updateVerRev")) {
                //it is true
                devFullVer.innerText = verMsg + " (UPDATE AVAIL.)";
            }
            else devFullVer.innerText = verMsg;
        }
        devBLVer.innerText = stateDevice.hard.bootVer;
        devUserID.innerText = stateDevice.hard.userIDNum;
        if(stateDevice.hard.serialNum == "FFFFFFFF") devSerialNum.innerText = "unavailable";
        else devSerialNum.innerText = "0x"+stateDevice.hard.serialNum;
    }
    else {
        devFullVer.innerText = verMsg + " (emulated)";
    }
}

//========================================================================================================================================

