"use strict";

//========================================================================================================================================

function systemLoadSequenceFileInit(filePath, targetFlag = true) {
    console.log("systemLoadSequenceFileInit() from " + filePath);

    //filePath = filePath.toString().replace(/\\/g, "/"); //make it work I guess, always forward slashes, avoids escaped character issues

    //uxProgressBarShow(); //removed v3.3.0 - does not need this, it loads instantly

    let theFile; //scratch
    //if (!fs.existsSync(filePath)
    //Asynch method for loading JSON
    fs.readFile(filePath, function (err, data) {
        if (err) throw err;

        //console.log(data)

        //--------------------- clear some junk ---------------------------------------------------------



        layoutGraphicIconReset(); // guess do this here, pretty much just want to clear the container. Prevents persistance after a file load
        mainObj.layout.advanced = false;
        stateSeq.index = 0;//reset

        //-------------------- detect what kind of file it is ---------------------------------------

        try {
            theFile = JSON.parse(data); //pass into scratch
            console.log("The loaded file is a JSON")
        }
        catch (e) {
            theFile = null;
            console.log("The loaded file is NOT a JSON - must be version 1 or 2")
        }

        if (theFile == null) {
            //could be v1/v2 file
            try {
                fileConvertSequenceFile(filePath);
                pushMessageToLog("The loaded sequence file is an old format, please select a new sequence file to save to. Use the Save As on the Software tab.");
            }
            catch (e) {
                console.log("File load errored. Not a JSON. May not be an Aurora sequence file.");
                pushMessageToLog("File was unable to load. May not be an Aurora sequence file.");
                return;
            }
        }
        else {
            //it is a json, handle if it is correct
            try {
                systemLoadSequenceFile(filePath);  //load normally
                if (targetFlag) {
                    selectConfirmBox('loadfile-savetarget', function () { uxTargetLoadedFile(filePath); });
                }
            }
            catch (e) {
                console.log("File load errored. File is a JSON");
            }
        }

        // uxProgressBarHide(); //removed v3.3.0 - does not need this, it loads instantly

        //------------------------- Load Device Profile if required-------------------------------

        if (mainObj.device.hasOwnProperty("simulate")) {
            console.log(mainObj.device);
            if (mainObj.device.simulate) {
                mainObj.device.id = Number(mainObj.device.id);
                //NOT SURE IF I WILL KEEP THIS
                if (systemValidateDeviceID(mainObj.device.id)) {
                    if (stateCommunication.connected == true) {
                        console.log(mainObj.device.id, stateDevice.hard.idNum);
                        if (mainObj.device.id == stateDevice.hard.idNum) {
                            pushMessageToLog("The loaded file is for the connected device.");
                        }
                        else pushMessageToLog("The loaded file is for a different device than connected. Please review.");
                    }
                    else {
                        stateDevice.hard.idNum = mainObj.device.id;
                        stateDevice.hard.hardVer = 0;
                        stateDevice.hard.version = mainObj.device.ver;
                        stateDevice.hard.revision = mainObj.device.rev;
                        stateDevice.hard.bootTargetID = 0;
                        stateDevice.hard.bootVer = 0;
                        stateDevice.hard.userIDNum = 0;
                        systemLoadDevice(); //loads device Profile for the files id/ver/rev
                        pushMessageToLog("No device currently connected. Emulating: " + stateDevice.soft.name);
                        commUXMessage('emulate');
                    }
                }
                else pushMessageToLog("The loaded file indicates an unknown device.");
            }
        }
        //-------------------------- Test and handle Layout Image ------------------------------

        if (mainObj.layout.imageEnabled) {
            if (!fs.existsSync(mainObj.layout.image.filePath)) {
                mainObj.layout.imageEnabled = false;
                pushMessageToLog("Graphic Layout Image file was not found, disabling. Tried: " + mainObj.layout.image.filePath);
            }
        }

        //------------------------ Handle Missing Information --------------------------------

        let descMsg = '';
        if (mainObj.description == undefined || mainObj.description == 'undefined' || mainObj.description == '') descMsg = "(enter a description here)"; //default text, will save this....
        else descMsg = mainObj.description;
        document.getElementById("ws-software-descripbox").value = descMsg;

        if (mainObj.layout.rotation == null || mainObj.layout.rotation == undefined) { //if older sequence file, may not define this
            mainObj.layout.rotation = 0;
        }

        //--------------------------------------------------------

        //mainObj.globalIntensityEnable = (mainObj.globalIntensityEnable == "true"); //ensure it is a boolean - was not working
        if (stateSeq.globalIntensity > 100) stateSeq.globalIntensity = 100; //clamp just in case

        //--------------------------- Initialize Various Things -----------------------------

        //.scrollTop won't work when menu is 'display: none' - so this does nothing, any hack way to set the display didn't work
        document.getElementById('ovmenu-1-workspace').scrollTop = 0; //select sequence menu
        document.getElementById('ovmenu-5-workspace').scrollTop = 0; //target sequence menu

        sequenceSelect(stateSeq.index, true); //now select the first sequence - updates UX and stuff as well

        if (mainObj.layout.advanced) layoutGraphicIconsRender();
        else layoutGraphicBuildIcons();
        uxToolsLayoutUpdate(); //tools and stuff

        pushMessageToLog("Finished Loading Sequence file from " + filePath.toString());
    });
}

//========================================================================================================================================

//not to be called by itself, has to run from the file dialog handler or the software emulater
function systemLoadSequenceFile(filePath) {
    console.log("systemLoadSequenceFile() from " + filePath);

    mainObj = JSON.parse(fs.readFileSync(filePath, 'utf8').toString()); //loads the sequence and all it's variables
    //console.log(mainObj); //DEBUG

    //--------------------------------------------------------

    //convert the JSON strings to uint8 arrays where required
    mainObj.seq.forEach(function (obj, index) {
        //console.log(obj)
        if (sequenceValidateData(obj)) {
            pushMessageToLog("While loading sequence file, an error was found on sequence #" + (index + 1) + ", " + obj.name);
        }

        if (obj.mode != 'mList') {
            for (let i = 0; i < mainObj.seq[index].frames; i++) mainObj.seq[index].dataFrames[i] = Uint8Array.from(mainObj.seq[index].dataFrames[i]);
        }
        if (cSeqModesStartVals.includes(obj.mode)) {
            //can not be undefined, above validate would have fixed it
            if (obj.startVals != undefined) mainObj.seq[index].startVals = Uint8Array.from(mainObj.seq[index].startVals);
            if (obj.frameVar != undefined) mainObj.seq[index].frameVar = Uint8Array.from(mainObj.seq[index].frameVar);
        }
    });

    //--------------------------------------------------------
}

//========================================================================================================================================

function sequenceSelect(val, flag = false) {
    console.log("sequenceSelect(" + val + ")")

    if (val > mainObj.seq.length) {
        pushMessageToLog("Attempted to load a sequence that is not available. Ignoring.");
        return;
    }

    stateSeq.index = val; //set selected sequence index/id

    //Update States
    SelSeq = mainObj.seq[val]; //set global by reference, for more readable name
    sequenceValidateData(SelSeq); //just in case, should never catch anything
    stateSeq.selectedFrame = 0; //reset to select frame#1 as default

    if (flag == true) pushMessageToLog("Opened Sequence #" + (val + 1) + ". Name: " + SelSeq.name);

    if (document.getElementById('ws-timeline-container') != undefined) //not sure if this can happen, but be safe
        document.getElementById('ws-timeline-container').scrollTop = 0; //reset scroll to top

    uxFrameButtonsRender(); //destroys and rebuilds frame buttons
    uxToolsSequenceUpdate(); //disables/enables UX tool elements based on the sequence
    uxDisableEnabledTools(); //always I guess, just in case
    if (SelSeq.mode == 'mPOV') uxPOVElementStates();
    refreshLayouts(); //updates timeline/graphic layouts

    stateCommunication.timeStamp = Math.random(); //bit of a trick. Force autoHWPV update if enabled

    //stateUX.updateGraphic = true; //Ensure layouts get updated next time they are viewed
    //stateUX.updateTimeline = true;
    //layoutUpdateBuffers(); //ugh this is the only way.. otherwise doesn't run after render

    //stateUX.refreshGraphic = true; //Ensure layouts get updated next time they are viewed
    //stateUX.refreshTimeline = true;

    //--------------------------------------------------------------------
}

//========================================================================================================================================

function sequenceCopy(seqID) {
    console.log("sequenceCopy() " + seqID);
    //mainObj.seq.push(JSON.parse(JSON.stringify(mainObj.seq[seqID]))); //dirty but pure JS way to copy an object rather than reference
    //mainObj.seq.push(mainObj.seq[seqID]);


    let newObj = (JSON.stringify(mainObj.seq[seqID], function (k, v) {
        if (v instanceof Uint8Array) {
            return Array.apply([], v);
        }
        return v;
    })); //add it to first index - deep clone, it seemed to work but dataframes[][] are still by reference. confirmed, but why?.


    //mainObj.seq.push(JSON.parse(newObj));
    newObj = JSON.parse(newObj); //parse it with the copied array values, then fix the arrays

    //--------------------- Apply typecasting, similar to save sequence file -----------------

    let chanSz = sequenceGetChannelAmount(newObj); //shorten this

    if (newObj.startVals != null && newObj.startVals != undefined) {
        newObj.startVals = systemResize8Array(newObj.startVals, chanSz);
    }

    if (newObj.frameVar != null && newObj.frameVar != undefined) {
        newObj.frameVar = systemResize8Array(newObj.frameVar, chanSz);
    }

    if (newObj.dataFrames != null && newObj.dataFrames != undefined) {
        if (newObj.dataFrames.length > newObj.frames) {
            let newArr = Array.from(seq.dataFrames);
            newArr.length = newObj.frames;
            newObj.dataFrames = []; //clear it, think this is easier than pop()

            newArr.forEach(function (data, i) {
                newObj.dataFrames.push(newArr[i]);
            });
        }

        newObj.dataFrames.forEach(function (data, i) {
            if (newObj.mode == 'mGradient') newObj.dataFrames[i] = systemResize8Array(data, 4);
            else newObj.dataFrames[i] = systemResize8Array(data, chanSz);
        });
    } //end dataFrames if()

    //------------------ End copy --------------------

    mainObj.seq.push(newObj);

    //--------------------------------------

    stateSeq.index = (mainObj.seq.length - 1); //set new SelectedSequence, which is the last entry
    mainObj.seq[stateSeq.index].name = mainObj.seq[stateSeq.index].name + " - copy"; //add copy to indicate to user

    //3-6-2022 - disabled close menu and open sequence, added refresh overlaymenu - scroll down and focus?
    overlayMenuEvent(event, 'open', 'menu-1'); //then right away open the sequence select menu, which refreshes it
    //overlayMenuEvent(event, "close"); //close overlay menu, the only place copy can be ran from
    // sequenceSelect(stateSeq.index, true);  //now select it
}

//========================================================================================================================================

function sequenceDelete(seqID) {
    console.log("sequenceDelete() " + seqID);

    //delete mainObj.seq[seqID];
    mainObj.seq.splice(seqID, 1); //delete the indicated sequence

    //console.log(stateSeq.index, seqID);

    let list = document.getElementById("ovmenu-1-workspace").childNodes;
    //console.log(document.getElementById("ovmenu-1-workspace").childNodes);
    list[seqID].classList.add('ovmenu-sequence-listing-deleted'); //plays the collapse animation'


    //setTimeout to refresh OVMenu after animation - because now the IDs do not match what the user sees
    let refresh = setTimeout(ovmSequenceSelectRefresh, 250, "ovmenu-1-workspace"); //rather than detect end of transition, just do this

    //------------------------------ UPDATE INDEX ----------------------------------
    //If seqID is equal to to remove it and set slot to null/undefined indicator.
    //  greater than seqID, shift all down 1. to fill empty slot


    if (stateSeq.index == seqID) {
        //check if still in range
        //console.log(stateSeq.index, mainObj.seq.length);
        if (stateSeq.index >= mainObj.seq.length) {
            stateSeq.index = mainObj.seq.length - 1;
        }
        //console.log("stateSeq.index: " + stateSeq.index);
        //sequenceSelect(stateSeq.index); //select the sequence
        //overlayMenuEvent(event, 'open', 'menu-1'); //then right away open the sequence select menu, which refreshes it
        clearTimeout(refresh); //cancel the transition refresh timeout
        refresh = setTimeout(function () {
            //repurpose to delay doing this until the animation plays. Which is for action feedback
            sequenceSelect(stateSeq.index); //select the sequence
            overlayMenuEvent(event, 'open', 'menu-1'); //then right away open the sequence select menu, which refreshes it
        }, 250);
    }
    else if (stateSeq.index > seqID) stateSeq.index--;

    //----------------------------- UPDATE INDEX -----------------------------------

    //console.log(mainObj.userIndex);

    for (let i = 0; i < mainObj.userIndex.length; i++) {
        let val = mainObj.userIndex[i];
        console.log(i, val, seqID);
        if (val == seqID) {
            mainObj.userIndex.splice(i, 1); //no error handling or anything, just delete it from the index
            i--; //account for recent splice
            pushMessageToLog("The deleted sequence was on the Index and has been removed. Please review");
        }
        else if (val > seqID) {
            mainObj.userIndex[i]--; //adjusts higher index sequences downwards
        }
    }

    //console.log(mainObj.userIndex);

    //----------------------------- UPDATE ALL LIST MODE SEQUENCES -----------------------------------

    mainObj.seq.forEach(function (val, index) {
        if (val.mode == 'mList') {
            //console.log(index, val.name);
            //scan and update the .listIsD
            //console.log(val.listID)
            val.listID.forEach(function (value, i) {
                if (value == seqID) {
                    val.listID[i] = null; //seqslot#0 will be a null indicators
                }
                else if (value > seqID) {
                    //console.log("value needs to be updated " + i);
                    val.listID[i]--;
                }
            });
        }
    });
}

//========================================================================================================================================

function sequenceCreateNew() {
    console.log("sequenceCreateNew()");
    mainObj.seq.push(JSON.parse(JSON.stringify(seqTemplate))); //dirty but pure JS way to copy an object rather than reference
    stateSeq.index = (mainObj.seq.length - 1); //set new SelectedSequence, which is the last entry
    sequenceValidateData(mainObj.seq[stateSeq.index]); //setup few missing template properties
    overlayMenuEvent(event, "close");
    sequenceSelect(stateSeq.index, true); //now select it
    SelSeq.errorFlag = false; //errorFlag will always be set since data was built from new
}

//========================================================================================================================================

function sequenceImportData() {
    console.log("sequenceImportData()");
    // if (SelSeq.mode == 'mFade' || SelSeq.mode == 'mInstant') {
    //     overlayMenuEvent(null, 'open', 'menu-importwizard');
    // }
    // else
    if (SelSeq.mode == 'mPOV') fileImportImage('pov');
    else if (SelSeq.mode == 'mFile') fileLoadSeqDataFile();
    else console.log("This should not have ran");
}

//========================================================================================================================================

function sequenceInitImageWizard(filePath) {
    uxProgressBarInternal(20); //clocked it at 20 ticks, but this is probably better

    document.getElementById("ovmenu-importwizard-filespath").innerText = filePath;
    stateSeq.filePath = filePath;

    let prevCanvas = document.getElementById("ovmenu-importwizard-prevcanvas");
    let context = prevCanvas.getContext('2d');

    var img = new Image();
    img.addEventListener('load', function () {

        // document.getElementById().disabled = false;

        prevCanvas.width = this.width;
        prevCanvas.height = this.height;
        context.drawImage(this, 0, 0);

    });
    img.addEventListener('error', function () {
        console.error("ERROR");
        uxProgressBarHide(); //hide the progress bar an error occured - not sure if this is best
    });
    img.src = filePath;
}

//========================================================================================================================================

/*
0: "First pixel is top left. Rows are displayed top to bottom." - width is numPixels, height is frames
1: "First pixel is top left. Columns are displayed left to right." - width is frames, height is numPixels
2: "First pixel is the bottom left. Columns are displayed left to right" - width is frames, height is numPixels		
3: "First pixel is top right. Rows are displayed top to bottom."
*/

function sequenceLoadImageAsData(filePath, mode, action) {
    console.log("sequenceLoadImageAsData() Mode:", mode, " Action:", action, "Path:", filePath)
    SelSeq.filePath = filePath;

    //------ Progress bar won't update since the for loop is syncs --------------------

    uxProgressBarInternal(20); //clocked it at 20 ticks, but this is probably better

    //---------------------------------- ENd progress bar fake -------------------------------

    //timeout is also to help the progress bar fake, otherwise remove it - delays 500mS, to allow the bar to progress a little bit
    setTimeout(function () {
        var img = new Image();
        img.addEventListener('load', function () {
            //console.log("Starting POV conversion");
            //prepare offscreen canvas, so the image can be drawn to it and then the pixel RGB values read
            let canvas = new OffscreenCanvas(this.width, this.height);
            let context = canvas.getContext('2d');
            context.drawImage(this, 0, 0);

            SelSeq.colorMode = "RGB"; //all images are RGB - user can switch to RGBW and utilize the conversion func

            //forgot rather than getting pixel by pixel with getImageData(), the whole image can be cached to an array, but as one long value, not height/width
            //console.log(context.getImageData(1, 1, 16, 16).data);

            let imgData = context.getImageData(0, 0, canvas.width, canvas.height).data; //oofda! A lot of data
            console.log(imgData)

            let buildFrame = [];

            switch (mode) {
                default:
                case 0: //"First pixel is top left. Rows are displayed top to bottom." - width is numPixels, height is frames
                    SelSeq.numPixels = this.width;
                    SelSeq.frames = this.height;

                    SelSeq.dataFrames = [];
                    for (let q = 0; q < SelSeq.frames; q++) {
                        buildFrame = [];
                        for (let i = 0; i < SelSeq.numPixels; i++) {
                            let offset = ((q * SelSeq.numPixels) + i) * 4; //4 is RGB+Alpha, which canvas uses
                            buildFrame.push(imgData[offset]); //red
                            buildFrame.push(imgData[offset + 1]); //green
                            buildFrame.push(imgData[offset + 2]); //blue
                        } //end for()
                        SelSeq.dataFrames.push(Uint8Array.from(buildFrame));
                    }//end for()
                    break;
                case 1: //"First pixel is top left. Columns are displayed left to right." - width is frames, height is numPixels
                    SelSeq.numPixels = this.height;
                    SelSeq.frames = this.width;

                    SelSeq.dataFrames = [];
                    for (let q = 0; q < SelSeq.frames; q++) {
                        buildFrame = [];
                        for (let i = 0; i < SelSeq.numPixels; i++) {
                            let offset = ((i * SelSeq.frames) + q) * 4; //4 is RGB+Alpha, which canvas uses
                            buildFrame.push(imgData[offset]); //red
                            buildFrame.push(imgData[offset + 1]); //green
                            buildFrame.push(imgData[offset + 2]); //blue
                            //+3 is transparency
                        } //end for()
                        SelSeq.dataFrames.push(Uint8Array.from(buildFrame));
                    }//end for()
                    break;
                case 2: //"First pixel is the bottom left. Columns are displayed left to right" - width is frames, height is numPixels	
                    SelSeq.numPixels = this.height;
                    SelSeq.frames = this.width;

                    SelSeq.dataFrames = [];
                    for (let q = 0; q < SelSeq.frames; q++) {
                        buildFrame = [];
                        for (let i = 0; i < SelSeq.numPixels; i++) {
                            let offset = ((((SelSeq.numPixels - 1) - i) * SelSeq.frames) + q) * 4; //4 is RGB+Alpha, which canvas uses
                            buildFrame.push(imgData[offset]); //red
                            buildFrame.push(imgData[offset + 1]); //green
                            buildFrame.push(imgData[offset + 2]); //blue
                            //+3 is transparency
                        } //end for()
                        SelSeq.dataFrames.push(Uint8Array.from(buildFrame));
                    }//end for()
                    break;
                case 3: //"First pixel is top right. Rows are displayed top to bottom." - width is numPixels, height is frames
                    SelSeq.numPixels = this.width;
                    SelSeq.frames = this.height;

                    SelSeq.dataFrames = [];
                    for (let q = 0; q < SelSeq.frames; q++) {
                        buildFrame = [];
                        for (let i = 0; i < SelSeq.numPixels; i++) {
                            let offset = ((q * SelSeq.numPixels) + ((SelSeq.numPixels - 1) - i)) * 4; //4 is RGB+Alpha, which canvas uses
                            buildFrame.push(imgData[offset]); //red
                            buildFrame.push(imgData[offset + 1]); //green
                            buildFrame.push(imgData[offset + 2]); //blue
                        } //end for()
                        SelSeq.dataFrames.push(Uint8Array.from(buildFrame));
                    }//end for()
                    break;
                case 10: //'mode2d':
                    SelSeq.numPixels = mainObj.layout.patchCoords.length;
                    if (action == 'scene') {
                        SelSeq.frames = 1;
                        SelSeq.mode = "mScene";
                        SelSeq.dataFrames = [];
                    }
                    else {
                        if (SelSeq.mode == 'mGradient') SelSeq.mode = "mFade";
                        else if (SelSeq.mode == 'mScene') SelSeq.mode = "mInstant";
                    }

                    buildFrame = [];

                    let pixMapCoords = mainObj.layout.patchCoords.map(e => e.split(",").map(ee => Number(ee)));
                    // console.log(pixMapCoords);
                    let maxW = Math.max(...pixMapCoords.map(e => e[0]));
                    let maxH = Math.max(...pixMapCoords.map(e => e[1]));

                    console.log("MAXW:", maxW, "MAXH:", maxH);
                    for (let i = 0; i < pixMapCoords.length; i++) {
                        if (pixMapCoords[i][0] > this.width || pixMapCoords[i][1] > this.height) {
                            buildFrame.push(0);
                            buildFrame.push(0);
                            buildFrame.push(0);
                        }
                        else {
                            let q = ((pixMapCoords[i][0] - 1) + ((pixMapCoords[i][1] - 1) * this.width)) * 4;
                            buildFrame.push(imgData[q++]);
                            buildFrame.push(imgData[q++]);
                            buildFrame.push(imgData[q++]);
                        }
                    }
                    //   console.log(buildFrame);

                    if (action == 'scene') SelSeq.dataFrames.push(Uint8Array.from(buildFrame));
                    else if (action == 'addframe') {
                        SelSeq.dataFrames = SelSeq.dataFrames.splice(0, SelSeq.frames);
                        SelSeq.dataFrames.push(Uint8Array.from(buildFrame));
                        SelSeq.frames = SelSeq.dataFrames.length;
                    }
                    else if (action == 'currentframe') SelSeq.dataFrames[stateSeq.selectedFrame] = Uint8Array.from(buildFrame);
                    break;
            } //end switch

            //console.log(SelSeq.dataFrames);
            console.log("Used POV variables: ", SelSeq.povDirMode, SelSeq.numPixels, SelSeq.frames);

            uxProgressBarHide(); //now hide the progress bar

            toolResetStartValues(SelSeq);
            uxFrameButtonsRender(); //destroys and rebuilds frame buttons
            uxToolsSequenceUpdate(); //update state changes to ux tool elements
            refreshLayouts();
        });
        img.addEventListener('error', function () {
            console.error("ERROR");
            uxProgressBarHide(); //hide the progress bar an error occured - not sure if this is best
        });
        img.src = filePath;
    }, 500);
}

//========================================================================================================================================

function sequenceAddToList(seqID, slotID) {
    console.log("sequenceAddToList() " + seqID + "   " + slotID);
    SelSeq.listID[slotID] = seqID;
    layoutListRender();
}

//========================================================================================================================================

function sequenceSetPixelData(seq, type, pixNum, frame, arr) {
    let colorMult = getColorModeMult(seq.colorMode); //Mono, RGB or RGBW multiplier
    let nextFrame = 0;
    let pixDataOffset; //handle fade vs gradient
    let pixFrameOffset = pixNum * colorMult;

    //let scratchArr = [];

    switch (type) {
        case 'frameVar':
            if (cDataSeqModesDataFrames.includes(seq.mode)) {
                //scratchArr = Array.from(seq.frameVar)
                // scratchArr.splice(pixFrameOffset, colorMult, new Array(colorMult).fill(frame));
                switch (seq.colorMode) {
                    case 'Mono':
                        seq.frameVar[pixFrameOffset] = frame;
                        break;
                    case 'RGBW':
                        seq.frameVar[pixFrameOffset + 3] = frame;
                    case 'RGB':
                        seq.frameVar[pixFrameOffset] = frame;
                        seq.frameVar[pixFrameOffset + 1] = frame;
                        seq.frameVar[pixFrameOffset + 2] = frame;
                        break;
                }
            }
            break;
        case 'startVal':
            if (cSeqModesStartVals.includes(seq.mode)) {
                switch (seq.colorMode) {
                    case 'Mono':
                        seq.startVals[pixFrameOffset] = arr[3];
                        break;
                    case 'RGBW':
                        seq.startVals[pixFrameOffset + 3] = arr[3];;
                    case 'RGB':
                        seq.startVals[pixFrameOffset] = arr[0];
                        seq.startVals[pixFrameOffset + 1] = arr[1];
                        seq.startVals[pixFrameOffset + 2] = arr[2];
                        break;
                }
                // sequenceLoadStartValPositions(seq, pixNum); //update startValPos float based on new start values
            }
            break;
    }
}

//========================================================================================================================================
function sequenceGetPixelData(seq, type, pixNum, frameVar) {
    // console.log("sequenceGetPixelData()",seq, type, pixNum, frameVar)
    let colorMult = getColorModeMult(seq.colorMode); //Mono, RGB or RGBW multiplier
    let nextFrame = 0;
    let pixDataOffset = pixNum * colorMult;
    let pixFrameOffset = pixNum * colorMult;
    if (seq.mode == 'mGradient') pixDataOffset = 0;

    if (frameVar == (seq.frames - 1)) nextFrame = 0; //wrap to first frame
    else nextFrame = (frameVar + 1); //target next frame
    //console.log("Pix#" + pixNum + "    pixDataOffset: ", pixDataOffset, "   Frames: ", frameVar + " to ", nextFrame);

    let retBegin = [];
    let retEnd = [];
    let retVal = [];
    let retFrames = [];

    //--------------------------- Start Values ----------------------------------------
    if (cSeqModesStartVals.includes(seq.mode)) {
        retVal = seq.startVals.slice(pixFrameOffset, (pixFrameOffset + colorMult));
    }
    //--------------------------- frameVars ----------------------------------------
    if (cDataSeqModesDataFrames.includes(seq.mode)) {
        retFrames = seq.frameVar.slice(pixFrameOffset, (pixFrameOffset + colorMult));
    }
    //--------------------------- Start and End Frames ----------------------------------------

    //  console.log(frameVar, pixDataOffset, colorMult);
    retBegin = seq.dataFrames[frameVar].slice(pixDataOffset, (pixDataOffset + colorMult));
    retEnd = seq.dataFrames[nextFrame].slice(pixDataOffset, (pixDataOffset + colorMult));

    if (type == 'begin') return retBegin;
    else if (type == 'end') return retEnd;
    else if (type == 'start') return retVal;
    else if (type == 'frames') return retFrames;
}

//========================================================================================================================================

//Runs layoutStartIndicatorPositionSet() for every pixel
function layoutTimelineIndicatorUpdate(seq, start, end) {
    // console.log("layoutTimelineIndicatorUpdate()",start,end);
    let q = 0;
    for (let i = start; i < end; i++) {
        //updates all them if they exist
        if (document.getElementById("timeline-indicator" + q) != undefined) layoutStartIndicatorPositionSet(q, i, stateSeq.startValPos[i]);
        q++;
    }
}

//========================================================================================================================================

//Positions a single indicator to the position indicated by stateSeq.startValPos[]
function layoutStartIndicatorPositionSet(elemID, pixelID, offset) {
    // console.log("layoutStartIndicatorPositionSet() ",elemID, pixelID, offset);

    let indicator = document.getElementById("timeline-indicator" + elemID);
    if (indicator == null) return; //ERROR! - not sure what to do or if it would actually happen
    //-------------------------------- Update UX stuff --------------------------------------------

    const halfW = (indicator.scrollWidth / 2);
    let newX = (stateUX.indicatorMax * offset) - halfW - (stateUX.indicatorStep * timelineViewport.offsetX);
    indicator.style.left = newX + "px"; //update indicator location, X only - updates right away since it has a transition

    //-------------------------------- Load Initial Tooltip Values --------------------------------------------

    let colorMult = getColorModeMult(SelSeq.colorMode); //Mono, RGB or RGBW multiplier
    let frame = SelSeq.frameVar[pixelID * colorMult];

    let startValArr; // will fill either with startVal[] or the dataframe color for instant
    if (cSeqModesStartVals.includes(SelSeq.mode)) startValArr = sequenceGetPixelData(SelSeq, 'start', pixelID, frame);
    else if (SelSeq.mode == 'mInstant') startValArr = sequenceGetPixelData(SelSeq, 'begin', pixelID, frame);
    //else nothing

    let startArr = sequenceGetPixelData(SelSeq, 'begin', pixelID, frame);
    let endArr = sequenceGetPixelData(SelSeq, 'end', pixelID, frame);
    let preload = (offset / (1 / SelSeq.frames)) - frame;
    const toolTipDiv = document.getElementById("global-tooltip");

    //update tooltip with values - already loaded in the per sequence mode switch
    if (isArrayEqual(startArr, endArr)) toolTipDiv.innerHTML = 'Frame: ' + (frame + 1) + '<br>Position: <br>' + Math.floor(preload * 100) + '%<br>';
    else {
        if (SelSeq.colorMode == "RGB" || SelSeq.colorMode == "RGBW") toolTipDiv.innerHTML = 'Red: ' + startValArr[0] + '<br>Green: ' + startValArr[1] + '<br>Blue: ' + startValArr[2] + '';
        else if (SelSeq.colorMode == "Mono") toolTipDiv.innerHTML = 'Value: ' + startValArr[0];
        if (SelSeq.colorMode == "RGBW") toolTipDiv.innerHTML += '<br>White: ' + startValArr[3];
    }

    indicator.setAttribute('data-tiptext', toolTipDiv.innerHTML); //update indicator element with new tooltip value

    //---------------------------------------------------------------------------------------------
}

//========================================================================================================================================

function sequenceValidateData(seq) {
    //Paths for checking: 
    //     edit Sequence mode, edit pixel amount, edit frame amount, edit color mode, 
    //console.log("sequenceValidateData() " + seq.name); //DEBUG

    let chanSz = sequenceGetChannelAmount(seq); //shorten this
    let errFlag = false;
    let errStartFlag = false;

    //---------------------------------------------------------------------------------------------

    if (cSeqModesStartVals.includes(seq.mode)) {
        if (seq.startVals == undefined || seq.startVals == null) {
            console.log("start values array is missing.");
            seq.startVals = new Uint8Array(chanSz); //make a new array
            errFlag = true;
        }
        else if (seq.startVals.length < chanSz) {
            console.log("start values are incomplete. Expanding. " + seq.startVals.length, seq.numPixels);
            let errStartVal = seq.startVals.length; //starting number to expand from, can not do it until frameVar is up
            let newArr = Array.from(seq.startVals); //expand what is defined
            newArr.length = chanSz;

            //Repairs the start values, for gradients. Fades don't need to as the start values of 0,0,0 will match the dataframe[][] of 0,0,0
            if (seq.mode == 'mGradient') {
                let colorMult = getColorModeMult(seq.colorMode); //Mono, RGB or RGBW multiplier
                while (errStartVal < newArr.length) {
                    let startArr = sequenceGetPixelData(seq, 'begin', (errStartVal / colorMult), 0); //returns RGB or RGBW values for starting frame
                    newArr.splice(errStartVal, colorMult, ...startArr);
                    errStartVal += colorMult;
                    errStartFlag = true;
                } //end while
            }
            seq.startVals = Uint8Array.from(newArr);
            errFlag = true;
        }
    } // end cSeqModesStartVals check

    //---------------------------------------------------------------------------------------------

    if (cDataSeqModesDataFrames.includes(seq.mode)) {
        if (seq.frameVar == undefined || seq.frameVar == null) {
            console.log("frameVar array is missing.");
            seq.frameVar = new Uint8Array(chanSz); //make a new array
            errFlag = true;
        }
        else if (seq.frameVar.length < chanSz) {
            console.log("frameVar are incomplete. Expanding. " + seq.frameVar.length, seq.numPixels);
            let newArr = Array.from(seq.frameVar); //expand what is defined
            newArr.length = (chanSz);
            seq.frameVar = new Uint8Array(newArr);
            errFlag = true;
        }

        seq.frameVar.forEach(function (v, i) {
            if (seq.frameVar[i] >= seq.frames) seq.frameVar[i] = 0; //if frames are added or removed this fixes it - just resets to 0
        });
    }

    //---------------------------------------------------------------------------------------------

    //console.log("dataframes " + seq.dataFrames.length, seq.dataFrames[0].length, seq.frames, seq.numPixels);
    if (seq.mode != 'mList') {
        //mList is only mode with out dataFrames
        if (!seq.hasOwnProperty("dataFrames")) {
            console.log("Dataframes not found");
            seq.dataFrames = []; //make it an array
            while (seq.dataFrames.length < seq.frames) {
                console.log("ERROR - Dataframes are incomplete. Expanding");
                if (seq.mode == 'mGradient') seq.dataFrames.push(new Uint8Array(4));
                else seq.dataFrames.push(new Uint8Array(chanSz));
            }
            errFlag = true;
        }
        if (seq.dataFrames.length < seq.frames) {
            console.log("ERROR - Dataframes are incomplete. Expanding");
            while (seq.dataFrames.length < seq.frames) {
                if (seq.mode == 'mGradient') seq.dataFrames.push(new Uint8Array(4));
                else seq.dataFrames.push(new Uint8Array(chanSz));
            }
            errFlag = true;
        }

        seq.dataFrames.forEach(function (array, index) {
            //console.log(array.length, chanSz);
            if (seq.mode == 'mGradient') {
                if (array.length < 4) {
                    //console.log("ERROR - fixed dataframes[" + index + "], too short, expanding", array.length, chanSz);
                    let newArr = Array.from(array); //expand what is defined
                    newArr.length = 4; //resize array, can not resize typed arrays
                    seq.dataFrames[index] = new Uint8Array(newArr);
                    errFlag = true;
                }
            }
            else {
                if (array.length < chanSz) {
                    //console.log("ERROR - fixed dataframes[" + index + "], too short, expanding", array.length, chanSz);
                    let newArr = Array.from(array); //expand what is defined
                    newArr.length = chanSz; //resize
                    seq.dataFrames[index] = new Uint8Array(newArr);
                    errFlag = true;
                }
            }
        });
    }

    //---------------------------------------------------------------------------------------------

    if (errStartFlag) {
        sequenceResetStartValPositions(seq); //wait til after dataFrames and vars are updated
    }

    //---------------------------------------------------------------------------------------------

    seq.errorFlag = errFlag; //save to it to sequence
    return errFlag;
}

//========================================================================================================================================

function sequenceMeasure(seq) {
    const cSeqHeaderLength = 16;
    let seqChans = sequenceGetChannelAmount(seq);
    let frames = seq.frames; //base 1
    let memory = 0;
    let devChans = stateDevice.soft.channels;

    //adjusts it for device.channels cropping. If sequence specifies more pixels than device supports
    let chanSz = seqChans;

    if (stateDevice.soft.lockChannelSize) chanSz = devChans; //or if the device is locksChannelSize, force it to the same size. No need to check if over or under.
    else {
        if (seq.mode == 'mFile') {
            if (stateDevice.soft.hasOwnProperty('channelsExt')) {
                if (stateDevice.soft.channelsExt > seqChans) chanSz = stateDevice.soft.channelsExt; //sequence defines channel size larger than extended maximum 
            }
            else if (devChans < seqChans) chanSz = devChans; //device does not have maximum extended channels defined
        }
        else if (devChans < seqChans) chanSz = devChans; //if sequence specifies more than device supports
    }

    let frameSz = frames;
    if (cDataSeqModeGlobal.includes(seq.mode)) {
        if (frames > stateDevice.soft.maxFramesExt) frameSz = stateDevice.soft.maxFramesExt; //set to extended max, 65,535
    }
    else {
        if (frames > stateDevice.soft.maxFrames) frameSz = stateDevice.soft.maxFrames; //set to standard max, 256
    }

    //-----------------------------------------------

    memory += cSeqHeaderLength;

    if (cSeqModesStartVals.includes(seq.mode)) {
        memory += chanSz; //startvals
        if (stateDevice.soft.bit16Data == true) memory += chanSz; //add it again for 16-bit color
    }

    if (seq.mode == 'mInstant' || seq.mode == 'mScene') { //&& version 2 firmware, this needs to get fixed. Startvals are not required
        memory += chanSz; //dummy startvals
        memory += (frameSz * 2); //dummy slide values and slide flags
    }

    if (cDataSeqModesDataFrames.includes(seq.mode) || seq.mode == 'mScene') { //v2 scene emulation
        memory += chanSz; //frameVars, same for 8-bit and 16-bit color
    }

    if (cDataSeqModeGlobal.includes(seq.mode)) { //['mPOV', 'mFile', 'mScene'];
        //nothing special
    }

    if (seq.mode == 'mList') {
        memory += (frameSz * 4); //3 values, 2x 8-bit, 1x 16bit = 4 bytes per frame

    }
    else if (seq.mode == 'mGradient') {
        memory += (frameSz * 4); //always 4 values per frame
        if (stateDevice.soft.bit16Data == true) memory += (frameSz * 4); //add it again for 16-bit data values
    }
    else {
        //all the other modes have normal dataframes
        memory += (frameSz * chanSz);
        if (stateDevice.soft.bit16Data == true) memory += (frameSz * chanSz); //add it again for 16-bit data values
    }

    //console.log("sequenceMeasure() with " + seq.name + " resulted with " + memory)

    seq.memory = memory; //save it to referenced object
    return memory;
}

//========================================================================================================================================

function sequenceMeasureAll() {
    console.log("sequenceMeasureAll()");
    if (mainObj.seq.length > 0) {
        mainObj.seq.forEach(function (seq, index) {
            sequenceMeasure(mainObj.seq[index]); // couldn't use seq for whatever reason
        });
    }
}

//========================================================================================================================================

function sequenceGetChannelAmount(seq) {
    return (getColorModeMult(seq.colorMode) * seq.numPixels);
}

//========================================================================================================================================

function sequenceIsIndexed(seqID) {
    if (mainObj.userIndex.includes(seqID)) return '#' + (seqID + 1);
    else return 'No';
}

//========================================================================================================================================

function sequenceConvertMode(mode) {
    switch (mode) {
        case 'mFade':
            return 'Fade';
        case 'mGradient':
            return 'Gradient';
        case 'mInstant':
            return 'Instant';
        case 'mPOV':
            return 'POV'
        case 'mFile':
            return 'File'
        case 'mList':
            return 'List'
        case 'mScene':
            return 'Scene';
        default:
            return 'error';
    }//end switch
}

//========================================================================================================================================

function sequenceMemoryLabel(memory) {
    //convert number into KB, MB
    let val = 0;
    if (memory >= 1000000) val = (Math.round((memory / 1000000) * 10) / 10) + "MB";
    else if (memory >= 1000) val = (Math.round((memory / 1000) * 10) / 10) + "KB";
    else val = memory + " bytes";
    return val;
}

//========================================================================================================================================

//Set startVal[] based on stateSeq.startValPos[] values
function sequenceSetStartValues(seq, pix) {
    //console.log("sequenceSetStartValues() " + pix + "    " + stateSeq.startValPos[pix]);
    //console.log("DEBUG sequenceSetStartValues() " + pix + "    " + stateSeq.startValPos[pix]);
    let colorMult = getColorModeMult(seq.colorMode); //Mono, RGB or RGBW multiplier
    let pixFrameOffset = pix * colorMult;
    const frameDiv = (1 / seq.frames); //float value of size of each frame

    let frame = Math.floor(stateSeq.startValPos[pix] / frameDiv);

    // console.log(stateSeq.startValPos[pix], frameDiv, frame, colorMult);

    if (frame < 0) frame = 0; //couldn't happen, probably
    else if (frame >= seq.frames) frame = (seq.frames - 1); //if dragged all the way to the high end

    if (seq.mode == 'mFade' || seq.mode == 'mGradient') {
        let startArr = sequenceGetPixelData(seq, 'begin', pix, frame); //returns RGB or RGBW values for starting frame
        let endArr = sequenceGetPixelData(seq, 'end', pix, frame);   //returns RGB or RGBW values for ending frame
        let preload = (stateSeq.startValPos[pix] / frameDiv) - frame;
        let preloadVal = Math.floor(preload * 255);

        //console.log(frame, stateSeq.startValPos[pix], preload, preloadVal);
        //console.log(startArr);
        //console.log(endArr);

        //detects frame same condition(preload)
        if (isArrayEqual(startArr, endArr) && seq.softwarePreview != true) {
            //start and end color is the same, it must be preloading. apply preload value - preloadVal
            //console.log("preload");
            localHelper(frame, preloadVal);
        }
        else {
            //start and end color are different fill normal start values
            let valueArr = []; //init
            if (colorMult == 1) {
                valueArr.push(helper(startArr[0], endArr[0]));
            }
            else if (colorMult == 3 || colorMult == 4) {
                valueArr.push(helper(startArr[0], endArr[0]));
                valueArr.push(helper(startArr[1], endArr[1]));
                valueArr.push(helper(startArr[2], endArr[2]));
                if (colorMult == 4) valueArr.push(helper(startArr[3], endArr[3]));
            }

            //console.log(valueArr);
            localHelper(frame, valueArr);

            function helper(start, end) {
                //console.log("helper() " + start, end)
                let val;
                if (start > end) {
                    //console.log(">"); //going down, start larger than end
                    val = start - ((start - end) * preload);
                }
                else if (start < end) {
                    //console.log("<"); //going up,  end larger than start
                    val = start + ((end - start) * preload);
                }
                else if (start == end) {
                    //console.log("=");
                    val = start;
                }
                //console.log("val = " + val + "   preload: " + preload);
                return Math.round(val);
            }
        }
    }
    else if (seq.mode == 'mInstant') {
        //snap to frame start(left)
        //console.log(position, frameDiv, frame, seq.frames);
        sequenceSetPixelData(seq, 'frameVar', pix, frame);
    }
    else {
        console.log("It should never have gotten here", seq.mode);
    }

    //--------------------------------------------------------------------

    function localHelper(frameVar, value) {
        if (value.length > 0) {
            //sent color not preload value, do nothing
        }
        else {
            //sent preload, not a color, expand to an array
            let hold = value;
            value = [];
            value.push(hold, hold, hold, hold); //expand to RGBW, doesn't matter if mono or RGB
        }

        sequenceSetPixelData(seq, 'frameVar', pix, frameVar);
        sequenceSetPixelData(seq, 'startVal', pix, frameVar, value);
    }

    //--------------------------------------------------------------------
}

//========================================================================================================================================

function sequencesReset() {
    console.log("sequencesReset()");
    mainObj.seq = []; //simple, only removes the sequence array, not device, idle or other top level entries
    indexReset();
}

//========================================================================================================================================

function sequenceSlideFrame(seq, frame, dir) {
    //Not added and probably won't be
}

//========================================================================================================================================

function sequenceRenderInit(seq) {
    console.log("Rendering Sequence Data for:", seq.name);

    let cloneSeq = { ...seq };
    if (seq.hasOwnProperty("startVals")) cloneSeq.startVals = Array.from(seq.startVals);
    if (seq.hasOwnProperty("frameVar")) cloneSeq.frameVar = Array.from(seq.frameVar);

    cloneSeq.softwarePreview = true; //tracker to prevent preload injection
    stateUX.softwarePlayCount = 0;
    seqWriteToFileBuf = []; //clear global data buffer
    let flag = false;
    while (!flag) {
        flag = sequenceRender(seq, cloneSeq, null, false)
    }
    console.log("DONE");
}

//========================================================================================================================================

function sequenceRender(seq, cloneSeq, theIcons, flag) {
    // console.log("sequenceRender()");
    //flag = true, playback preview, flag = false, render data to file
    let mult = getColorModeMult(SelSeq.colorMode);
    var dataFrame = [];

    switch (seq.mode) {
        case 'mFade':
        case 'mGradient':
            let stepValue = ((1 / seq.frames) / 255);
            // if (seq.speed < 25) stepValue = stepValue * (25 / seq.speed);

            stateSeq.progress += (100 / (seq.frames * 255)); //not right but whatever
            if (stateSeq.progress > 100) stateSeq.progress = 0;

            for (let i = 0; i < seq.numPixels; i++) {
                stateSeq.startValPos[i] += stepValue;
                if (stateSeq.startValPos[i] > 1) stateSeq.startValPos[i] -= 1;
                sequenceSetStartValues(cloneSeq, i);

                //MICROSOFT SD CONTROLLER FIX
                let data = sequenceGetPixelData(cloneSeq, 'start', i, 0);
                let rearrange = [...data];
                data[0] = data[1];
                data[1] = rearrange[0];
                dataFrame.push(...data);
                // dataFrame.push(...sequenceGetPixelData(cloneSeq, 'start', i, 0)); //ORIGINAL


                if (flag) {
                    if (i >= timelineViewport.offsetY && i < (timelineViewport.offsetY + timelineViewport.cellsHigh)) layoutStartIndicatorPositionSet((i - timelineViewport.offsetY), i, stateSeq.startValPos[i]); //just for timeline indicators
                }
            }
            stateUX.softwarePlayCount++;
            break;
        case 'mInstant':
            stateSeq.progress += (100 / seq.frames); //not right but whatever
            if (stateSeq.progress > 100) stateSeq.progress = 0;

            for (let i = 0; i < seq.numPixels; i++) {
                stateSeq.startValPos[i] += (1 / seq.frames);
                if (stateSeq.startValPos[i] >= 1) stateSeq.startValPos[i] = 0;
                console.log(cloneSeq, 'begin', i, cloneSeq.frameVar[i * mult], mult);
                dataFrame.push(...sequenceGetPixelData(cloneSeq, 'begin', i, cloneSeq.frameVar[i * mult]));
                cloneSeq.frameVar[i * mult]++;
                if (cloneSeq.frameVar[i * mult] >= seq.frames) cloneSeq.frameVar[i * mult] = 0;

                //causes errors when in RGBW mode
                //  if (i >= timelineViewport.offsetY && i < (timelineViewport.offsetY + timelineViewport.cellsHigh)) layoutStartIndicatorPositionSet(i, stateSeq.startValPos[i]); //just for timeline indicators
            }
            break;

        case 'mFile':
        case 'mPOV':
            //console.log("GLOBAL" + stateUX.softwarePlayCount);
            stateSeq.progress += (100 / seq.frames); //not right but whatever
            if (stateSeq.progress > 100) stateSeq.progress = 0;

            if (stateUX.softwarePlayCount >= seq.frames) stateUX.softwarePlayCount = 0;

            for (let i = 0; i < seq.numPixels; i++) {
                dataFrame.push(...sequenceGetPixelData(cloneSeq, 'begin', i, stateUX.softwarePlayCount));
            }
            stateUX.softwarePlayCount++;
            break;

        case 'mScene':
            //nothing happens!
            break;

        case 'mTechnical':
        case 'mList':
            //can not possibly get here
            break;
    } //end switch

    if (flag) {
        let index = 0;
        for (let i = 0; i < seq.numPixels; i++) {
            let getColors = () => {
                if (cloneSeq.colorMode == 'Mono') return dataFrame[index++];
                if (cloneSeq.colorMode == 'RGB') return [dataFrame[index++], dataFrame[index++], dataFrame[index++]];
                if (cloneSeq.colorMode == 'RGBW') return [dataFrame[index++], dataFrame[index++], dataFrame[index++], dataFrame[index++]];
            }
            let colors = getColors();

            if (i < theIcons.length) {
                theIcons[i].style.backgroundColor = rgbToHex(...colors);
                if (cloneSeq.colorMode == 'RGBW') {
                    theIcons[i].style.backgroundImage = "linear-gradient(transparent 60%," + rgbToHex(colors[3], colors[3], colors[3]) + " 40%)";
                }
            }
        }
    }
    else {
        //  if (stateDevice.soft.indexMemoryModel == 99) {
        //devices no connection, write to file
        seqWriteToFileBuf.push(...dataFrame);

        if (seq.mode == 'mFile' || seq.mode == 'mPOV') {
            if (stateUX.softwarePlayCount >= seq.frames) {
                writeBinFile();
                return true;
            }
        }
        else {
            if (stateUX.softwarePlayCount == (256 * seq.frames)) {
                writeBinFile();
                return true;
            }
        }
        //  }


        function writeBinFile() {
            console.log("Sequence Render COMPLETE", stateUX.softwarePlayCount);
            cloneSeq.softwarePreview = false; //why cloneSeq will be disposed
            //DEBUG
            let header = new Array(16).fill(0);
            header[0] = ((seq.numPixels * mult) >> 8);//Data Size MSB
            header[1] = ((seq.numPixels * mult) & 0xFF);//Data Size LSB
            header[2] = (seq.speed >> 8);
            header[3] = (seq.speed & 0xFF);
            let numOfFrames = seqWriteToFileBuf.length / (seq.numPixels * mult);
            header[4] = (numOfFrames >> 24); //number of frames
            header[5] = (numOfFrames >> 16);
            header[6] = (numOfFrames >> 8);
            header[7] = (numOfFrames & 0xFF);
            console.log(header);
            seqWriteToFileBuf.unshift(...header);

            // fs.writeFile(path.join(process.cwd(), '/bin/'+seq.name.replace( /[<>:"\/\\|?*]#+/g, '' ).replace(" ","")+'.bin'), new Uint8Array(seqWriteToFileBuf), () => {
            //     console.log("file write complete", seqWriteToFileBuf.length);
            // });
        }
    }

    return false;
}


