
//========================================================================================================================================

"use strict";

//========================================================================================================================================

function seqEditName(elem) {
    console.log("seqEditName()");
    systemUndoRedoPush('seq');
    SelSeq.name = elem.value;
}
//========================================================================================================================================

function seqEditDescription(elem) {
    console.log("seqEditDescription()");
    SelSeq.description = elem.value;
}

//========================================================================================================================================

function seqEditMode(elem) {
    console.log("seqEditMode(), changed to " + elem.options[elem.selectedIndex].value + "  from  " + SelSeq.mode);
    //systemUndoRedoPush('seq'); 
    let newMode = elem.options[elem.selectedIndex].value;
    let oldMode = SelSeq.mode;
    SelSeq.mode = newMode; //update the object state

    //---------------------------- Special Sequence Mode Functions ----------------------------

    if (SelSeq.mode == 'mScene') {
        SelSeq.frames = 1;
        uxFrameButtonsRender(); //destroys and rebuilds frame buttons

    }
    else if (SelSeq.frames < 2) SelSeq.frames = stateSeq.minFrames; //only will happen if was previously mScene or had an error

    //------------------------------- Converts dataframes to the sequence mode's format ----------------------------

    //creates dataframes
    if (oldMode == 'mList') {
        console.log("Previous sequence mode was list- handle it ");

        if (SelSeq.numPixels < 1 || SelSeq.numPixels == undefined) SelSeq.numPixels = 1;

        if (SelSeq.dataFrames == undefined) {
            let newFrame = new Uint8Array(sequenceGetChannelAmount(SelSeq)); //need to set numPixels before running this
            newFrame.fill(0);
            //console.log(newFrame);
            //set to a default
            SelSeq.dataFrames = [];
            SelSeq.dataFrames.push(newFrame);
            SelSeq.dataFrames.push(newFrame);
            SelSeq.frames = 2;
            SelSeq.startValues = [];
            SelSeq.startValues.push(newFrame);
            SelSeq.frameVar = [];
            SelSeq.frameVar.push(newFrame);
        }

        if (SelSeq.frames > SelSeq.dataFrames.length) SelSeq.frames = SelSeq.dataFrames.length;

        //updatePixelAmount(); //need to create the dataframes first
        sequenceValidateData(SelSeq); //replacement
    }

    if (newMode == 'mList') {
        if (SelSeq.listID == undefined) {
            //set to a default
            SelSeq.listID = [0, 0];
            SelSeq.listIteration = [1, 1];
            SelSeq.listFlags = [0, 0];
            SelSeq.frames = 2;
        }
        else SelSeq.frames = SelSeq.listID.length;

        SelSeq.transition = "transInstant"; //can only be instant, at least for now. Version 2 firmware gets messed up if try to transition in
        SelSeq.playMode = "playLoop";
    }

    if (oldMode == 'mGradient' && (SelSeq.mode == 'mFade' || SelSeq.mode == 'mInstant')) {
        //mode was gradient now is a fade, reformat dataframes
        //console.log("CHANGED TO FADE - UPDATE DATA");

        SelSeq.dataFrames.forEach(function (array, index) {
            let newArr = Array.from(array); //best way to expand the typed array, copy -> destory -> create
            newArr.length = (sequenceGetChannelAmount(SelSeq));
            SelSeq.dataFrames[index] = new Uint8Array(newArr);
            let y = 0;

            for (let i = 0; i < SelSeq.numPixels; i++) {
                //console.log(i, index, y);

                switch (SelSeq.colorMode) {
                    case 'Mono':
                        SelSeq.dataFrames[index][y++] = SelSeq.dataFrames[index][0];
                        break;
                    case 'RGB':
                        SelSeq.dataFrames[index][y++] = SelSeq.dataFrames[index][0];
                        SelSeq.dataFrames[index][y++] = SelSeq.dataFrames[index][1];
                        SelSeq.dataFrames[index][y++] = SelSeq.dataFrames[index][2];
                        break;
                    case 'RGBW':
                        SelSeq.dataFrames[index][y++] = SelSeq.dataFrames[index][0];
                        SelSeq.dataFrames[index][y++] = SelSeq.dataFrames[index][1];
                        SelSeq.dataFrames[index][y++] = SelSeq.dataFrames[index][2];
                        SelSeq.dataFrames[index][y++] = SelSeq.dataFrames[index][3];
                        break;
                }//end switch
            }  //end for()
        });
    }
    else if (SelSeq.mode == 'mGradient') {
        SelSeq.dataFrames.forEach(function (array, index) {
            let newArr = Array.from(array);
            newArr.length = 4;
            SelSeq.dataFrames[index] = new Uint8Array(newArr);

            switch (SelSeq.colorMode) {
                case 'Mono':
                    //[0] would be the value for single colors
                    SelSeq.dataFrames[index][1] = 0;
                    SelSeq.dataFrames[index][2] = 0;
                    SelSeq.dataFrames[index][3] = 0;
                    break;
                case 'RGB':
                    SelSeq.dataFrames[index][3] = 0; //clear white, which is currently be the second pixel's red value.....
                    break;
                case 'RGBW':
                    //nuffin
                    break;
            }//end switch
        });
    }
    else if (SelSeq.mode == 'mList') {

    }

    if (SelSeq.mode == 'mPOV') {
        if (!SelSeq.hasOwnProperty("povDirMode")) {
            SelSeq.povDirMode = 0;
            SelSeq.povVideoMode = false;
            SelSeq.povGridSpeed = 0;
        }
        uxPOVElementStates(); //update dir buttons etc
    }
    
    //--------------------------------------------------------------------------------------------------------------------------

    sequenceValidateData(SelSeq); //replacement

    //CHECK AND FIX ANY START VALUES - converting POV to fade causes issues - after refresh layotus since indicators don't exist yet
    //  problem is the start values initialize as 0, rather than the correct values
    if (cSeqModesStartVals.includes(SelSeq.mode) && !cSeqModesStartVals.includes(oldMode) && SelSeq.errorFlag) {
        console.log("ERROR - New mode has start values, old one didn't.")
        SelSeq.startVals = new Uint8Array(SelSeq.dataFrames[0]); //copy first dataFrame into start values, as values... I hope
    }
    //else console.log("NOPE");

    //----------------------------------------------------- Common ---------------------------------------------------------------

    uxFrameButtonsRender();
    uxToolsSequenceUpdate();
    refreshLayouts();
}

//========================================================================================================================================

function seqEditSpeed(elem) {
    console.log("seqEditSpeed()");
    systemUndoRedoPush('seq');
    SelSeq.speed = parseInt(elem.value);
}

//========================================================================================================================================

function seqEditNumOfPixels(elem) {
    console.log("seqEditNumOfPixels()", parseInt(elem.value));
    systemUndoRedoPush('seq');
    SelSeq.numPixels = parseInt(elem.value); //update sequence state to match UX state
    uxToolsSequenceUpdate(); //pretty much just for the extended pixels update...... wasteful
    
    stateSeq.startValPos.forEach(function (val, index) {
        sequenceLoadStartValPositions(SelSeq, index);
    });

    sequenceValidateData(SelSeq); //replacement
    SelSeq.errorFlag = false; //hacky, errorFlag will always be set since pixels were added
    refreshLayouts();
}

//========================================================================================================================================

function seqEditTransition(elem) {
    console.log("seqEditTransition()");
    systemUndoRedoPush('seq');
    SelSeq.transition = elem.options[elem.selectedIndex].value;
}

//========================================================================================================================================

function seqEditPlayMode(elem) {
    console.log("seqEditPlayMode()");
    systemUndoRedoPush('seq');
    SelSeq.playMode = elem.options[elem.selectedIndex].value;
}

//========================================================================================================================================

function seqEditColorMode(flag) {
    console.log("seqEditColorMode() " + flag);
    systemUndoRedoPush('seq');
    if (flag == 0) {
        //set colormode tools drop down back to what it was
        uxToolsSequenceUpdate(); //brutish but whatever
    }
    else {
        //else user selected yes, go ahead and conver the data
        let elem = document.getElementById('toolmenu-sequences-colormode'); //elem will always undefined due to how it is defined/called from index.html
        let oldColorMode = SelSeq.colorMode; //save old color mode

        //---------------------- run conversion before colorMode switch, prepares data -------------------------------------
        console.log("------------ RUN CONVERSION CODE. " + oldColorMode + " changed to " + SelSeq.colorMode + "  -------------");
        if (oldColorMode == 'RGB' && elem.options[elem.selectedIndex].value == 'RGBW') convertRGBtoRGBW(SelSeq);
        else if (oldColorMode == 'RGBW' && elem.options[elem.selectedIndex].value == 'RGB') convertRGBWtoRGB(SelSeq);
        //Mono has no conversion

        SelSeq.colorMode = elem.options[elem.selectedIndex].value;//elem.value; //apply new color mode
        sequenceValidateData(SelSeq); //can only be SelSeq
        toolResetStartValues(SelSeq);

        //--------------------------------------------------------------------
        uxToolsSequenceUpdate(); //update device max pixels
        refreshLayouts(); // sure....
    }
}

//========================================================================================================================================

function seqEditSeqIntensity(elem) {
    console.log("seqEditSeqIntensity()");
    systemUndoRedoPush('seq');
    SelSeq.intensity = elem.value;
}

//========================================================================================================================================

function seqEditPacketClone(elem) {
    console.log("seqEditPacketClone()");
    systemUndoRedoPush('seq');
    SelSeq.packetClone = parseInt(elem.value);
    uxToolsSequenceUpdate(); //update extended pixel label
}

//========================================================================================================================================

function seqEditDescriptionMenu() {
    console.log("seqEditDescriptionMenu()");
    //alert("Can not be edited from this menu yet. Use left tools bar icon. \nSequence Description: \n" + SelSeq.description); //CAUSED BUG - text fields stopped working
    console.log("NOT YET IMPLEMENTED");
}
//========================================================================================================================================

function seqEditGradientSteps(elem) {
    console.log("seqEditGradientSteps()");
    systemUndoRedoPush('seq');
    SelSeq.generateSteps = elem.value;
}

//========================================================================================================================================

//Updates the data variables to match the new number of frames
function sequenceFrameAmountUpdate() {
    //COMPATIBLE FOR INC/DEC NOT FOR SETTING TO NUMBER like pixelamount

    //console.log(sequenceGetChannelAmount())
    //console.log(SelSeq.dataFrames.length, SelSeq.frames)

    //----------------------------------------------------------------------------------------

    if (SelSeq.mode == 'mList') {
        if (SelSeq.listID.length < SelSeq.frames) SelSeq.listID.push(0);
      //  else SelSeq.listID.pop();

        if (SelSeq.listIteration.length < SelSeq.frames) SelSeq.listIteration.push(0);
        //else SelSeq.listIteration.pop();

        if (SelSeq.listFlags.length < SelSeq.frames) SelSeq.listFlags.push(0);
      //  else SelSeq.listFlags.pop();
    }
    else {
        //console.log(SelSeq.dataFrames)
        if (SelSeq.dataFrames.length < SelSeq.frames) {
            //more frames are specified than defined data arrays - add another array
            let newFrame;
            if(SelSeq.mode == 'mGradient')  newFrame  = new Uint8Array(4).fill(0); //create an array intialized with all 0s
            else newFrame = new Uint8Array(sequenceGetChannelAmount(SelSeq)).fill(0); //create an array intialized with all 0s
            SelSeq.dataFrames.push(newFrame);
        }
        //else SelSeq.dataFrames.pop(); //subtract or pop off the highest array - NO NOT DOING IT THIS WAY, DATA IS PRESERVED
        //console.log(SelSeq.dataFrames);
    }

	if (stateSeq.selectedFrame >= SelSeq.frames) selectSeqFrame(SelSeq.frames - 1, false); //if the selectedFrame has been removed, select next highest

}

//========================================================================================================================================

//profile for this mode - update any UX elements as needed
function uxToolsSequenceUpdate() {
    console.log("uxToolsSequenceUpdate()");

    //--------------------------------- for cleaner code, define elements ----------------------------------

    //effects settings
    let startValGenerate = document.getElementById("toolmenu-effects-generaterun");
    let startValReset = document.getElementById("toolmenu-effects-resetstartvals");
    let startValsBox = document.getElementById("toolmenu-effects-gensteps");
    //add linear/radial other tools

    //General settings
    let frameAddButton = document.getElementById("frame-add-button");
    let frameSubtractButton = document.getElementById("frame-subtract-button");

    //Seq settings
    let importButton = document.getElementById("toolmenu-sequences-import");
    let imageWizardButton = document.getElementById("toolmenu-sequences-imgwiz");
    let colorModeDropDown = document.getElementById("toolmenu-sequences-colormode");
    let perIntensityValueBox = document.getElementById("toolmenu-sequences-intensity");
    let packetCloneBox = document.getElementById("toolmenu-sequences-clone");
    let playModeDropDown = document.getElementById("toolmenu-sequences-playmode");
    let transitionDropDown = document.getElementById("toolmenu-sequences-transition");
    let numPixValueBox = document.getElementById("toolmenu-sequences-numpix");
    let speedValueBox = document.getElementById("toolmenu-sequences-speed");

    //------------------------ update all the UX Sequence Menu tools -------------------------------

    //Sequence Tools
    document.getElementById("toolmenu-sequences-name").value = SelSeq.name;
    document.getElementById("toolmenu-sequences-speed").value = SelSeq.speed;
    document.getElementById("ovmenu-6-textbox").value = SelSeq.description;
    numPixValueBox.value = SelSeq.numPixels;
    perIntensityValueBox.value = SelSeq.intensity;
    convertSelectInputOption("toolmenu-sequences-mode", SelSeq.mode);    //Sequence Mode
    convertSelectInputOption("toolmenu-sequences-transition", SelSeq.transition);    //Transition
    convertSelectInputOption("toolmenu-sequences-playmode", SelSeq.playMode);    //Play Mode
    convertSelectInputOption("toolmenu-sequences-colormode", SelSeq.colorMode);     //Color Mode

    if (SelSeq.packetClone != undefined) {
        packetCloneBox.value = SelSeq.packetClone;
        document.getElementById("toolmenu-sequences-extendedval").innerText = (SelSeq.numPixels * (SelSeq.packetClone + 1));
    }
    else {
        packetCloneBox.value = 0;
        document.getElementById("toolmenu-sequences-extendedval").innerText = SelSeq.numPixels;
    }

    if (stateDevice.soft.channels != undefined) document.getElementById("toolmenu-sequences-devicepixmax").innerText = Math.floor(stateDevice.soft.channels / getColorModeMult(SelSeq.colorMode));
    else document.getElementById("toolmenu-sequences-devicepixmax").innerText = '(1024)';

    //Effects Tools
    document.getElementById("toolmenu-effects-gensteps").value = SelSeq.generateSteps;

    //Layout Tools
    //are in another function still...

    //-----------------------------------------------------------------------------------------

    //---------------------------------- easiest, just enable all elements that could be disabled ----------------------------------

    enable(startValGenerate);
    enable(startValReset);
    enable(startValsBox);

    enable(frameAddButton);
    enable(frameSubtractButton);

    enable(importButton);
    enable(imageWizardButton);
    enable(colorModeDropDown);
    enable(perIntensityValueBox);
    enable(packetCloneBox);
    enable(playModeDropDown);
    enable(transitionDropDown);
    enable(numPixValueBox);
    enable(speedValueBox);

    //------------------------------ disable elements based on mode -------------------------------------------------------------
    switch (SelSeq.mode) {
        case 'mFade':
            disable(importButton);
            break;
        case 'mGradient':
            disable(importButton);
            disable(imageWizardButton);
            break;
        case 'mInstant':
            disable(importButton);
            disable(startValGenerate);
            //disable(startValReset);
            disable(startValsBox);
            break;
        case 'mPOV':
            disable(startValGenerate);
            disable(startValReset);
            disable(startValsBox);
            disable(imageWizardButton);
            break;
        case 'mFile':
            disable(startValGenerate);
            disable(startValReset);
            disable(startValsBox);
            break;
        case 'mList':
            disable(importButton);
            disable(startValGenerate);
            disable(startValReset);
            disable(startValsBox);
            disable(colorModeDropDown);
            disable(perIntensityValueBox);
            disable(packetCloneBox);
            disable(playModeDropDown);
            disable(transitionDropDown);
            disable(numPixValueBox);
            disable(speedValueBox);
            disable(imageWizardButton);
            break;
        case 'mScene':
            disable(importButton);
            disable(startValGenerate);
            disable(startValReset);
            disable(startValsBox);
            disable(frameAddButton);
            disable(frameSubtractButton);
            disable(speedValueBox);
            disable(playModeDropDown);
            break;
        case 'mTechnical':
            disable(importButton);
            disable(startValGenerate);
            disable(startValReset);
            disable(startValsBox);
            disable(imageWizardButton);
            break;
        default:
            console.log("Sequence mode did not match!!!")
            break;
    }//end switch

    //--------------------------------- Disable Tools by Device----------------------------------------------------------

    if (stateDevice.soft.lockChannelSize) disable(packetCloneBox); //so far a reasonable way to track it

    //-------------------------------------------------------------------------------------------

    function disable(elem) {
        elem.setAttribute('disabled', true);
    }

    function enable(elem) {
        elem.removeAttribute('disabled');
    }
}

//========================================================================================================================================

function sequenceUpdateUXElements() {
    console.log("sequenceUpdateUXElements()");


}
//========================================================================================================================================


