"use strict";

//========================================================================================================================================

function commSerialPrevConnect() {
    console.log("commSerialPrevConnect()")
    let userBaudElem = document.getElementById('ws-hardware-baud');
    let userPortElem = document.getElementById('ws-hardware-port');
    //console.log(userPortElem.options.length);
    //could use convertSelectInputOption()
    for (let i = 0; i < userPortElem.options.length; i++) {
        if (userPortElem.options[i].text == stateCommunication.prevSerialPort) {
            console.log("Port Matches " + userPortElem.options[i].text);
            userPortElem.selectedIndex = i;
            //match baud
            for (let q = 0; q < userBaudElem.options.length; q++) {
                if (userBaudElem.options[q].text == stateCommunication.baud) {
                    userBaudElem.selectedIndex = q;
                    console.log("Baud Matches " + userBaudElem.options[q].text);
                }
            }
            commSerialPortConnect(stateCommunication.prevSerialPort, userBaudElem.value);
        }
    }
}

//========================================================================================================================================

function commSerialAutoScan() {
    console.log("commSerialAutoScan()");
    serialPort.list().then(ports => { //.list() does not provide serial port attributes like isOpen()
        console.log(ports);
        ports.forEach(function (port) {
            //console.log(port.path);
            //console.log(port.pnpId);
            //console.log(port.productId);
            if (port.productId == '000A') {
                //if the port attempts reconnection while connnected, it will fall into a reconnect loop, but stay connected
                console.log(port.path + " is probably a NLED device");
                pushMessageToLog("Serial port auto-scan found a NLED device on " + port.path + ", attempting connection.");
                convertSelectInputOption('ws-hardware-port', port.path);
                commSerialPortConnect(port.path);
            }
            else console.log(port.path + " is not a NLED device");
        });
    });
}

//========================================================================================================================================

function commSerialPortConnect(portName, baudRate) {
    console.log("commSerialPortConnect()");
    if (portName == undefined) portName = document.getElementById('ws-hardware-port').value;
    stateCommunication.serialPort = portName; //update these
    if (baudRate == undefined) baudRate = document.getElementById('ws-hardware-baud').value;
    stateCommunication.baud = parseInt(baudRate);
    commPrepareToConnect(function () {
        auroraCMD = new AuroraDeviceInterface(); //construct new
        auroraCMD.enableLogging = true;
        commSerialOpenPort(portName, parseInt(baudRate), auroraCMD); //connect to new port now, since it is not waiting to close any ports
    }); //common to serial and TCP and ???
}

//========================================================================================================================================

function commPrepareToConnect(callback) {
    console.log("commPrepareToConnect()");
    //if protocol instance exists, check if there is an open port, close it, then try again to connect
    if (stateCommunication.connected) {
        try {
            if (auroraCMD.port.isOpen) auroraCMD.port.close();
        }
        catch (e) { }
        setTimeout(callback, 50); //wait 50mS for 'close' event to run, then run callback. Worst case if it is not ready, the user can try again
        return; // will come back and run again when .close callback fires, then run the rest of the code
    }
    else callback(); //else run callback normally
}

//========================================================================================================================================

function commAuroraDeviceConnection() {
    console.log("commAuroraDeviceConnection()");

    commUXMessage('open');
    commUXMessage('request');
    auroraCMD.requestDeviceInfo(function (buf) {
        //console.log("PayloadRX callback. Populating data for get device connect"); //DEBUG
        if (systemValidateDeviceID(buf[0])) {
            //Device ID is validated, populate info.
            stateDevice.hard.idNum = buf[0];
            stateDevice.hard.hardVer = buf[1];
            stateDevice.hard.version = buf[2];
            stateDevice.hard.revision = buf[3];
            stateDevice.hard.bootTargetID = buf[4];
            stateDevice.hard.bootVer = buf[5];
            stateDevice.hard.userIDNum = buf[6];
            stateCommunication.connected = true;
            systemLoadDevice(); //load device profile
            commUXMessage('success'); //show message on hardware tab

            // if (stateDevice.hard.version >= 3) {
            //     serialPort.list().then(ports => {
            //         ports.forEach(function (v, i) {
            //             if (v.path == auroraCMD.port.path) {
            //                 if (v.vendorId == "04D8" && v.productId == "000A") {
            //                     //auroraCMD.serialUSB = true;
            //                     // console.log(" auroraCMD.serialUSB FOUND",v.path,auroraCMD.port.path);
            //                 }
            //             }
            //         });
            //     });
            // }

            stateCommunication.prevSerialPort = stateCommunication.serialPort;
            stateCommunication.baud = stateCommunication.baud;
            fileSaveSettingsFile(); //saves prevSerialPort and baud rate
        }
        else if (cCompatibilityList.includes(buf[6])) {
            commUXMessage('fail'); //run first so top log message will be this one.
            pushMessageToLog("NLED device found. It is not compatible with this software version. Use the old software release. Device support will be added soon. Found ID: " + buf[6]);
        }
        else {
            commUXMessage('fail');
        }
        //request to download configs, after delay, since auroraCMD.reset() hasn't happened yet, so the command is not complete and a new one can't be started
        if (stateCommunication.connected && stateSoftware.autoDownloadEnabled) setTimeout(commDownloadConfigs, 25);
        if (stateDevice.hard.version >= 3)  commGetDeviceSerialNumber();
    });
}

//========================================================================================================================================

function commSerialOpenPort(argPortPath, argBaudRate, devInterface) {
    //devInterface is auroraCMD object

    let newPort = new serialPortBridge({
        path: argPortPath,
        baudRate: argBaudRate,
        parity: 'none',
        stopBits: 1,
        dataBits: 8,
    }, function (err) {
        //console.log(devInterface)
        if (err) commUXMessage('err', err.message);
        else {
            console.log("Serial Port Opened Successfully");
            commAuroraDeviceConnection(newPort);
        }
    });

    //---------------------------------------- Setup events for .open() and .close() -----------------------------------------------------

    newPort.on('open', function () {
        console.log("Port is opened, sending request");

        devInterface.init(newPort); //now that port is setup, attach it to the deviceInterface
        //no longer accessible without going through contextbridge
        devInterface.port.isOpen = true;
        devInterface.port.path = argPortPath;
        devInterface.port.baudRate = argBaudRate;

        newPort.flush();
        //   console.log("Flushed the serial port buffer");

        //-------------- App specific AuroraDeviceInterface Events -------------------------

        devInterface.eventTimeout = function () {
            commUXMessage('timeout');
        }

        devInterface.eventNoPort = function () {
            pushMessageToLog("Error sending to port.");
            //what else could i do here?
        }

    });

    newPort.on('data', function (data) {
        // process.stdout.write("\r" + 'Serial - Received: '+ data);
        // process.log('Serial - Received: ' + data + "\n" + new TextDecoder().decode(data)); //DEVELOPMENT - prints in main terminal, not renderer
        //   console.log('Serial - Received: ', data);
        devInterface.receiveData(data);
    });

    //Close errors will be emitted as an error event
    newPort.on('close', function () {
        console.log("Serial port closed event");
        devInterface.port.isOpen = false;
        commUXMessage('closed');
        stateCommunication.connected = false;
    });
    newPort.on('error', function (err) {
        console.log("Serial port error", err);
        newPort.close();
    });
} //end func

//========================================================================================================================================

function commDisconnectFromPort() {
    //Close port if it is open
    console.log("commDisconnectFromPort()", auroraCMD.port.isOpen);
    try {
        if (auroraCMD.port.isOpen) {
            auroraCMD.port.close(); //will run the newPort.on('close') as callback
        }
        else pushMessageToLog("No open ports to close");
    }
    catch {
        console.log("error closing port or something");
    }
}

//========================================================================================================================================

// Discontinued. But keep it.
// function commSerialPortReconnect() {
//     pushMessageToLog("Attempting connection to previous port and settings");
//     commSerialPrevConnect();
// }

//========================================================================================================================================

function commUXMessage(arg, msg) {
    console.log("commUXMessage(" + arg + "   " + msg + ")");

    let notificationText = document.getElementById('ws-hardware-communication-message');
    let primaryMsg, badgeMsg, portMsg;
    let badgeCol = false;


    switch (arg) {
        case 'open':
            if (auroraCMD.commType == 'serial')
                primaryMsg = 'Opening port ' + stateCommunication.serialPort + " at " + stateCommunication.baud + " baud rate.";
            else if (auroraCMD.commType == 'tcp')
                primaryMsg = 'Opening TCP: ' + stateCommunication.networkIP + " : " + stateCommunication.networkPort;
            badgeCol = true; //green = connected color
            badgeMsg = "Opened";
            if (auroraCMD.commType == 'serial')
                portMsg = 'Port: ' + getShortened();
            else if (auroraCMD.commType == 'tcp')
                portMsg = 'Port: TCP';
            break;

        case 'request':
            if (auroraCMD.commType == 'serial')
                primaryMsg = "Attempting device connection on " + getShortened();
            else if (auroraCMD.commType == 'tcp')
                primaryMsg = "Attempting device connection on " + stateCommunication.networkIP + " : " + stateCommunication.networkPort;
            badgeCol = true; //green = connected color
            badgeMsg = "Requesting";
            portMsg = 'Port: ' + getShortened();
            break;
        case 'closed':
            if (auroraCMD.commType == 'serial')
                primaryMsg = "Port Closed";
            else if (auroraCMD.commType == 'tcp')
                primaryMsg = "TCP Closed";
            badgeCol = false; //red = connected color
            badgeMsg = "Connect"
            portMsg = 'Port: CLOSED';
            break;
        case 'success':

// console.log("commUXMessage() SUCCESS", auroraCMD.commType);

            if (auroraCMD.commType == 'serial') {
                primaryMsg = "Found a " + stateDevice.soft.name + " on " + stateCommunication.serialPort;
                portMsg = 'Port: ' + getShortened();
            }
            else if (auroraCMD.commType == 'tcp') {
                primaryMsg = "Found a " + stateDevice.soft.name + " on " + stateCommunication.networkIP + ":" + stateCommunication.networkPort;
                portMsg = 'Port: TCP';
            }
            badgeCol = true; //green = connected color
            badgeMsg = "Connected";
            break;
        case 'err':
            if (auroraCMD.commType == 'serial')
                primaryMsg = 'Serial port connection error: ' + msg;
            else if (auroraCMD.commType == 'tcp')
                primaryMsg = 'TCP connection error: ' + msg;
            else primaryMsg = 'Error ' + msg;
            badgeMsg = "Connect"
            badgeCol = false; //red = connected color
            portMsg = 'Port: ERROR';
            break;
        case 'emulate':
            badgeMsg = "Emulated"
            badgeCol = false; //red = connected color
            portMsg = 'Port: NONE';
            break;
        case 'fail':
            primaryMsg = "Device on port not recognized.";
            badgeCol = false; //red = connected color
            badgeMsg = "Connect"
            portMsg = 'Port: NONE'
            break;
        case 'timeout':
            if (auroraCMD.commType == 'serial')
                primaryMsg = "Device did not respond, command timed out.";
            else if (auroraCMD.commType == 'tcp')
                primaryMsg = "TCP Server did not respond. Check IP and port settings.";
            badgeCol = true; //red = connected color
            badgeMsg = "Timeout"
            portMsg = 'Port: ' + getShortened();
            break;
    } //end switch

    //----------------------------------------------------------------------------------

    if (primaryMsg != undefined) pushMessageToLog(primaryMsg);
    notificationText.textContent = primaryMsg;
    document.getElementById('status-bar-port-label').textContent = portMsg;
    document.getElementById('status-bar-badge').textContent = badgeMsg;
    if (badgeCol) document.getElementById('status-bar-badge').style.backgroundColor = 'var(--nled-green)'
    else document.getElementById('status-bar-badge').style.backgroundColor = 'var(--nled-red)'

    function getShortened() {
        return stateCommunication.serialPort.substring(stateCommunication.serialPort.length - 14); //shorten name for Mac/Linux
    }
}

//========================================================================================================================================

