"use strict"

//========================================================================================================================================

function toolGenerateCheckbox(flag) {
    console.log("toolGenerateCheckbox()");

    let elem = document.getElementById('toolmenu-effects-graphgenenableval');
    if (flag != undefined) elem.checked = flag;

    let theIcons = document.getElementById("ws-graphic-layout-container").querySelectorAll("div.pxSz");

    if (elem.checked) {
        if (stateUX.activeTab != 'navtab-graphic') uxTriggerElementClick('navtab-graphic');
        document.getElementById("ws-graphic-container").style.cursor = "crosshair";
        document.getElementById("ws-graphic-container").addEventListener('mousedown', toolGenerateMouseDown);
        //Set no events for pixel icons
        theIcons.forEach(function (val, index) {
            theIcons[index].style.pointerEvents = 'none';
        });
    }
    else {
        //disabling - revert to normal
        document.getElementById("ws-graphic-container").removeEventListener('mousedown', toolGenerateMouseDown);
        document.getElementById("ws-graphic-container").style.cursor = '';
        theIcons.forEach(function (val, index) {
            theIcons[index].style.removeProperty('pointer-events'); //restore normal pointer events and cursor
        });
        layoutGraphicIconsUpdate();
    }
}

//========================================================================================================================================

//has to be global scope functions or hard/can't removeEventListener
function toolGenerateMouseDown(event) {
    console.log("toolGenerateMouseDown()");

    let adjScrollX = document.getElementById("ws-graphic-container").scrollLeft;
    let adjScrollY = document.getElementById("ws-graphic-container").scrollTop;

    stateUX.toolCoordX = event.clientX - cWSOffsetX + adjScrollX;
    stateUX.toolCoordY = event.clientY - cWSOffsetY + adjScrollY;
    //console.log(stateUX.toolCoordX, stateUX.toolCoordY);

    let theCanvas = document.createElement('canvas');
    theCanvas.id = "tool-generate-canvas";
    theCanvas.width = document.getElementById("ws-graphic-container").scrollWidth;
    theCanvas.height = document.getElementById("ws-graphic-container").scrollHeight;
    document.getElementById("ws-graphic-container").appendChild(theCanvas);

    document.addEventListener('mousemove', toolGenerateMouseMove); //global scope

    document.addEventListener('mouseup', function toolGenerateMouseUp(event) {
        console.log("toolGenerateMouseUp()");
        clearTimeout(stateUX.mouseTimer); //clear mousemove throttle timer
        document.removeEventListener('mouseup', toolGenerateMouseUp); //run only once -remove itself
        document.removeEventListener('mousemove', toolGenerateMouseMove); //global scope
        document.getElementById("ws-graphic-container").querySelector('canvas').remove();
        toolGraphicGenerateApply((event.clientX - cWSOffsetX), (event.clientY - cWSOffsetY));
        layoutGraphicIconsUpdate(); //redraw icons, start values????
    });
}

//========================================================================================================================================

//has to be global scope functions or hard/can't removeEventListener
function toolGenerateMouseMove(event) {
    //Throttle mousemove speed
    stateUX.mouseTimer = setTimeout(function () {
        document.addEventListener("mousemove", toolGenerateMouseMove)
    }, 30);
    document.removeEventListener("mousemove", toolGenerateMouseMove);

    console.log("toolGenerateMouseMove()");
    //console.log(stateUX.toolCoordX, stateUX.toolCoordY);
    //console.log(event.clientX, event.clientY);
    let adjScrollX = document.getElementById("ws-graphic-container").scrollLeft;
    let adjScrollY = document.getElementById("ws-graphic-container").scrollTop;

    let adjMouseX = (event.clientX - cWSOffsetX) + adjScrollX;
    let adjMouseY = (event.clientY - cWSOffsetY) + adjScrollY;


    let typeElem = document.getElementById('toolmenu-effects-generategraphicmode'); //shorten
    let type = typeElem.options[typeElem.selectedIndex].value; //get value as string

    //let angle = Math.atan2(-(event.clientX - stateUX.toolCoordX), -(stateUX.toolCoordY - event.clientY));

    //--------------------- common canvas setup ------------------------------------------
    let canvas = document.getElementById('tool-generate-canvas');
    let ctx = canvas.getContext("2d");
    ctx.clearRect(0, 0, canvas.width, canvas.height);
    ctx.lineWidth = 3;
    ctx.setLineDash([5, 5]);
    ctx.shadowBlur = 10;
    ctx.shadowColor = "#FF0000";
    ctx.strokeStyle = "#FFFFFF";

    switch (type) {
        case 'genModeGraphicLinear':
            ctx.clearRect(0, 0, canvas.width, canvas.height);
            ctx.lineWidth = 3;
            ctx.beginPath();
            ctx.moveTo(stateUX.toolCoordX, stateUX.toolCoordY);
            ctx.lineTo(adjMouseX, adjMouseY);
            ctx.moveTo(stateUX.toolCoordX, stateUX.toolCoordY); //center T point
            let X1 = stateUX.toolCoordX + (adjMouseY - stateUX.toolCoordY);
            let Y1 = stateUX.toolCoordY + (stateUX.toolCoordX - adjMouseX);
            let X2 = stateUX.toolCoordX + (stateUX.toolCoordY - adjMouseY);
            let Y2 = stateUX.toolCoordY + (adjMouseX - stateUX.toolCoordX);
            ctx.lineTo(X1, Y1); //draw top of T line				
            ctx.moveTo(stateUX.toolCoordX, stateUX.toolCoordY);
            ctx.lineTo(X2, Y2); //draw top of T line		
            ctx.stroke();
            break;

        case 'genModeGraphicRadial':
            let a = stateUX.toolCoordX - adjMouseX;
            let b = stateUX.toolCoordY - adjMouseY;
            let c = Math.sqrt(a * a + b * b);
            //console.log(a,b,c);
            ctx.beginPath();
            ctx.moveTo(stateUX.toolCoordX, stateUX.toolCoordY);
            ctx.lineTo(adjMouseX, adjMouseY);
            ctx.moveTo(stateUX.toolCoordX, stateUX.toolCoordY);
            ctx.lineTo(stateUX.toolCoordX + a, stateUX.toolCoordY + b);
            ctx.stroke();
            ctx.beginPath();
            ctx.arc(stateUX.toolCoordX, stateUX.toolCoordY, c, 0, 2 * Math.PI);
            ctx.stroke();
            break;
    } //end switch
}

//========================================================================================================================================

function toolGraphicGenerateApply(endX, endY) {
    console.log("toolGraphicGenerateApply() " + endX, endY);

    systemUndoRedoPush('seq'); //run before any changes

    //radius/diameter selects what icons are affected, 0 to 100
    //the multiplier is the maximum percentage value, rolls over at 100
    //so a setting of 200 will do two full cycles on the selected icons

    let multiplier = parseInt(document.getElementById('toolmenu-effects-multval').value);
    let typeElem = document.getElementById('toolmenu-effects-generategraphicmode'); //shorten
    let type = typeElem.options[typeElem.selectedIndex].value; //get value as string

    let theIcons = document.getElementById("ws-graphic-layout-container").querySelectorAll("div.pxSz:not(.disabled)");
    //console.log(theIcons);

    //----------------- get distance between the two points, based on which one is larger. Think it works without this but keep it smart ---------------------------
    let a;
    if (stateUX.toolCoordX > endX) a = stateUX.toolCoordX - endX;
    else a = endX - stateUX.toolCoordX;
    let b;
    if (stateUX.toolCoordY > endY) b = stateUX.toolCoordY - endY;
    else b = endY - stateUX.toolCoordY;
    let lineLength = Math.round(Math.sqrt(a * a + b * b)); //length of the line
    console.log(a, b, lineLength);

    //----------------- Test to make sure it was a deliberate action ---------------------------

    if (lineLength < 40) { console.log("Selection too short"); return; } //just chose 40, good enough

    switch (type) {
        case 'genModeGraphicLinear':
            //Copied, gets the top T section points. 
            let X1 = stateUX.toolCoordX + (endY - stateUX.toolCoordY);
            let Y1 = stateUX.toolCoordY + (stateUX.toolCoordX - endX);
            let X2 = stateUX.toolCoordX + (stateUX.toolCoordY - endY);
            let Y2 = stateUX.toolCoordY + (endX - stateUX.toolCoordX);

            //------------- The meaty bits, get the distance to the pixel icon from the line
            for (let i = 0; i < theIcons.length; i++) {
                let result = distance({ x: X1, y: Y1 }, { x: X2, y: Y2 }, mainObj.layout.ledIconCoordX[i], mainObj.layout.ledIconCoordY[i]);
                if (result[1] == true) {
                    if (result[0] <= lineLength) {
                        //console.log("ran " + i + "   result: " + result[0]);
                        stateSeq.startValPos[i] = (result[0] / lineLength) * multiplier;
                        while (stateSeq.startValPos[i] > 1) stateSeq.startValPos[i] -= 1;
                        sequenceSetStartValues(SelSeq, i); //updates the seq.startVals to match the float
                    }
                }
            } //end i for()

            //This checks the distance from an infinite line along the point1 & point2 slope. Calcultes distance to x0,y0
            function distance(point1, point2, x0, y0) {
                return [
                    Math.round((Math.abs((point2.y - point1.y) * x0 -
                        (point2.x - point1.x) * y0 +
                        point2.x * point1.y -
                        point2.y * point1.x)) /
                        (Math.pow((Math.pow(point2.y - point1.y, 2) +
                            Math.pow(point2.x - point1.x, 2)), 0.5))),
                    (Math.sign((point2.x - point1.x) * (y0 - point1.y) - (point2.y - point1.y) * (x0 - point1.x)) == -1 ? true : false)];
            }
            break;

        case 'genModeGraphicRadial':
            let drawID = [];
            let drawList = [];
            let count = 0;

            if (theIcons.length > 0) {
                for (let i = 0; i < lineLength; i++) {
                    //check for icon x,y that are within the circle
                    for (let q = 0; q < theIcons.length; q++) {
                        if (theIcons[q].style.display != 'none') {
                            let disX = stateUX.toolCoordX - mainObj.layout.ledIconCoordX[q];
                            let disY = stateUX.toolCoordY - mainObj.layout.ledIconCoordY[q];
                            //prase through all the shown icons for match
                            //console.log("trying on " + q + " with " + disX, disY);
                            if (!drawList.includes(q)) {
                                //icon has not been set yet
                                if (Math.sqrt(Math.pow(disX, 2) + Math.pow(disY, 2)) < i) {
                                    //console.log("yep on " + (q + 1));
                                    drawList.push(q); //excludes it from being checked again
                                    drawID.push(count); //post increment
                                }
                            }
                        }
                    } //end q for()
                    count++;
                } //end i for()
                //Now points are in order of distance from center [0] is closest, lower IDs is closer
                //now ID holds the iconID# with percList holding the percentage

                //console.log(drawList);
                //console.log(drawID);

                let perc = 100 / lineLength;
                let percList = [];

                //console.log(perc);

                //could be condensed
                percList = drawID.map(function (e) {
                    let val = (e * perc) * multiplier;
                    while (val > 100) val -= 100;
                    return val / 100;
                });
                //console.log(percList);

                for (let i = 0; i < percList.length; i++) {
                    stateSeq.startValPos[drawList[i]] = percList[i]; //order the values correctly
                    sequenceSetStartValues(SelSeq, drawList[i]); //updates the seq.startVals to match the float
                }
            }
            break;
    } //end switch
}

//========================================================================================================================================

function toolResetStartValues(seq) {
    systemUndoRedoPush('seq'); //run before any changes
    //should this only be allowed on the correct sequence modes? yes others shouldn't have it
    for (let i = 0; i < seq.numPixels; i++) {
        if (!mainObj.layout.advanced || (stateUX.zoneView == 0 || mainObj.layout.ledIconZoneID[i] == stateUX.zoneView || mainObj.layout.ledIconZoneID[i] == undefined)) {
            stateSeq.startValPos[i] = 0;
            sequenceSetStartValues(seq, i); //set the seq.startVals with the new offset float value
        }
        //else nuffin
    }
    layoutUpdateBuffers(); //now update the indicator position or graphic icon's color
}

//========================================================================================================================================

function toolGenerateStartVals(seq) {
    let modeDD = document.getElementById("toolmenu-effects-generatemode");
    let mode = modeDD.options[modeDD.selectedIndex].value;
    let steps = document.getElementById("toolmenu-effects-gensteps").value;
    let start = Number(document.getElementById("toolmenu-effects-genstart").value);
    let end = Number(document.getElementById("toolmenu-effects-genend").value);

    end = end / 100; //float
    start = start / 100; //float
    let val = start; //increment value, so can handle < 1 and == 1

    let div;
    let updown = false;

    console.log("toolGenerateStartVals() with " + mode + " steps: " + steps);
    systemUndoRedoPush('seq'); //run before any changes

    switch (mode) {
        default:
            console.log("DEFAULTED", mode);
            break;
        case 'genModeRandom':
            if (steps > 100) steps = 100; //clamp
            document.getElementById("toolmenu-effects-gensteps").value = steps; //update it here I guess.....
            steps = steps / 100; //internal function use

            for (let i = 0; i < seq.numPixels; i++) {
                if (!mainObj.layout.advanced || (stateUX.zoneView == 0 || mainObj.layout.ledIconZoneID[i] == stateUX.zoneView || mainObj.layout.ledIconZoneID[i] == undefined)) {
                    //stateSeq.startValPos[i] = Math.random();
                    // stateSeq.startValPos[i] = (Math.random() * steps); //allows ranges 0 to 100% to be the maximum
                    stateSeq.startValPos[i] = (Math.random() * (end - start) + start) * steps;
                    sequenceSetStartValues(seq, i); //set the seq.startVals with the new offset float value
                }
            }
            break;
        case 'genModeLinearDown': //technically saw-tooth I think
            div = 1 / steps; //number as float of position for linear
            for (let i = 0; i < seq.numPixels; i++) {
                if (!mainObj.layout.advanced || (stateUX.zoneView == 0 || mainObj.layout.ledIconZoneID[i] == stateUX.zoneView || mainObj.layout.ledIconZoneID[i] == undefined)) {
                    stateSeq.startValPos[i] = val;
                    sequenceSetStartValues(seq, i); //set the seq.startVals with the new offset float value
                    val += div; //FIX WHAT IF IT IS 3?
                    if (val > end) {
                        val -= end;
                        if (val < start) val = start;
                    }
                    else if (val == end) val = start; //rather than place the handle at 100%, set it to 0
                }
            }
            break;
        case 'genModeSawTooth':
            div = (1 / steps) * 2; //number as float of position for linear
            updown = false;
            for (let i = 0; i < seq.numPixels; i++) {
                if (!mainObj.layout.advanced || (stateUX.zoneView == 0 || mainObj.layout.ledIconZoneID[i] == stateUX.zoneView || mainObj.layout.ledIconZoneID[i] == undefined)) {
                    stateSeq.startValPos[i] = val;
                    sequenceSetStartValues(seq, i); //set the seq.startVals with the new offset float value
                    if (updown) {
                        val -= div;
                        if (val < start) {
                            if (val < 0) val = (val * -1);
                            else val = 0;
                            val += start;
                            updown = false;
                        }
                    }
                    else {
                        val += div;
                        if (val > end) {
                            val = end - (val - end);
                            updown = true;
                        }
                    }
                }
            }
            break;
        case 'genModeSquare':
            div = (1 / steps) * 2; //number as float of position for linear
            updown = false;
            for (let i = 0; i < seq.numPixels; i++) {
                if (!mainObj.layout.advanced || (stateUX.zoneView == 0 || mainObj.layout.ledIconZoneID[i] == stateUX.zoneView || mainObj.layout.ledIconZoneID[i] == undefined)) {
                    if (updown) stateSeq.startValPos[i] = end;
                    else stateSeq.startValPos[i] = start;
                    sequenceSetStartValues(seq, i); //set the seq.startVals with the new offset float value
                    if (updown) {
                        val -= div;
                        if (val < start || val == start) updown = false;
                    }
                    else {
                        val += div;
                        if (val > end) updown = true;
                    }
                }
            }
            break;
        // case 'genModeTriangle':
        //     break;
        // case 'genModeSine':
        //     break;
    } //end switch
    layoutUpdateBuffers();//layoutTimelineIndicatorUpdate(seq);
}

//========================================================================================================================================

function toolPaintManualCheckbox(flag) {
    console.log("toolPaintManualCheckbox() ")

    let elem = document.getElementById("toolmenu-paint-brushenableval")
    if (flag != undefined) elem.checked = flag;
    stateUX.toolPaintEnable = elem.checked; //update state variable to match the elem.checked

    let brush = document.getElementById('tool-paint-brush');
    if (brush != undefined) brush.remove();

    let timeRows = document.getElementById("ws-timeline-container").querySelectorAll("[id*='tl-row']");

    if (stateUX.toolPaintEnable) {
        let theBrush = document.createElement('div');
        theBrush.id = 'tool-paint-brush';
        document.getElementById("ws-graphic-container").appendChild(theBrush);

        //All have to be global, couldn't get it to work right without it
        //Mousemove and mouse down only when over the container element
        document.getElementById("ws-graphic-container").addEventListener('mousemove', toolPaintMouseMove); //gotta be global funciton for throttling to work
        document.getElementById("ws-graphic-container").addEventListener('mousedown', toolPaintMouseDown);
        for (let i = 0; i < timeRows.length; i++) timeRows[i].style.cursor = 'crosshair';
    }
    else {
        clearTimeout(stateUX.mouseTimer);
        document.getElementById("ws-graphic-container").removeEventListener('mousedown', toolPaintMouseDown);
        document.getElementById("ws-graphic-container").removeEventListener("mousemove", toolPaintMouseMove);
        for (let i = 0; i < timeRows.length; i++) timeRows[i].style.removeProperty('cursor'); //removes inline and reverts to class default(pointer)
    }
}

//========================================================================================================================================

function toolPaintMouseDown() {
    stateUX.mouseDown = true;
    //console.log("mouseDown");
    toolPaintMouseMove(event); //forces the action right away in case user only makes a single click without move/drag
    //MouseUp has to be on the document, in case user takes cursor off container-overlay 
    document.addEventListener('mouseup', function () { stateUX.mouseDown = false; }, { once: true });
}

//========================================================================================================================================

function toolPaintMouseMove(event) {
    //------------------------------ Throttle mousemove speed ----------------------------------------------
    stateUX.mouseTimer = setTimeout(function () {
        document.getElementById("ws-graphic-container").addEventListener("mousemove", toolPaintMouseMove)
    }, 30);
    document.getElementById("ws-graphic-container").removeEventListener("mousemove", toolPaintMouseMove);

    //console.log("moving " + stateUX.mouseDown);

    //Draw a circle or square. Follows the mouse cursor(set mouse move)
    let adjScrollX = document.getElementById("ws-graphic-container").scrollLeft;
    let adjScrollY = document.getElementById("ws-graphic-container").scrollTop;
    let adjMouseX = (event.clientX - cWSOffsetX) + adjScrollX;
    let adjMouseY = (event.clientY - cWSOffsetY) + adjScrollY;

    let brushSz = parseInt(document.getElementById("toolmenu-paint-brushval").value);
    let brush = document.getElementById('tool-paint-brush');

    /*
    if (brush == undefined) {
        //  document.getElementById("toolmenu-paint-brushenableval").checked = false;
        //   toolPaintManualCheckbox(document.getElementById("toolmenu-paint-brushenableval"));
        return;
    }
    */

    brush.style.left = adjMouseX + 'px';
    brush.style.top = adjMouseY + 'px';
    brush.style.width = brushSz + 'px';
    brush.style.height = brushSz + 'px';
    brush.style.borderRadius = (brushSz / 2) + 'px';

    //------------------------------ check if icon is within brush area --------------------------------------
    if (stateUX.mouseDown) {
        let theIcons = document.getElementById("ws-graphic-layout-container").querySelectorAll("div.pxSz")

        if (theIcons.length > 0) {
            systemUndoRedoPush('seq');
            stateUX.undoRedoDisable = true;
            for (let q = 0; q < theIcons.length; q++) {
                if (theIcons[q].style.display != 'none') {
                    let disX = adjMouseX - mainObj.layout.ledIconCoordX[q];
                    let disY = adjMouseY - mainObj.layout.ledIconCoordY[q];

                    if (pointInCircle(mainObj.layout.ledIconCoordX[q], mainObj.layout.ledIconCoordY[q], adjMouseX, adjMouseY, (brushSz / 2))) {
                        //console.log("yep on " + (q + 1));
                        //Apply color if not already painted that color
                        let colorArr = sequenceGetPixelData(SelSeq, 'begin', q, stateSeq.selectedFrame);
                        let compareArr = [...stateUX.paintManColor];
                        compareArr.length = getColorModeMult(SelSeq.colorMode); //crop it down, since paintManColor is always RGBW
                        if (!isArrayEqual(colorArr, compareArr)) {
                            handleColorPickerSelection(stateSeq.selectedFrame, q, SelSeq.colorMode, 'pixel', stateUX.paintManColor);
                            layoutUpdateBuffers();
                            // sequenceSetPixelData(SelSeq, 'frameVar', q, stateSeq.selectedFrame);
                            // sequenceSetPixelData(SelSeq, 'startVal', q, stateSeq.selectedFrame, stateUX.paintManColor);
                            // sequenceLoadStartValPositions(SelSeq, q);
                        }
                    }
                } //end view check
            } //end q for()
            stateUX.undoRedoDisable = false;
        }
    } //end mouseDown if()
    //--------------------------------------------------------------------

    // x,y is the point to test
    // cx, cy is circle center, and radius is circle radius
    function pointInCircle(x, y, cx, cy, radius) {
        var distancesquared = (x - cx) * (x - cx) + (y - cy) * (y - cy);
        return distancesquared <= radius * radius;
    }

    //--------------------------------------------------------------------
}

//========================================================================================================================================

function toolSelectPaintColor(id) {
    colorPicker.open(event.clientX, event.clientY, SelSeq.colorMode, 0, 0, `paint${id}`);
}

//========================================================================================================================================

function toolFuncPaintApply() {
    let modeElem = document.getElementById("toolmenu-paint-funcmode");
    let mode = modeElem.options[modeElem.selectedIndex].value;

    console.log("toolFuncPaintApply() ", mode);

    switch (mode) {
        case 'frameall':
            handleColorPickerSelection(stateSeq.selectedFrame, 0, SelSeq.colorMode, 'frameall', stateUX.paintFuncColor);
            break;
        case 'framezone':
            handleColorPickerSelection(stateSeq.selectedFrame, 0, SelSeq.colorMode, 'framezone', stateUX.paintFuncColor);
            break;
        case 'zoneall':
            handleColorPickerSelection(0, 0, SelSeq.colorMode, 'zoneall', stateUX.paintFuncColor);
            break;
        case 'frameeven':
            systemUndoRedoPush('seq');
            stateUX.undoRedoDisable = true;
            for (let i = 0; i < SelSeq.numPixels; i++) {
                if ((i % 2) == 0) handleColorPickerSelection(stateSeq.selectedFrame, i, SelSeq.colorMode, 'pixel', stateUX.paintFuncColor);
            }
            stateUX.undoRedoDisable = false;
            break;
        case 'frameodd':
            systemUndoRedoPush('seq');
            stateUX.undoRedoDisable = true;
            for (let i = 0; i < SelSeq.numPixels; i++) {
                if ((i % 2) == 1) handleColorPickerSelection(stateSeq.selectedFrame, i, SelSeq.colorMode, 'pixel', stateUX.paintFuncColor);
            }
            stateUX.undoRedoDisable = false;
            break;
    } //end switch
    layoutUpdateBuffers();
}

//========================================================================================================================================

function toolPlaybackPreview(arg) {
    console.log("toolPlaybackPreview() " + arg);
    let mult = getColorModeMult(SelSeq.colorMode);

    switch (arg) {
        case 'play':

            if (SelSeq.mode == 'mList') {
                pushMessageToLog("Software Preview playback does not support List sequence modes.");
                return;
            }

            if (SelSeq.mode == 'mScene') {
                pushMessageToLog("Software Preview playback will have no affect with the Scene sequence mode.");
                return;
            }

            stateUX.softwarePrevPlay = true;
            pushMessageToLog("Software Preview playback has started. Output Speed and Colors may not be accurate.");
            let theIcons = document.getElementById("ws-graphic-layout-container").querySelectorAll("div.pxSz");

            document.getElementById('toolmenu-playback-play').style.animation = "notificationborder 2s infinite"; //add user indication
            document.addEventListener('mousedown', function () {
                toolPlaybackPreview('stop');
            }, { once: true })
            //-------------------------- Clone the sequence so start and frameVars are unaffected -------------------------------------------------

            let cloneSeq = { ...SelSeq };
            if (SelSeq.hasOwnProperty("startVals")) cloneSeq.startVals = Array.from(SelSeq.startVals);
            if (SelSeq.hasOwnProperty("frameVar")) cloneSeq.frameVar = Array.from(SelSeq.frameVar);

            cloneSeq.softwarePreview = true; //tracker to prevent preload injection

            stateSeq.progress = 0;

            //-------------------------- Setup styles and classes -------------------------------------------------
            for (let i = 0; i < theIcons.length; i++) {
                if (document.getElementById('timeline-indicator' + i) != null) document.getElementById('timeline-indicator' + i).style.transition = "left 0ms";
                theIcons[i].style.transition = 'background-color 0ms'; //make it instant, can't remove it, it overwrites
                theIcons[i].style.removeProperty('border-color');
            }

            if (SelSeq.mode == 'mInstant') {
                //load start value colors int pixel icons, rather than frame 
                for (let i = 0; i < SelSeq.numPixels; i++) {
                    if (i < theIcons.length) theIcons[i].style.backgroundColor = rgbToHex(...sequenceGetPixelData(cloneSeq, 'begin', i, cloneSeq.frameVar[i * mult]));
                    cloneSeq.frameVar[i * mult]++;
                    if (cloneSeq.frameVar[i * mult] >= SelSeq.frames) cloneSeq.frameVar[i * mult] = 0;
                }
            }

            stateUX.softwarePlayCount = 0;

            let playbackSpeed = 25;
            //  if (SelSeq.mode == 'mInstant' || SelSeq.mode == 'mPOV' || SelSeq.mode == 'mFile' || SelSeq.speed > 25)
            playbackSpeed = SelSeq.speed;

            clearInterval(stateUX.softwarePrevTimer);
            stateUX.softwarePrevTimer = setInterval(function () {
                document.getElementById("toolmenu-playback-progress").textContent = Math.round(stateSeq.progress) + "%";
                sequenceRender(SelSeq, cloneSeq, theIcons, true);
            }, playbackSpeed);
            break;
        case 'pause':

            break;
        case 'stop':

            if (stateUX.softwarePrevPlay) {
                clearInterval(stateUX.softwarePrevTimer);
                stateUX.softwarePrevPlay = false;
                pushMessageToLog("Software Preview stopped.");

                document.getElementById('toolmenu-playback-play').style.animation = "";

                if (cSeqModesStartVals.includes(SelSeq.mode)) for (let i = 0; i < SelSeq.numPixels; i++) sequenceLoadStartValPositions(SelSeq, i);
                //layoutTimelineRender(); //need it to revert the indicators class transition change via inline css
                //layoutUpdateBuffers();
                refreshLayouts();
            }
            break;
    } //end switch
}

//========================================================================================================================================

function toolsCopyFrame() {
    console.log("toolsCopyFrame()");
    //can be a shallow copy or reference
    stateSeq.copyFrame = SelSeq.dataFrames[stateSeq.selectedFrame]; //gradient? over size, under size?
    pushMessageToLog(`Color data for frame #${(stateSeq.selectedFrame + 1)} has been copied.`);
    //console.log(stateSeq.copyFrame);
}

//========================================================================================================================================

function toolsPasteFrame() {
    console.log("toolsPasteFrame()");
    let newArr;
    let chans = SelSeq.numPixels * getColorModeMult(SelSeq.colorMode);

    systemUndoRedoPush('seq'); //run before any changes

    //zones not respected......
    //Have to resize a typed array, so extra steps - this expects 'undefined' array elements to default to 0
    if (SelSeq.mode == 'mGradient') {
        newArr = Array.from(stateSeq.copyFrame);
        newArr.length = 4; //crop array down
        SelSeq.dataFrames[stateSeq.selectedFrame] = Uint8Array.from(newArr);
    }
    else if (stateSeq.copyFrame.length != chans) {
        let newArr = Array.from(stateSeq.copyFrame);
        newArr.length = chans; //crop array down
        SelSeq.dataFrames[stateSeq.selectedFrame] = Uint8Array.from(newArr);
    }
    else if (stateSeq.copyFrame.length < chans) {
        let newArr = Array.from(stateSeq.copyFrame);
        newArr.length = chans; //expand array - will be undefined.....
        SelSeq.dataFrames[stateSeq.selectedFrame] = Uint8Array.from(newArr); //no undefined will get set to 0
    }
    else SelSeq.dataFrames[stateSeq.selectedFrame] = Uint8Array.from(stateSeq.copyFrame);   //[...stateSeq.copyFrame];

    pushMessageToLog(`Color data has been pasted into frame #${(stateSeq.selectedFrame + 1)}.`);
    layoutUpdateBuffers();
}

//========================================================================================================================================

function uxToolsUpdatePaintRect(param, color) {
    let colSq = document.getElementById("toolmenu-paint-col" + param);
    colSq.style.backgroundColor = rgbToHex(...color);
    if (SelSeq.colorMode != 'RGB') colSq.style.backgroundImage = "linear-gradient(180deg, transparent 60%, " + rgbToHex(color[3], color[3], color[3]) + " 40%)";
    else colSq.style.removeProperty('background-image');
    document.getElementById("toolmenu-paint-col" + param + "-label").innerHTML = `R: ${color[0]}<br>G: ${color[1]}<br>B: ${color[2]}<br>W: ${color[3]}`; //terrible place for this
}

//========================================================================================================================================

