'use strict';

//========================================================================================================================================

function pushMessageToLog(msg) {
	console.log("pushMessageToLog: " + msg);
	let timeStamp = new Date().toLocaleTimeString();
	let theDiv = document.getElementById("status-bar-message-container");
	let content = document.createElement("div");
	content.innerHTML = (timeStamp + ":   " + msg);
	theDiv.prepend(content);  //not appendChild since it is placing message on the top/highest/beginning of the div

	document.getElementById("status-bar-openicon").classList.add('active');

	setTimeout(function () {
		document.getElementById("status-bar-openicon").classList.remove('active');
	}, 200);

	//When a new message is pushed, it needs to ensure it is displayed, move scroll bar to top
	theDiv.scrollTop = 0; //theDiv.scrollHeight
}

//========================================================================================================================================

function uxCreateFrame(key) {
	//console.log("uxCreateFrame() with " + key);
	const theContainer = document.getElementById('frame-icons-container');
	let textValue = (key + 1);
	let theFrame = document.createElement('div');
	theFrame.id = `frame-icon-${key}`;
	theFrame.classList.add('frame-icon');
	theFrame.textContent = textValue;
	if (textValue.toString().length > 4) theFrame.style.fontSize = "12px"; //resize the font based on the character width of the number
	else if (textValue.toString().length > 3) theFrame.style.fontSize = "16px";
	else if (textValue.toString().length > 2) theFrame.style.fontSize = "20px";
	theFrame.addEventListener('click', function () { selectSeqFrame(key) });
	theContainer.appendChild(theFrame);
}

//========================================================================================================================================

function uxFlashElementBlue(id) {
	//console.log(document.getElementById(id));
	document.getElementById(id).style.backgroundColor = "blue";
	setTimeout(function () {
		document.getElementById(id).style.removeProperty('background-color');
	}, 250);
}

//========================================================================================================================================

function uxFrameButtonsRender(flag = true) {
	document.getElementById("frame-icons-container").innerHTML = ''; //easy
	document.getElementById("frame-max").innerText = SelSeq.frames;
	document.getElementById("frame-input").max = SelSeq.frames;
	for (let i = 0; i < SelSeq.frames; i++) uxCreateFrame(i);
	if (flag) selectSeqFrame(0, false); 	//select the current frame
	else selectSeqFrame(stateSeq.selectedFrame, false); 	//select the current frame

	if (SelSeq.mode == 'mGradient') {
		//update frame button colors, I guess here....
		let theButtons = document.getElementById('frame-icons-container').querySelectorAll("div");
		theButtons.forEach(function (val, index) {
			getFrameSolid(theButtons[index], 0, index);
		});
	}


	utilityAwaitDOMUpdate(() => { enableDragSort('frame-icon') });
}

//========================================================================================================================================

function selectLeftFrameClick(type = event.type) {
	console.log("selectLeftFrameClick() ", type);
	if (type == 'click' || type == 'mousedown') uxFlashElementBlue('frame-left-button'); //ignore mousewheel
	if (stateUX.frameScroll.target >= 0)
		stateUX.frameScroll.target = (stateUX.frameScroll.target - 55);
	uxFrameBarScrollLeft(10);
}

//========================================================================================================================================

function selectRightFrameClick(type = event.type) {
	console.log("selectRightFrameClick() ", type);
	if (type == 'click' || type == 'mousedown') uxFlashElementBlue('frame-right-button'); //ignore mousewheel
	const cont = document.getElementById('frame-icons-container');
	if (stateUX.frameScroll.target <= (cont.scrollWidth - cont.clientWidth))
		stateUX.frameScroll.target = (stateUX.frameScroll.target + 55);
	uxFrameBarScrollRight(10);
}

//========================================================================================================================================

function selectCheckFrameAmount(tag) {
	//no need to check the sequence mode, this checks if disabled which is done elsewhere
	if (tag == 'subtract') {
		if (document.getElementById('frame-subtract-button').getAttribute("disabled")) {
			pushMessageToLog("This sequence mode can only have one frame.");
			return true;
		}
	}
	else if (tag == 'add') {
		if (document.getElementById('frame-add-button').getAttribute("disabled")) {
			pushMessageToLog("This sequence mode can only have one frame.");
			return true;
		}
	}
	return false;
}

//========================================================================================================================================

function commonFrameNumberEdit() {
	document.getElementById("frame-max").innerText = SelSeq.frames;
	document.getElementById("frame-input").max = SelSeq.frames;
	sequenceFrameAmountUpdate(); //Aurora handler
	uxFrameButtonsRender(false);
	sequenceValidateData(SelSeq);
	refreshLayouts(); //Aurora function

}

//========================================================================================================================================

function selectInsertFrame() {
	console.log("selectInsertFrame()");
	uxDisableEnabledTools(); // HACKY - otherwise manual paint could stay enabled but the cursor changes when timeline re-renders. 

	let newFrame;
	if (SelSeq.mode == 'mGradient') newFrame = new Uint8Array(4).fill(0); //create an array intialized with all 0s
	else newFrame = new Uint8Array(sequenceGetChannelAmount(SelSeq)).fill(0); //create an array intialized with all 0s

	if (SelSeq.mode == 'mList') {
		newFrame = 0;
		SelSeq.listID.splice(stateSeq.selectedFrame + 1, 0, newFrame);
		SelSeq.listFlags.splice(stateSeq.selectedFrame + 1, 0, newFrame);
		SelSeq.listIteration.splice(stateSeq.selectedFrame + 1, 0, newFrame);
	}
	else SelSeq.dataFrames.splice(stateSeq.selectedFrame + 1, 0, newFrame);
	SelSeq.frames++;
	//----------------------------------------------------------------------------------------
	commonFrameNumberEdit();
}

//========================================================================================================================================

function selectRemoveFrame() {
	console.log("selectRemoveFrame()", stateSeq.selectedFrame);

	// console.log(JSON.stringify(SelSeq));

	if (SelSeq.frames < 3) return; //minimum of 2 frames except for mScene
	uxDisableEnabledTools(); // HACKY - otherwise manual paint could stay enabled but the cursor changes when timeline re-renders. 


	SelSeq.frames--;
	if (SelSeq.mode == "mList") {
		SelSeq.listID.splice(stateSeq.selectedFrame, 1);
		SelSeq.listFlags.splice(stateSeq.selectedFrame, 1);
		SelSeq.listIteration.splice(stateSeq.selectedFrame, 1);
	}

	else {
		SelSeq.dataFrames.splice(stateSeq.selectedFrame, 1);

		let holdFrameNum = stateSeq.selectedFrame;
		selectSeqFrame(holdFrameNum, false);
	}






	//----------------------------------------------------------------------------------------
	commonFrameNumberEdit();
	console.log(JSON.stringify(SelSeq));
}

//========================================================================================================================================

function selectAddFrame() {
	if (selectCheckFrameAmount('subtract')) return; //aurora specific
	uxFlashElementBlue('frame-add-button'); //indicate to user
	systemUndoRedoPush('seq');
	uxDisableEnabledTools(); // HACKY - otherwise manual paint could stay enabled but the cursor changes when timeline re-renders. 
	//Can't change the cursor now, it is overwritten

	if (SelSeq.frames >= stateSoftware.maxFrames) {
		pushMessageToLog("Maximum number of frames reached");
		return;
	}
	SelSeq.frames++;

	// uxCreateFrame(SelSeq.frames - 1);

	const cont = document.getElementById('frame-icons-container');
	stateUX.frameScroll.target = (cont.scrollWidth - cont.clientWidth);

	if (stateUX.frameScroll.target < 500) uxFrameBarScrollRight(5); //scroll if it is not too far
	else cont.scrollLeft = stateUX.frameScroll.target; //or just jump to the end

	//----------------------------------------------------------------------------------------
	commonFrameNumberEdit();

}

//========================================================================================================================================

function selectSubtractFrame(elem) {
	if (selectCheckFrameAmount('subtract')) return; //aurora specific
	uxFlashElementBlue('frame-subtract-button'); //indicate to user

	systemUndoRedoPush('seq');

	uxDisableEnabledTools(); // HACKY - otherwise manual paint could stay enabled but the cursor changes when timeline re-renders. 
	//Can't change the cursor now, it is overwritten

	if (SelSeq.frames <= stateSeq.minFrames) {
		pushMessageToLog("Minimum number of frames reached");
		return;
	}
	SelSeq.frames--;

	const cont = document.getElementById('frame-icons-container');
	cont.removeChild(cont.lastChild);

	stateUX.frameScroll.target = (cont.scrollWidth - cont.clientWidth);

	if (stateUX.frameScroll.target < 500) uxFrameBarScrollRight(5); //scroll if it is not too far
	else cont.scrollLeft = stateUX.frameScroll.target; //or just jump to the end

	//----------------------------------------------------------------------------------------
	commonFrameNumberEdit();
}

//========================================================================================================================================


let draggedFrameIcon;
let droppedFrameIcon;
let indicatorElem;

function enableDragSort(listClass) {
	// console.log("enableDragSort()")
	const iterable = document.getElementsByClassName(listClass);
	for (let item of iterable) {
		enableDragItem(item)
	}
}

function enableDragItem(item) {
	// console.log("enableDragItem()")
	item.setAttribute('draggable', true)
	item.addEventListener('drag', function () { handleDrag(this) });
	item.addEventListener('dragend', function () { handleDrop(this) });
	item.addEventListener('dragstart', function () {
		draggedFrameIcon = Array.prototype.indexOf.call(item.parentNode.children, item);

		for (let i = 0; i < item.parentNode.children.length; i++) {
			console.log(item.parentNode.children[i])
			item.parentNode.children[i].classList.remove("active")
		}

		indicatorElem = document.createElement('div');
		indicatorElem.id = 'tempElem';
		indicatorElem.style.width = "0px";
		indicatorElem.style.height = "100%";
		//  indicatorElem.style.backgroundColor = "green";
		indicatorElem.style.outline = "4px solid #0FF";
		// indicatorElem.style.border = "2px dashed #F00";
		// indicatorElem.style.boxSizing ="content-box"
		indicatorElem.style.outlineOffset = "2px"
		indicatorElem.style.zIndex = "99"
		item.parentNode.insertAdjacentElement("beforebegin", indicatorElem)
	});

}

function handleDrag(item) {
	// console.log("handleDrag()", item)
	const list = item.parentNode;

	try {
		if (document.getElementById('dummyElem') == null) {
			//dummy element allows dropping after the last frame icon
			let dummy = document.createElement('div');
			dummy.id = 'dummyElem';
			dummy.classList = "frame-icon"
			dummy.style.opacity = "0"; //hide element but keep in DOM flow
			item.parentNode.insertAdjacentElement("beforeend", dummy)
		}
	}
	catch (e) {
	}

	let swapItem = document.elementFromPoint(event.clientX, event.clientY) === null ? item : document.elementFromPoint(event.clientX, event.clientY);

	if (item.parentNode === swapItem.parentNode) {
		swapItem.insertAdjacentElement("beforebegin", indicatorElem)
	}
}

function handleDrop(item) {
	console.log("handleDrop()", item, draggedFrameIcon);

	droppedFrameIcon = Array.prototype.indexOf.call(item.parentNode.children, indicatorElem);
	// console.log(droppedFrameIcon)

	Array.prototype.move = function (from, to) {
		this.splice(to, 0, this.splice(from, 1)[0]);
	};

	indicatorElem.remove();

	let dropContainer = document.elementFromPoint(event.clientX, event.clientY);

	// console.log(dropContainer);
	if (item.parentNode === dropContainer || item.parentNode === dropContainer.parentNode) {
		//confirmed dropped in frame container, otherwise ignore
		if (draggedFrameIcon != droppedFrameIcon && draggedFrameIcon != (droppedFrameIcon - 1)) {
			if (draggedFrameIcon < 2) {
				if (SelSeq.mode == 'mList') {
					SelSeq.listID.move(draggedFrameIcon, ((droppedFrameIcon == 0) ? 0 : droppedFrameIcon - 1));
					SelSeq.listFlags.move(draggedFrameIcon, ((droppedFrameIcon == 0) ? 0 : droppedFrameIcon - 1));
					SelSeq.listIteration.move(draggedFrameIcon, ((droppedFrameIcon == 0) ? 0 : droppedFrameIcon - 1));
				}
				else SelSeq.dataFrames.move(draggedFrameIcon, ((droppedFrameIcon == 0) ? 0 : droppedFrameIcon - 1));
			}
			else {
				if (SelSeq.mode == 'mList') {
					SelSeq.listID.move(draggedFrameIcon, droppedFrameIcon);
					SelSeq.listFlags.move(draggedFrameIcon, droppedFrameIcon);
					SelSeq.listIteration.move(draggedFrameIcon, droppedFrameIcon);
				}
				else SelSeq.dataFrames.move(draggedFrameIcon, droppedFrameIcon);
			}
		}
	}
	uxFrameButtonsRender(false);
	refreshLayouts();
}

//========================================================================================================================================

function uxFrameBarScrollRight(time) {
	const cont = document.getElementById('frame-icons-container');
	clearInterval(stateUX.frameScroll.scrollTimer);
	stateUX.frameScroll.scrollTimer = setInterval(function () {
		//console.log("tick", cont.scrollLeft, stateUX.frameScroll.target, cont.clientWidth);
		cont.scrollLeft += 5;
		if (cont.scrollLeft >= stateUX.frameScroll.target || cont.scrollLeft >= (cont.scrollWidth - cont.clientWidth)) {
			clearInterval(stateUX.frameScroll.scrollTimer);
		}
		if (stateUX.frameScroll.target >= (cont.scrollWidth - cont.clientWidth)) stateUX.frameScroll.target = (cont.scrollWidth - cont.clientWidth);
	}, time);
}

//========================================================================================================================================

function uxFrameBarScrollLeft(time) {
	const cont = document.getElementById('frame-icons-container');
	clearInterval(stateUX.frameScroll.scrollTimer);
	stateUX.frameScroll.scrollTimer = setInterval(function () {
		//console.log("tick", cont.scrollLeft, stateUX.frameScroll.target, cont.clientWidth);
		cont.scrollLeft -= 5;
		if (cont.scrollLeft <= stateUX.frameScroll.target || cont.scrollLeft <= 0) {
			clearInterval(stateUX.frameScroll.scrollTimer);
		}
		if (stateUX.frameScroll.target < 0) stateUX.frameScroll.target = 0;
	}, time);
}

//========================================================================================================================================

function selectSeqFrame(num, flag = true) {
	console.log("selectSeqFrame() ", num, flag);

	let theContainer = document.getElementById('frame-icons-container')

	let theIcons = theContainer.querySelectorAll('div');
	theIcons.forEach(function (el) {
		el.classList.remove('active');
	});

	theIcons[num].classList.add('active');

	if (flag == true) { 	//user click
		if (SelSeq.mode == 'mGradient' && stateSeq.selectedFrame == num) {
			stateSeq.selectedFrame = num;
			colorPicker.open(400, 200, SelSeq.colorMode, num, 0, 'pixel'); //STATIC POSITION
		}
		else {
			stateSeq.selectedFrame = num;
			layoutUpdateBuffers(); // special could roll in with timeline
		}
	}
	else {
		//function click
		stateSeq.selectedFrame = num;
		//ensure the selected frame icon is in view
		let moveTo = (theIcons[num].offsetLeft - theContainer.offsetLeft - 50);
		if (moveTo < 0) moveTo = 0;
		document.getElementById('frame-icons-container').scrollLeft = moveTo;
	}

	document.getElementById("frame-input").value = (stateSeq.selectedFrame + 1);
}

//========================================================================================================================================

function uxToolTipShow(elem) {
	//console.log("Show it" + elem.getAttribute("data-tiptext"));
	if (stateSoftware.tooltipsEnabled != true) return; //check user setting

	const coords = elem.getBoundingClientRect(); //heard this is really slow......
	const tipElem = document.getElementById('global-tooltip');
	const tipCoords = tipElem.getBoundingClientRect(); //heard this is really slow...... just need width and height

	tipElem.classList.replace('g-tooltip-hide', 'g-tooltip-show');
	tipElem.innerHTML = elem.getAttribute("data-tiptext"); //set tip element text to the stored tip string
	//console.log(tipCoords); //DEBUG
	//console.log(coords); //DEBUG

	//auto assign direction of tip bubble
	//possibly, scan class/ID, get the CSS somehow calculate its absolute location relative to the window/viewport/document??
	//	then get it's width, and using the window size, figure out if there is room to render the tooltip in what direction

	//some directions use either of the 4 positioning tags
	tipElem.style.bottom = null;
	tipElem.style.top = null;
	tipElem.style.left = null;
	tipElem.style.right = null;

	//now set the dir
	//DOMRect {x: -1, y: 178, width: 42, height: 42, top: 178, …}
	switch (elem.getAttribute('data-tipdir')) {
		case 'left':
		case null:
		case undefined:
		//default to up if the attribute is not set
		case 'up':
			if ((coords.x + tipCoords.width) > window.innerWidth) tipElem.style.right = '5px'; //if off the right side
			else if ((coords.x + (coords.width / 2)) - (tipCoords.width / 2) < 5) tipElem.style.left = '5px'; //, if off left side,
			else tipElem.style.left = coords.x - (tipCoords.width / 2) + (coords.width / 2) + 'px'; //else center over element
			tipElem.style.top = ((coords.y - tipCoords.height) - 5) + 'px';
			//5 is just a constant padding value
			break;
		case 'right':
			//console.log('right');
			tipElem.style.left = coords.x + coords.width + 5 + 'px';
			tipElem.style.top = (coords.y + (coords.height / 2)) - (tipCoords.height / 2) + 'px';
			break;

		case 'down':
			//console.log('down');
			if ((coords.x + tipCoords.width) > window.innerWidth) tipElem.style.right = '5px'; //if off the right side
			else if ((coords.x + (coords.width / 2)) - (tipCoords.width / 2) < 5) tipElem.style.left = '5px'; //, if off left side,
			else tipElem.style.left = coords.x - (tipCoords.width / 2) + (coords.width / 2) + 'px'; //else center below element
			tipElem.style.top = (coords.y + coords.height) + 5 + 'px';
			//5 is just a constant padding value
			break;
	}
}

//========================================================================================================================================

function uxToolTipHide() {
	//console.log("Hide it")
	document.getElementById('global-tooltip').classList.replace('g-tooltip-show', 'g-tooltip-hide');
}

//========================================================================================================================================

function uxClampNumberBox(elem) {
	//console.log("uxClampNumberBox()", elem.id, elem.value, elem.max, elem.min)
	if (parseInt(elem.value) > parseInt(elem.max)) elem.value = elem.max;
	else if (parseInt(elem.value) < parseInt(elem.min)) elem.value = elem.min;
}

//========================================================================================================================================

//funcion that is atached to all Number Steppers automatically at program start, unless overwritten
function uxElemNumberStepper(elem, dir) {
	//console.log("uxElemNumberStepper(" + elem + ", " + dir + ")")
	//dir options - up, down, set(when user enters value manually)
	let numberBox = elem.parentNode.querySelectorAll('input')[0];
	//console.log(numberBox)
	//console.log(numberBox.value,numberBox.min,numberBox.max)

	let tmp;

	//keeps value in range if user has manually entered a number
	if (parseInt(numberBox.value) > numberBox.max) numberBox.value = numberBox.max;
	else if (parseInt(numberBox.value) < numberBox.min) numberBox.value = numberBox.min;
	//now if it was in range, the function can be ran
	else if (dir == 'up') {
		tmp = parseInt(numberBox.value);
		if (tmp < numberBox.max) numberBox.value = tmp + 1;
	}
	else if (dir == 'down') {
		tmp = parseInt(numberBox.value);
		if (tmp > numberBox.min) numberBox.value = tmp - 1;
	}
	return parseInt(numberBox.value); //ensure it is a number
}

//========================================================================================================================================

//used to change tabs from buttons other than the top tab buttons
function uxTriggerElementClick(elId) {
	console.log("uxTriggerElementClick()", elId)
	document.getElementById(elId).click();
}

//========================================================================================================================================

