'use strict';

//========================================================================================================================================

window.addEventListener('resize', function (e) {
    e.preventDefault();
    clearTimeout(stateUX.windowResizeTimer);
    stateUX.windowResizeTimer = setTimeout(uxWindowResize, 100);
});

function uxWindowResize() {
    console.log("uxWindowResize()")
    let w = window.innerWidth;
    let h = window.innerHeight;
    if (stateSoftware.screenWidth != w || stateSoftware.screenHeight != h) {
        console.log("Window resized to " + w, h + "  from " + stateSoftware.screenWidth, stateSoftware.screenHeight);
        stateSoftware.screenWidth = w;
        stateSoftware.screenHeight = h;
        fileSaveSettingsFile(); //now save new window size
        uxAdjustZoom();
        refreshLayouts(); //always
    }
    //else window was resized to the same size, ignore it. This happens when window is min/maximized normally.
}

//========================================================================================================================================

function uxAdjustZoom() {
    //console.log(win.getSize()); //actual window size
    //console.log(win.getContentSize()); //size of the content without insets and junk
    //console.log(window.screen.availHeight, window.screen.availWidth);
    let insets = win.getSize()[0] - win.getContentSize()[0];
    if (insets == 0) insets = 40; //fixes a bug with linux not reporting the correct window size(does not include frame/insets/title bar) and windows/mac. 40 is a tested guess.

    if (window.screen.availHeight < (768 + insets)) {
        win.setMinimumSize(1024, (window.screen.availHeight - insets));
        win.setSize(1024, (window.screen.availHeight - insets));
        let factor = (window.screen.availHeight - insets) / 768;
        webFrame.setZoomFactor(factor);
    }
    else webFrame.setZoomFactor(1);
}

//========================================================================================================================================

function selectConfirmBox(arg, yesCallback, noCallback) {
    console.log("selectConfirmBox() " + arg)
    let lang = stateSoftware.language;
    console.log(userConfirm[arg].text[lang]);

    stateUX.userConfirmOpen = true;

    document.getElementById('toplevel-fog').classList.remove('visibility-hidden');
    document.getElementById('toplevel-fog').classList.add('visibility-visible');

    document.getElementById('overlay-confirmbox-container').classList.remove('visibility-hidden');
    document.getElementById('overlay-confirmbox-container').classList.add('visibility-visible');

    if (userConfirm[arg].title[stateSoftware.language] == undefined) lang = 'en' //default language if specified is not defined
    else lang = stateSoftware.language;
    document.getElementById('overlay-confirmbox-text').innerHTML = "<h3>" + userConfirm[arg].title[lang] + "</h3>";

    if (userConfirm[arg].text[stateSoftware.language] == undefined) lang = 'en' //default language if specified is not defined
    else lang = stateSoftware.language;
    document.getElementById('overlay-confirmbox-text').innerHTML += "<p>" + userConfirm[arg].text[lang] + "</p>";

    document.getElementById('overlay-confirmbox-yes').addEventListener('click', function () { yesCallback(); selectConfirmBoxClose() }, { once: true }); //runs yes callback and close handler, once is required
    document.getElementById('overlay-confirmbox-yes').focus(); //makes enter work, and ensures pressing enter won't click the button that called this function again.

    //if no 'decline' callback was specified, just run the common which hides the menu and fog
    if (noCallback == undefined) document.getElementById('overlay-confirmbox-no').addEventListener('click', function () { selectConfirmBoxClose() }); //sets no callback and always calls close handler
    else document.getElementById('overlay-confirmbox-no').addEventListener('click', function () { noCallback(); selectConfirmBoxClose() }, { once: true }); //once is required or multiple 'click' events will be attached
}

function selectConfirmBoxClose() {
    //console.log("selectConfirmBoxClose()");
    stateUX.userConfirmOpen = false;
    document.getElementById('toplevel-fog').classList.replace('visibility-visible', 'visibility-hidden');
    document.getElementById('overlay-confirmbox-container').classList.replace('visibility-visible', 'visibility-hidden');
}

//========================================================================================================================================

function uxProgressBarEvent() {
    uxProgressBarShow();
    var timer = setInterval(function () {
        //console.log('progress:', auroraCMD.progress); //DEBUG
        if (auroraCMD.progress < 100) {
            if (auroraCMD.progress > 0) uxProgressBarFunction(auroraCMD.progress, 100); //if progress is 0, it could be faking, wait
        }
        else if (auroraCMD.state == 0) { //else it is over 100 and is done, or it is 'err' and should stop and hide
            clearInterval(timer);
            uxProgressBarHide();
        }
    }, 50);
}

function uxProgressBarFake(start, end, time) {
    uxProgressBarShow();
    let fakeTime = start; //fake progress
    let fakeIt = setInterval(function () {
        // console.log('fakeit', fakeTime, end); //DEBUG
        fakeTime++;
        if (auroraCMD.progress == 0) uxProgressBarFunction(fakeTime, end); //show fake progress out of 100
        else {
            clearInterval(fakeIt); //end fake timer, real progress timer takes over
        }

        if (fakeTime >= end || auroraCMD.progress == 'abort') {
            clearInterval(fakeIt); //end fake timer, real progress timer takes over
            uxProgressBarHide();
        }
    }, time);
}


function uxProgressBarInternal(end) {
    //end is animation frames ticks
    uxProgressBarShow();
    let timer = 0;


    const animate = () => {
        timer++;
        uxProgressBarFunction(timer, end); //show fake progress out of 100
        if (timer >= end) {
            //clearInterval(barTimer); //end fake timer, real progress timer takes over
            uxProgressBarHide();
        }
        else requestAnimationFrame(animate);
    }

    requestAnimationFrame(animate);


    // let barTimer = setInterval(function () {
    //     timer++;
    //     uxProgressBarFunction(timer, end); //show fake progress out of 100
    //     if (timer >= end) {
    //         clearInterval(barTimer); //end fake timer, real progress timer takes over
    //         uxProgressBarHide();
    //     }
    // }, time);
}

function uxProgressBarShow(val = 0) {
    //val allows preloading of bar progress
    //fog won't show if is started by a confirm dialog, really don't need it.
    //setTimeout(function () { document.getElementById('toplevel-fog').classList.replace('visibility-hidden', 'visibility-visible'); }, 50)
    document.getElementById('progressbar-container').classList.replace('visibility-hidden', 'visibility-visible');
    document.getElementById("progressbar-progress").style.width = val + "%";
    document.getElementById("progressbar-progress-num").innerText = val + "%";
}

function uxProgressBarHide() {
    // document.getElementById('toplevel-fog').classList.replace('visibility-visible', 'visibility-hidden');
    document.getElementById('progressbar-container').classList.replace('visibility-visible', 'visibility-hidden');
    uxProgressBarFunction(0, 0);
}

function uxProgressBarFunction(current, end) {
    let prog = Math.floor((current / end) * 100);
    document.getElementById("progressbar-progress").style.width = prog + "%";
    document.getElementById("progressbar-progress-num").innerText = prog + "%";
}

//========================================================================================================================================

function userCancelProgress() {
    console.log("userCancelProgress() - NOT FULLY IMPLEMENTED")
    //what to do? confirm or not?
    //kill any serial transactions, what else is progress bar for?
    auroraCMD.abort();
    uxProgressBarHide();
}

//========================================================================================================================================

function uxGlobalIntensityVal(elem) {
    console.log("uxGlobalIntensityVal()");
    mainObj.globalIntensity = parseInt(elem.value);
}

//========================================================================================================================================

function uxTargetLoadedFile(filePath) {
    console.log("uxTargetLoadedFile() ")
    stateUX.targetSaveFile = filePath;
    uxTargetSaveFileUpdate();
}

//========================================================================================================================================

function uxTargetSaveFileUpdate() {
    console.log("uxTargetSaveFileUpdate() ",stateSoftware.appTitle)
    document.title = stateSoftware.appTitle + stateUX.targetSaveFile;
    fileSaveSettingsFile();
}

//========================================================================================================================================

function uxButtonIndication(id, flag) {
    console.log("uxButtonIndication() " + id, flag)
    if (flag) document.getElementById(id).style.animation = "notificationborder 2s infinite";
    else document.getElementById(id).style.removeProperty('animation');
}

//========================================================================================================================================


function uxMenuImageWizardUpdate() {
    document.getElementById("ovmenu-importwizard-filespath").innerText = SelSeq.filePath;
    let modeDD = document.getElementById("ovmenu-importwizard-mode");
    let startDD = document.getElementById("ovmenu-importwizard-startpos");
    let actionDD = document.getElementById("ovmenu-importwizard-action");
    document.getElementById("ovmenu-importwizard-framenum").innerText = " "+Number(stateSeq.selectedFrame+1)



    actionDD.disabled = true;
    startDD.disabled = false;
    if (modeDD.selectedIndex == 0) {
        //2D
        startDD.disabled = true;
        actionDD.disabled = false;
        startDD.innerHTML = `<option value="">As Map</option>`;
    }
    else if (modeDD.selectedIndex == 1 || modeDD.selectedIndex == 2) {
        //rows
        startDD.innerHTML = `<option value="left">Left</option> <option value="right">Right</option>`;
    }
    else if (modeDD.selectedIndex == 3 || modeDD.selectedIndex == 4) {
        //columns
        startDD.innerHTML = `<option value="top">Top</option>     <option value="bottom">Bottom</option>`;
    }
}

//========================================================================================================================================

function systemUndoRedoPush(type) {
    if (stateUX.undoRedoDisable) return;
    console.log("systemUndoRedoPush()", type, stateUX.undoRedoObj.length, stateUX.undoRedoLevel);

    stateUX.undoRedoStamp = new Date(); //tracks stuff, mostly for autoHWPV

    let obj = {};
    let data1;
    //let data2;

    switch (type) {
        default:
        case 'none':
            //console.warn("Error!");
            break;
        case 'seq':
            obj = SelSeq;
            data1 = stateSeq.index;
            break;
        case 'layout':
            obj = mainObj.layout;
            break;
        case 'paint1':
            obj = stateUX.paintManColor;
            break;
        case 'paint2':
            obj = stateUX.paintFuncColor;
            break;
    } //end switch

    //Replacer converts Unit8Arrays to regular arrays for writing.
    let objStr = (JSON.stringify(obj, function (k, v) {
        if (v instanceof Uint8Array) {
            return Array.apply([], v);
        }
        return v;
    })); //add it to first index - deep clone, it seemed to work but dataframes[][] are still by reference. confirmed, but why?.

    let stackEntry = {
        type: type,
        arg1: data1,
        //arg2: data2,
        //objRef: obj,
        objData: objStr
    }

    //console.warn(stackEntry);

    stateUX.undoRedoObj.push(stackEntry);

    if (stateUX.undoRedoObj.length > cMaxUndoStack) {
        stateUX.undoRedoObj.shift(); //remove oldest
        stateUX.undoRedoLevel--; //track it???????????? right maybe 
    }

    stateUX.undoRedoLevel++; //targets the element with the up-to-date data
    // console.warn(stateUX.undoRedoFlag, " LEVEL SET TO:", stateUX.undoRedoLevel);

    stateUX.undoRedoFlag = false;
    //once a new action occurs, the old redo(s) are invalid, branch ends so the rest of the actions can be removed
    stateUX.undoRedoObj.length = stateUX.undoRedoLevel + 1; //base 1 value

    //stateUX.undoRedoObj.unshift(Object.assign({},obj)); 
    //all data arrays convert to arrays, dropped uint8... fix or something or not
    // console.warn(stateUX.undoRedoObj);
}

//========================================================================================================================================

function systemUndo() {
    console.log("systemUndo() ", stateUX.undoRedoObj.length, stateUX.undoRedoLevel, stateUX.undoRedoFlag);
    if (stateUX.undoRedoLevel >= 0) {

        if (!stateUX.undoRedoFlag) {
            //Save current state, since states are only saved before action. So current state is not saved on the stack. G
            //console.warn("PUSHED CURRENT STATE ", stateUX.undoRedoObj[stateUX.undoRedoLevel].type, stateUX.undoRedoObj[stateUX.undoRedoLevel - 1].type);
            //This is the cause of the redo dead end. It pushes the current type, so any changes done to other types are untracked
            systemUndoRedoPush(stateUX.undoRedoObj[stateUX.undoRedoLevel].type); //assuming it is always SelSeq
            stateUX.undoRedoFlag = true; //do last cause push() would clear it again
        } //end flag check

        if (stateUX.undoRedoLevel <= 0) { console.log("Undo stack is empty"); return; }

        stateUX.undoRedoLevel--;
        // console.warn(stateUX.undoRedoFlag, " SETTING TO LEVEL:", stateUX.undoRedoLevel);
        systemUndoRedoApply();

        pushMessageToLog("Shortcut key triggered action undo");
    }
    else console.log("Undo stack is empty");
}

//========================================================================================================================================

function systemRedo() {
    console.log("systemRedo() " + stateUX.undoRedoObj.length, stateUX.undoRedoLevel);
    //stateUX.undoRedoObj.length is base 1, stateUX.undoRedoLevel is base 0
    if (stateUX.undoRedoObj.length > stateUX.undoRedoLevel && stateUX.undoRedoLevel != (stateUX.undoRedoObj.length - 1)) {
        if (stateUX.undoRedoLevel < 0) stateUX.undoRedoLevel = 0;
        else stateUX.undoRedoLevel++;
        //console.warn(stateUX.undoRedoFlag, " REDO " + stateUX.undoRedoObj[stateUX.undoRedoLevel].type + " TO LEVEL:", stateUX.undoRedoLevel, stateUX.undoRedoObj.length);
        systemUndoRedoApply();
        pushMessageToLog("Shortcut key triggered action redo");
    }
    else console.log("Redo stack is empty");
}

//========================================================================================================================================

function systemUndoRedoApply() {
    console.log("systemUndoRedoApply() " + stateUX.undoRedoObj[stateUX.undoRedoLevel].type)
    stateUX.undoRedoStamp = new Date(); //tracks stuff, mostly for autoHWPV
    switch (stateUX.undoRedoObj[stateUX.undoRedoLevel].type) {
        default:
        case 'none':
            console.log("Undo type is default or none. Error!");
            break;
        case 'seq':
            let seqID = stateUX.undoRedoObj[stateUX.undoRedoLevel].arg1; //shorten
            mainObj.seq[seqID] = JSON.parse(stateUX.undoRedoObj[stateUX.undoRedoLevel].objData);
            if (cSeqModesStartVals.includes(mainObj.seq[seqID].mode)) {
                mainObj.seq[seqID].startVals = Uint8Array.from(mainObj.seq[seqID].startVals); //convert to typed array
            }
            if (cDataSeqModesDataFrames.includes(mainObj.seq[seqID].mode)) {
                mainObj.seq[seqID].frameVar = Uint8Array.from(mainObj.seq[seqID].frameVar); //convert to typed array
            }
            for (let q = 0; q < mainObj.seq[seqID].frames; q++) {
                mainObj.seq[seqID].dataFrames[q] = Uint8Array.from(mainObj.seq[seqID].dataFrames[q]);
            }
            sequenceSelect(seqID);   //will change sequence if action was done on not the selected one
            break;
        case 'layout':
            let store = stateUX.undoRedoObj[stateUX.undoRedoLevel].objData;
            stateUX.undoRedoObj[stateUX.undoRedoLevel].objData = (JSON.stringify(mainObj.layout, function (k, v) {
                if (v instanceof Uint8Array) {
                    return Array.apply([], v);
                }
                return v;
            })); //add it to first index - deep clone, it seemed to work but dataframes[][] are still by reference. confirmed, but why?.
            mainObj.layout = JSON.parse(store);//JSON.parse(stateUX.undoRedoObj[stateUX.undoRedoLevel].objData);
            layoutGraphicIconsRender();
            uxToolsLayoutUpdate();
            if (stateUX.shapeCenterEnable) layoutShapeCenterDragEnd();
            break;
        case 'paint1':
            stateUX.paintManColor = JSON.parse(stateUX.undoRedoObj[stateUX.undoRedoLevel].objData);
            uxToolsUpdatePaintRect(1, stateUX.paintManColor)
            break;
        case 'paint2':
            stateUX.paintFuncColor = JSON.parse(stateUX.undoRedoObj[stateUX.undoRedoLevel].objData);
            uxToolsUpdatePaintRect(2, stateUX.paintFuncColor)
            break;
    } //end switch
}

//========================================================================================================================================

function refreshTab() {
    console.log("refreshTab() " + stateUX.activeTab);
    //clearInterval(stateUX.serialPortScanTimer); //turn off serial port scanning

    uxDisableEnabledTools(); //always I guess, just in case

    switch (stateUX.activeTab) {
        case 'navtab-graphic':
            if (stateUX.refreshGraphic == true) {
                stateUX.refreshGraphic = false;
                refreshLayouts('disable');
            }
            if (stateUX.updateGraphic == true) {
                layoutUpdateBuffers();
                stateUX.updateGraphic = false;
            }
            break;
        case 'navtab-timeline':
            if (stateUX.refreshTimeline == true) {
                stateUX.refreshTimeline = false;
                refreshLayouts('disable');
            }
            if (stateUX.updateTimeline == true) {
                layoutUpdateBuffers();
                stateUX.updateTimeline = false;
            }
            break;
        case 'navtab-connection':
            document.getElementById("ws-connection-enableintensityval").checked = mainObj.globalIntensityEnable;
            document.getElementById("ws-connection-intensity").value = mainObj.globalIntensity;
            renderIdleSequenceTile(mainObj.idleSeqID);
            sequenceMeasureAll();
            indexCalculateMemory();
            indexRefresh();
            break;
        case 'navtab-software':
            //none
            document.getElementById('ws-software-recentfilename').innerText = path.basename(stateSoftware.previousSaveFile);
            break;
        case 'navtab-hardware':
            //none
            break;
    } //end switch
}

//========================================================================================================================================

function uxToolsLayoutUpdate() {
    //console.log("uxToolsLayoutUpdate()");

    //-----------------------------------------------------------------------------------------

    document.getElementById("toolmenu-ledicon-iconsize").value = mainObj.layout.iconSize;
    document.getElementById("toolmenu-ledicon-icondensity").value = mainObj.layout.iconDensity;
    document.getElementById("toolmenu-ledicon-numofpixels").value = mainObj.layout.iconAmount;
    document.getElementById("toolmenu-ledicon-layoutshape").value = mainObj.layout.shape;

    document.getElementById("toolmenu-ledicon-rotation").value = mainObj.layout.rotation;
    document.getElementById("toolmenu-ledicon-rotation-label").innerText = mainObj.layout.rotation;

    if (mainObj.layout.iconShape == "round") document.getElementById("toolmenu-ledicon-iconshapeval").checked = true;
    else document.getElementById("toolmenu-ledicon-iconshapeval").checked = false;

    if (mainObj.layout.shapeDir == 'cw') document.getElementById("toolmenu-ledicon-dirval").checked = true;
    else document.getElementById("toolmenu-ledicon-dirval").checked = false;

    if (mainObj.layout.shape != "shapeMap" && mainObj.layout.shape != "shapeVector") {
        helper("toolmenu-ledicon-loadfile", false);
        helper("toolmenu-ledicon-createmap", false);
    }
    else {
        helper("toolmenu-ledicon-loadfile", true);
        helper("toolmenu-ledicon-createmap", true);
    }

    if (mainObj.layout.shape == "shapeMap") document.getElementById("toolmenu-ledicon-dirval").setAttribute('disabled', true);
    else document.getElementById("toolmenu-ledicon-dirval").removeAttribute('disabled');

    //if (mainObj.layout.fileName != undefined && (mainObj.layout.shape == "shapeMap" || mainObj.layout.shape == "shapeVector"))
    //    document.getElementById("toolmenu-ledicon-loadfile-path").innerText = mainObj.layout.fileName
    //else document.getElementById("toolmenu-ledicon-loadfile-path").innerHTML = "&nbsp;";

    //-----------------------------------------------------------------------------------------

    document.getElementById("toolmenu-ledicon-advancedval").checked = mainObj.layout.advanced;

    if (mainObj.layout.advanced) {
        helper("toolmenu-ledicon-zoneval", true); //true is enabled(so not disabled)
        helper("toolmenu-ledicon-addlayout", true);
        helper("toolmenu-ledicon-resetlayout", true);
        helper("toolmenu-ledicon-manplace", true);
        helper("toolmenu-ledicon-advplace", true);
        helper("toolmenu-ledicon-advnumofpixels", true);

        helper("toolmenu-zones-clear", true);
        helperStepper("toolmenu-zones-view", true);
        helper("toolmenu-zones-enableassignval", true);
        helperStepper("toolmenu-zones-assign", true);
        helper("toolmenu-zones-merge", true);

        helper("toolmenu-ledicon-reset", false);

        helperStepper("toolmenu-ledicon-zone", false); //disable

        if (stateUX.layoutAdvancedPreview) {
            helper("toolmenu-ledicon-layoutshape", true); //enable
            helper("toolmenu-ledicon-icondensity", true); //enable
            helper("toolmenu-ledicon-rotation", true); //enable

            helper("toolmenu-ledicon-iconsize", true); //enable
            //do not affect the load file button
            helper("toolmenu-ledicon-addlayout", true); //enable
            helper("toolmenu-ledicon-manplace", false); //disable
            helperStepper("toolmenu-ledicon-zone", true);
        }
        else {
            helper("toolmenu-ledicon-layoutshape", false); //disable
            helper("toolmenu-ledicon-icondensity", false); //disable
            helper("toolmenu-ledicon-iconsize", false); //disable
            helper("toolmenu-ledicon-rotation", false); //disable

            //do not affect the load file button
            helper("toolmenu-ledicon-dirval", false);  //disable
            helper("toolmenu-ledicon-addlayout", false);  //disable
            helper("toolmenu-ledicon-loadfile", false);  //disable
            helper("toolmenu-ledicon-createmap", false);  //disable

        }

        if (stateUX.layoutManualEnable) {
            helper("toolmenu-ledicon-advplace", false);
            helperStepper("toolmenu-ledicon-zone", true);
        }
        else {
            helper("toolmenu-ledicon-advplace", true);
        }

        helper("toolmenu-ledicon-numofpixels", false);
    }
    else {
        helper("toolmenu-ledicon-addlayout", false);
        helper("toolmenu-ledicon-resetlayout", false);
        helper("toolmenu-ledicon-manplace", false);
        helper("toolmenu-ledicon-advplace", false);
        helper("toolmenu-ledicon-advnumofpixels", false);

        //added
        helper("toolmenu-zones-clear", false);
        helperStepper("toolmenu-zones-view", false);//doesn't disable buttons but whatever
        helper("toolmenu-zones-enableassignval", false);
        helperStepper("toolmenu-zones-assign", false); //doesn't disable buttons but whatever
        helper("toolmenu-zones-merge", false);

        helper("toolmenu-ledicon-reset", true);

        //helper("toolmenu-ledicon-loadfile", false);
        helper("toolmenu-ledicon-layoutshape", true);
        helper("toolmenu-ledicon-icondensity", true);
        helper("toolmenu-ledicon-rotation", true); //enable
        helper("toolmenu-ledicon-iconsize", true);
        helper("toolmenu-ledicon-numofpixels", true);
        helperStepper("toolmenu-ledicon-zone", false);
    }

    //-----------------------------------------------------------------------------------------

    if (mainObj.layout.imageEnabled == true) {
        document.getElementById("toolmenu-graphicimage-enable").checked = mainObj.layout.imageEnabled;

        //dumb but was not working otherwise
        try {
            document.getElementById("toolmenu-graphicimage-filename").innerText = path.parse(mainObj.layout.image.filePath).base;
        }
        catch (e) {
            document.getElementById("toolmenu-graphicimage-filename").innerText = "-";
        }

        helper("toolmenu-graphicimage-load", true);
        helper("toolmenu-graphicimage-reset", true);
        helper("toolmenu-graphicimage-positioning", true);
    }
    else {
        document.getElementById("ws-graphic-image-container").style.backgroundImage = "unset"; //ensure container has no bg image persisting. guess do this here
        document.getElementById("toolmenu-graphicimage-filename").innerText = "(disabled)";
        document.getElementById("toolmenu-graphicimage-enable").checked = mainObj.layout.imageEnabled;
        helper("toolmenu-graphicimage-load", false); //false is disabled
        helper("toolmenu-graphicimage-reset", false); //false is disabled
        helper("toolmenu-graphicimage-positioning", false); //false is disabled
    }
    //-----------------------------------------------------------------------------------------

    if (mainObj.layout.gridEnabled == true) document.getElementById("ws-graphic-layout-container").classList.add('bg-grid-100');
    else document.getElementById("ws-graphic-layout-container").classList.remove('bg-grid-100');

    //-----------------------------------------------------------------------------------------

    function helperStepper(elem, state) {
        let buttons = document.getElementById(elem).querySelectorAll("button, input");
        buttons.forEach(function (val, index) {
            if (state) {
                buttons[index].removeAttribute('disabled');
            }
            else {
                buttons[index].setAttribute('disabled', true);
            }
        });
    }

    function helper(elem, state) {
        if (state) {
            document.getElementById(elem).removeAttribute('disabled');
        }
        else {
            document.getElementById(elem).setAttribute('disabled', true);
        }
    }
}

//========================================================================================================================================

function uxZoneViewLabel() {
    //console.log("uxGetZoneViewLabel() ", mainObj.layout.ledIconZoneID);
    let view = (stateUX.zoneView == 0) ? 'all' : stateUX.zoneView;
    document.getElementById("ws-graphic-zonemessage").innerHTML =
        `<p>&nbsp;Zone View:<br>&nbsp;${view} of ${(mainObj.layout.ledIconZoneID.length == 0) ? 0 : Math.max(...mainObj.layout.ledIconZoneID)}</p>`;
}

//========================================================================================================================================

//really do not like this way....... it is better now.....
function uxDisableEnabledTools(flag) {
    console.log("uxDisableEnabledTools()");

    //graphic layout image positioning
    if (document.getElementById('toolmenu-graphicimage-positioning').checked) layoutImagePosition(false);

    //graphic layout manual icon placment
    if (stateUX.layoutManualEnable) layoutManualPlacementEnable(false); //handles event listener

    //graphic layout generate tool
    if (document.getElementById('toolmenu-effects-graphgenenableval').checked) toolGenerateCheckbox(false);

    //manual paint enable
    if (stateUX.toolPaintEnable) toolPaintManualCheckbox(false);

    //zone assignment enable
    if (stateUX.toolZoneAssign) layoutZoneAssignment(false);

    //end shape layout centering if enabled
    if (stateUX.shapeCenterEnable) layoutShapeCenterDragEnd();

    //advanced shape layout preview
    if (stateUX.layoutAdvancedPreview) layoutAdvancedPreviewCheckBox(false);

    //software preview disable - although it will get stopped if any clicks are detected
    if (stateUX.softwarePrevPlay) toolPlaybackPreview('stop');
}

//========================================================================================================================================

function uxDisableTransitionEffects() {

    //.zero-width
    //.show-width 
    //.show-height 
    //.zero-width 
    //.tl-row-indicator
    //.popout-menu-hidden
    //.ovmenu-sequence-listing-deleted

}