
const DEVELOPMENTFLAG = true;

//========================================================================================================================================

"use strict";

const win = {
	setSize(argW, argH) {
		window.electronAPI.uxSetWindowSize(argW, argH);
	},
	setMinimumSize(argW, argH) {
		window.electronAPI.uxSetWindowMinSize(argW, argH);

	},
	getSize() {
		return window.electronAPI.uxGetWindowSize();
	},
	getContentSize() {
		return window.electronAPI.uxGetContentSize();
	},
	reload() {  //development only
		window.electronAPI.spCloseAll();
		setTimeout(window.electronAPI.uxReload, 1000)
		// window.electronAPI.uxReload();
	}
}

const path = {
	join(...paths) {
		return window.electronAPI.pathJoin(...paths);
	},
	basename(arg) {
		return window.electronAPI.pathBasename(arg);
	},
	parse(arg) {
		return window.electronAPI.pathParse(arg);
	},
	extname(arg) {
		return window.electronAPI.pathExtname(arg);
	}
}


const process = {
	dir: '',
	cwd() {
		// console.log("cwd()", this.dir)
		return this.dir;
	},
	async getCwd() {
		this.dir = await window.electronAPI.processCWD();
		console.log("cwd()", this.dir)

		initApp(); //Initialization hook - needed for macos?
	},
	log(data) {
		window.electronAPI.processLog(data);
	}
}

window.addEventListener('DOMContentLoaded', function () {
	process.getCwd();
});



const dialog = {
	async showOpenDialog(options) {
		return await window.electronAPI.dialogOpen(options);
	},
	async showSaveDialog(options) {
		return await window.electronAPI.dialogSave(options);
	},
}

const fs = {
	writeFile(filePath, val, err) {
		window.electronAPI.fsWriteFile(filePath, val, err);
	},
	existsSync(filePath) {
		const f = window.electronAPI.fsExistsSync(filePath);
		return f;
	},
	readFileSync(filePath, encoding) {
		//console.log(filePath);
		const f = window.electronAPI.fsReadFileSync(filePath, encoding);
		return f;
	},
	readFile(filePath, cb) {
		window.electronAPI.fsReadFile(filePath, cb);
	},
	createWriteStream(path, options) {
		window.electronAPI.fsCreateWriteStream(path, options)
	},
	writeToLogFile(str) {
		window.electronAPI.fsWrite(str);
	},
	readdirSync(folder) {
		const f = window.electronAPI.fsReaddirSync(folder);
		return f;
	}
}

const webFrame = {
	setZoomFactor(arg) {
		window.electronAPI.uxSetZoomFactor(arg);
	}
}

//class for device/port
class serialPortBridge {
	constructor(argOptions, argCB) {
		console.log("serialPortBridge: ", argOptions.path)
		console.log(argOptions)
		window.electronAPI.spNew(argOptions, argCB);

		this.isOpen = false;
		this.path = '?';
		this.baudRate = '?'
	}

	write(data,cb) {
		//console.log("WRITE: ",data);
		window.electronAPI.spWrite(data,cb);
	}

	on(id, cb) {
		// console.log("Serialport on", id)
		window.electronAPI.spOn(id, cb);
	}

	flush() {
		//console.log("FLUSH")
		window.electronAPI.spFlush();
	}

	drain(cb) {
		//console.log("FLUSH")
		window.electronAPI.spDrain(cb);
	}


	async close() {
		const p = await window.electronAPI.spCloseAll();
		console.log("CLOSE RESULT", p)
	}
}

//reference to serialPort library
const serialPort = {
	list() {
		return window.electronAPI.spList();
	},
	// ports() {
	// 	return window.electronAPI.spPorts();
	// }

}

class networkBridge {

	constructor() {
		this.type =  'tcp';
		this.isOpen =  'false';
		this.path =  '';
	  }



	connect(port, ip, cb) {
		console.log("NETWORK CONNECT");
		// window.electronAPI.netConnect(23,'127.0.0.1',cb);
		window.electronAPI.netConnect(port, ip, cb);
	}

	write(data) {
		// console.log("WRITE: ",data);
		window.electronAPI.netWrite(data);
	}

	on(id, cb) {
		window.electronAPI.netOn(id, cb);
	}

	close() {
		window.electronAPI.netCloseAll();
	}

	setTimeout(time) {
		window.electronAPI.netTimeout(time);
	}
	// writable() {
	// 	return window.electronAPI.netWritable();
	// }
}



const shell = {
	openExternal(arg) {
		window.electronAPI.shellOpenExternal(arg);
	},
	openItem(arg) {
		window.electronAPI.shellOpenItem(arg);
	}
}




//========================================================================================================================================

var stateSeq = {
	index: 0, //replaces selectedSequence
	selectedFrame: 0,
	minFrames: 2,
	startValPos: [],
	copyFrame: [],
	progress: 0,
	filePath: ""
}

var stateDevice = {
	hard: {
		idNum: 0,
		hardVer: 0, //not used
		version: 0,
		revision: 0,
		bootTargetID: 0,
		bootVer: 0,
		userIDNum: 0,
	},
	soft: {
		name: "None - Software Emulated",
	}
}


var seqWriteToFileBuf;

var playDataFrame = [];

var mainObj = {
	globalIntensity: 100,
	globalIntensityEnable: false,
	layout: {
		shape: "shapeRound",
		centerX: 0,
		centerX: 0,
		iconAmount: 16,
		iconSize: 25,
		iconDensity: 25,
		iconShape: "round",
		layoutDir: "cw",
		rotation: 0,
		advanced: false,
		imageEnabled: false,
		gridEnabled: false,
	},
	userIndex: [],
	indexedMemory: 0,
	idleSeqID: 0,
	seq: [],
	description: "Describe what sequence file is for, versioning, client name, etc"
}

const seqTemplate = {
	name: "template",
	description: "description of sequence",
	mode: "mFade",
	filePath: "none",
	speed: 1,
	numPixels: 1,
	colorMode: "RGB",
	transition: "transInstant",
	playMode: "playLoop",
	intensity: 100,
	generateSteps: 1,
	errorFlag: false,
	memory: 0,
	frames: 2,
	startVals: [],
	frameVar: [],
	dataFrames: [],


	/*
	packetclone: 0,
	povGridSpeed: 0,
	povFramesAmt: 0, //not sure what this is yet, will dig into it when I do POVs
	povVideoMode: 0,
	*/
	/*
	startVals: new Uint8Array(),
	frameVar: new Uint8Array(),
	dataFrames: [new Uint8Array(), new Uint8Array()],
	listID: [0, 1, 2, 3, 4],
	listIteration: [44, 65535, 4, 1000, 10],
	listFlags: [135, 2, 1, 0, 7]
	*/
};

var SelSeq = {}; //global reference for the currently selected sequence

var stateCommunication = {
	prevSerialPort: "default",
	serialPort: "",
	baud: "19200",
	networkIP: "192.168.0.1",
	networkPort: 23,
	connected: false,
	liveUpdatePending: false,
	autoHWPVEnable: false,
	autoHWPVTimer: null,
	timeStamp: 0,
	liveControlPacket: [],
}

var userConfirm = {};

var stateSoftware = {
	maxFrames: 65535,
	maxPixels: 1024, //software is limited by pixels, which is 1024/3048/4096
	screenWidth: 1024,
	screeHeight: 768,
	targetSaveFile: "",
	autoConnectEnabled: true,
	autoDownloadEnabled: true,
	autoSequencesLoad: true,
	tooltipsEnabled: true,
	language: "en",
	deviceProfileLoaded: false,

	version: {
		//filled from package.json at startup
	},
	updateVer: {
		date: '',
		major: 0,
		minor: 0,
		patch: 0
	},
	appTitle: "NLED Aurora Magician v",
}

var settingsDefault = {
	software: {
		screenWidth: 1024, screenHeight: 768, previousSaveFile: "none", autoConnectEnabled: true, autoDownloadEnabled: true, tooltipsEnabled: true, language: "en", firstRun: false, logging: true,
	},
	communication: { prevSerialPort: "COM4", networkIP: "192.168.0.1", networkPort: 23, baud: 115200 }
}

var auroraCMD; //make the global command interface

//============================================== GLOBAL CONSTANTS ====================================================

//decides if startvals/frameVar is required per sequence mode - uses .includes()
const cSeqModesStartVals = ['mFade', 'mGradient', 'mTech'];
const cDataSeqModesDataFrames = ['mFade', 'mGradient', 'mInstant', 'mTech'];
const cDataSeqModeGlobal = ['mPOV', 'mFile', 'mScene'];

const cWSOffsetX = 195;
const cWSOffsetY = 95;

const cMaxUndoStack = 20;

//========================================================================================================================================

const cDefaultGammaTable = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2,
	2, 2, 2, 3, 3, 3, 3, 3, 4, 4, 4, 4, 5, 5, 5, 5,
	6, 6, 6, 7, 7, 7, 8, 8, 8, 9, 9, 9, 10, 10, 11, 11,
	11, 12, 12, 13, 13, 13, 14, 14, 15, 15, 16, 16, 17, 17, 18, 18,
	19, 19, 20, 21, 21, 22, 22, 23, 23, 24, 25, 25, 26, 27, 27, 28,
	29, 29, 30, 31, 31, 32, 33, 34, 34, 35, 36, 37, 37, 38, 39, 40,
	40, 41, 42, 43, 44, 45, 46, 46, 47, 48, 49, 50, 51, 52, 53, 54,
	55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70,
	71, 72, 73, 74, 76, 77, 78, 79, 80, 81, 83, 84, 85, 86, 88, 89,
	90, 91, 93, 94, 95, 96, 98, 99, 100, 102, 103, 104, 106, 107, 109, 110,
	111, 113, 114, 116, 117, 119, 120, 121, 123, 124, 126, 128, 129, 131, 132, 134,
	135, 137, 138, 140, 142, 143, 145, 146, 148, 150, 151, 153, 155, 157, 158, 160,
	162, 163, 165, 167, 169, 170, 172, 174, 176, 178, 179, 181, 183, 185, 187, 189,
	191, 193, 194, 196, 198, 200, 202, 204, 206, 208, 210, 212, 214, 216, 218, 220,
	222, 224, 227, 229, 231, 233, 235, 237, 239, 241, 244, 246, 248, 250, 252, 255];

var gammaTables = {
	red: [],
	green: [],
	blue: [],
	white: []
}

//========================================================================================================================================

var cCompatibilityList = [43, 44, 60, 61, 108, 10, 11]; //Currently unsupported devices

//========================================================================================================================================

